@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -28,14 +30,21 @@
 */
 
 /// \file action_request.cc
+#define GRB_LOG_FAC GrbLogFacility::requests
 
 #include "action_request.h" // API
 
-#include "upnp_xml.h"
+#include "exceptions.h"
+#include "upnp/quirks.h"
+#include "upnp/upnp_common.h"
+#include "upnp/xml_builder.h"
 #include "util/grb_net.h"
-#include "util/upnp_quirks.h"
+#include "util/logger.h"
+#include "util/tools.h"
 
-ActionRequest::ActionRequest(const UpnpXMLBuilder& xmlBuilder, const ClientManager& clients, UpnpActionRequest* upnpRequest)
+#include <cstring>
+
+ActionRequest::ActionRequest(std::shared_ptr<UpnpXMLBuilder> xmlBuilder, std::shared_ptr<ClientManager> clients, UpnpActionRequest* upnpRequest)
     : upnp_request(upnpRequest)
     , actionName(UpnpActionRequest_get_ActionName_cstr(upnpRequest))
     , UDN(UpnpActionRequest_get_DevUDN_cstr(upnpRequest))
@@ -43,7 +52,7 @@ ActionRequest::ActionRequest(const UpnpXMLBuilder& xmlBuilder, const ClientManag
 {
     auto ctrlPtIPAddr = std::make_shared<GrbNet>(UpnpActionRequest_get_CtrlPtIPAddr(upnpRequest));
     std::string userAgent = UpnpActionRequest_get_Os_cstr(upnpRequest);
-    quirks = std::make_unique<Quirks>(xmlBuilder, clients, ctrlPtIPAddr, userAgent);
+    quirks = std::make_shared<Quirks>(std::move(xmlBuilder), clients, ctrlPtIPAddr, userAgent);
 }
 
 std::string ActionRequest::getActionName() const
@@ -61,7 +70,7 @@ std::string ActionRequest::getServiceID() const
     return serviceID;
 }
 
-const std::unique_ptr<Quirks>& ActionRequest::getQuirks() const
+const std::shared_ptr<Quirks>& ActionRequest::getQuirks() const
 {
     return quirks;
 }
@@ -78,7 +87,7 @@ std::unique_ptr<pugi::xml_document> ActionRequest::getRequest() const
 #endif
 
     if (ret.status != pugi::xml_parse_status::status_ok)
-        throw_std_runtime_error("Unable to parse ixml");
+        throw_std_runtime_error("Unable to parse ixml: {}", ret.description());
 
     return request;
 }
@@ -97,7 +106,7 @@ void ActionRequest::update()
 {
     if (response) {
         std::string xml = UpnpXMLBuilder::printXml(*response, "", 0);
-        log_debug("ActionRequest::update(): {}", xml);
+        log_debug("xml: {}", xml);
 
 #if defined(USING_NPUPNP)
         UpnpActionRequest_set_xmlResponse(upnp_request, xml);
@@ -107,7 +116,7 @@ void ActionRequest::update()
         int err = ixmlParseBufferEx(xml.c_str(), &result);
 
         if (err != IXML_SUCCESS) {
-            log_error("ActionRequest::update(): could not convert to iXML");
+            log_error("ActionRequest::update(): could not convert to iXML, code {}", err);
             UpnpActionRequest_set_ErrCode(upnp_request, UPNP_E_ACTION_FAILED);
             if (result)
                 ixmlDocument_free(result);
