@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -37,8 +39,6 @@
 #include <pugixml.hpp>
 #include <upnp.h>
 
-#include "common.h"
-
 // forward declaration
 class UpnpXMLBuilder;
 class Quirks;
@@ -74,7 +74,7 @@ protected:
     std::string serviceID;
 
     /// \brief Client quirks
-    std::unique_ptr<Quirks> quirks;
+    std::shared_ptr<Quirks> quirks;
 
     /// \brief XML holding the response, we fill it in.
     ///
@@ -83,8 +83,10 @@ protected:
 
 public:
     /// \brief The Constructor takes the values from the upnp_request and fills in internal variables.
-    /// \param *upnp_request Pointer to the Upnp_Action_Request structure.
-    explicit ActionRequest(const UpnpXMLBuilder& xmlBuilder, const ClientManager& clients, UpnpActionRequest* upnpRequest);
+    /// \param xmlBuilder builder for xml
+    /// \param clients list of all clients
+    /// \param upnpRequest Pointer to the Upnp_Action_Request structure.
+    ActionRequest(std::shared_ptr<UpnpXMLBuilder> xmlBuilder, std::shared_ptr<ClientManager> clients, UpnpActionRequest* upnpRequest);
     ~ActionRequest() = default;
 
     /// \brief Returns the name of the action.
@@ -100,7 +102,7 @@ public:
     std::unique_ptr<pugi::xml_document> getRequest() const;
 
     /// \brief Returns the client quirks
-    const std::unique_ptr<Quirks>& getQuirks() const;
+    const std::shared_ptr<Quirks>& getQuirks() const;
 
     /// \brief Sets the response (XML created outside as the answer to the request)
     /// \param response XML holding the action response.
