@@ -4,7 +4,7 @@
 
     config_definition.h - this file is part of Gerbera.
 
-    Copyright (C) 2020-2022 Gerbera Contributors
+    Copyright (C) 2020-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -27,10 +27,40 @@
 #ifndef __CONFIG_DEFINITION_H__
 #define __CONFIG_DEFINITION_H__
 
-#include "common.h"
 #include "config.h"
+#include "exceptions.h"
 #include "util/tools.h"
 
+#include <fmt/format.h>
+
+#define CONFIG_XML_VERSION 2
+
+// default values
+#define DEFAULT_FILESYSTEM_CHARSET "UTF-8"
+#define DEFAULT_FALLBACK_CHARSET "US-ASCII"
+
+#define DEFAULT_CONFIG_NAME "config.xml"
+#define DEFAULT_ACCOUNT_USER "gerbera"
+#define DEFAULT_ACCOUNT_PASSWORD "gerbera"
+#define ALIVE_INTERVAL_MIN 62 // seconds
+#define DEFAULT_WEB_DIR "web"
+#define DEFAULT_JS_DIR "js"
+
+#define GRB_UDN_AUTO "grb_udn_auto"
+
+#define DEFAULT_MARK_PLAYED_ITEMS_STRING_MODE "prepend"
+
+#define DEFAULT_AUDIO_BUFFER_SIZE 1048576
+#define DEFAULT_AUDIO_CHUNK_SIZE 131072
+#define DEFAULT_AUDIO_FILL_SIZE 262144
+
+#define DEFAULT_VIDEO_BUFFER_SIZE 14400000
+#define DEFAULT_VIDEO_CHUNK_SIZE 512000
+#define DEFAULT_VIDEO_FILL_SIZE 120000
+
+// device description defaults
+#define DESC_MANUFACTURER_URL "https://gerbera.io/"
+
 class ConfigSetup;
 
 class ConfigDefinition {
@@ -42,11 +72,11 @@ public:
         return complexOptions;
     }
 
-    static const char* mapConfigOption(config_option_t option);
+    static const char* mapConfigOption(ConfigVal option);
     /// \brief check whether option is only available when other option is set
-    static bool isDependent(config_option_t option);
+    static bool isDependent(ConfigVal option);
 
-    static std::shared_ptr<ConfigSetup> findConfigSetup(config_option_t option, bool save = false);
+    static std::shared_ptr<ConfigSetup> findConfigSetup(ConfigVal option, bool save = false);
     static std::shared_ptr<ConfigSetup> findConfigSetupByPath(const std::string& key, bool save = false, const std::shared_ptr<ConfigSetup>& parent = nullptr);
     template <class CS>
     static std::vector<std::shared_ptr<CS>> getConfigSetupList()
@@ -62,7 +92,7 @@ public:
     }
 
     template <class CS>
-    static std::shared_ptr<CS> findConfigSetup(config_option_t option, bool save = false)
+    static std::shared_ptr<CS> findConfigSetup(ConfigVal option, bool save = false)
     {
         auto base = findConfigSetup(option, save);
         if (!base && save)
@@ -75,14 +105,14 @@ public:
         return result;
     }
 
-    static std::string ensureAttribute(config_option_t option, bool check = true)
+    static std::string ensureAttribute(ConfigVal option, bool check = true)
     {
         auto attr = std::string(mapConfigOption(option));
         if (!startswith(attr, ATTRIBUTE) && check)
             return fmt::format("{}{}", ATTRIBUTE, attr);
         return attr;
     }
-    static std::string removeAttribute(config_option_t option)
+    static std::string removeAttribute(ConfigVal option)
     {
         auto attr = std::string(mapConfigOption(option));
         if (attr.size() > ATTRIBUTE.size() && startswith(attr, ATTRIBUTE))
@@ -94,9 +124,9 @@ private:
     /// \brief all known options
     static const std::vector<std::shared_ptr<ConfigSetup>> complexOptions;
     /// \brief parent options for path search
-    static const std::map<config_option_t, std::vector<config_option_t>> parentOptions;
+    static const std::map<ConfigVal, std::vector<ConfigVal>> parentOptions;
     /// \brief option dependencies for automatic loading
-    static const std::map<config_option_t, config_option_t> dependencyMap;
+    static const std::map<ConfigVal, ConfigVal> dependencyMap;
 };
 
 #endif // __CONFIG_DEFINITION_H__
