@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -28,10 +30,25 @@
 */
 
 /// \file config_manager.cc
+#define GRB_LOG_FAC GrbLogFacility::config
 
 #include "config_manager.h" // API
 
-#include <filesystem>
+#include "config/result/autoscan.h"
+#include "config/result/box_layout.h"
+#include "config/result/client_config.h"
+#include "config/result/transcoding.h"
+#include "config/setup/config_setup_boxlayout.h"
+#include "config/setup/config_setup_path.h"
+#include "config_definition.h"
+#include "config_option_enum.h"
+#include "config_options.h"
+#include "config_setup.h"
+#include "config_val.h"
+#include "database/database.h"
+#include "util/string_converter.h"
+#include "util/tools.h"
+
 #include <numeric>
 #include <sstream>
 
@@ -39,42 +56,26 @@
 #include <langinfo.h>
 #endif
 
-#include "client_config.h"
-#include "config_definition.h"
-#include "config_options.h"
-#include "config_setup.h"
-#include "database/database.h"
-#include "transcoding/transcoding.h"
-#include "util/string_converter.h"
-#include "util/tools.h"
-
-bool ConfigManager::debug = false;
-
 ConfigManager::ConfigManager(fs::path filename,
     const fs::path& userHome, const fs::path& configDir,
-    fs::path dataDir, fs::path magicFile,
-    std::string ip, std::string interface, in_port_t port,
-    bool debug)
+    fs::path dataDir, bool debug)
     : filename(std::move(filename))
     , dataDir(std::move(dataDir))
-    , magicFile(std::move(magicFile))
-    , ip(std::move(ip))
-    , interface(std::move(interface))
-    , port(port)
 {
-    ConfigManager::debug = debug;
+    options = std::vector<std::shared_ptr<ConfigOption>>(to_underlying(ConfigVal::MAX));
+    GrbLogger::Logger.setDebugLogging(debug);
 
     if (this->filename.empty()) {
         // No config file path provided, so lets find one.
         fs::path home = userHome / configDir;
         this->filename = home / DEFAULT_CONFIG_NAME;
     }
-
+    this->filename = fs::weakly_canonical(this->filename);
     std::error_code ec;
     if (!isRegularFile(this->filename, ec)) {
         std::ostringstream expErrMsg;
         expErrMsg << "\nThe server configuration file could not be found: ";
-        expErrMsg << this->filename << "\n";
+        expErrMsg << this->filename.string() << "\n";
         expErrMsg << "Gerbera could not find a default configuration file.\n";
         expErrMsg << "Try specifying an alternative configuration file on the command line.\n";
         expErrMsg << "For a list of options run: gerbera -h\n";
@@ -88,18 +89,18 @@ std::shared_ptr<Config> ConfigManager::getSelf()
     return shared_from_this();
 }
 
-std::shared_ptr<ConfigOption> ConfigManager::setOption(const pugi::xml_node& root, config_option_t option, const std::map<std::string, std::string>* arguments)
+std::shared_ptr<ConfigOption> ConfigManager::setOption(const pugi::xml_node& root, ConfigVal option, const std::map<std::string, std::string>* arguments)
 {
     auto co = ConfigDefinition::findConfigSetup(option);
     auto self = getSelf();
     co->makeOption(root, self, arguments);
-    log_debug("Config: option set: '{}'", co->xpath);
+    log_debug("Config: option set: '{}' = '{}'", co->xpath, co->getCurrentValue());
     return co->getValue();
 }
 
-void ConfigManager::addOption(config_option_t option, const std::shared_ptr<ConfigOption>& optionValue)
+void ConfigManager::addOption(ConfigVal option, const std::shared_ptr<ConfigOption>& optionValue)
 {
-    options.at(option) = optionValue;
+    options.at(to_underlying(option)) = optionValue;
 }
 
 void ConfigManager::load(const fs::path& userHome)
@@ -107,13 +108,13 @@ void ConfigManager::load(const fs::path& userHome)
     std::map<std::string, std::string> args;
     auto self = getSelf();
 
-    log_info("Loading configuration from: {}", filename.c_str());
+    log_info("Loading configuration from: {}", filename.string());
     pugi::xml_parse_result result = xmlDoc->load_file(filename.c_str());
     if (result.status != pugi::xml_parse_status::status_ok) {
         throw ConfigParseException(result.description());
     }
 
-    log_info("Checking configuration...");
+    log_info("Parsing configuration...");
 
     auto root = xmlDoc->document_element();
 
@@ -131,109 +132,97 @@ void ConfigManager::load(const fs::path& userHome)
 
     // now go through the mandatory parameters,
     // if something is missing we will not start the server
-    auto co = ConfigDefinition::findConfigSetup(CFG_SERVER_HOME);
+    auto co = ConfigDefinition::findConfigSetup(ConfigVal::SERVER_HOME);
     // respect command line if available; ignore xml value
     {
-        std::string temp = !userHome.empty() ? userHome.string() : co->getXmlContent(root);
-        if (!fs::is_directory(temp))
-            throw_std_runtime_error("Directory '{}' does not exist", temp);
-        co->makeOption(temp, self);
-        ConfigPathSetup::Home = temp;
+        std::string activeHome = userHome.string();
+        bool homeOverride = setOption(root, ConfigVal::SERVER_HOME_OVERRIDE)->getBoolOption();
+        if (userHome.empty() || homeOverride) {
+            activeHome = co->getXmlContent(root);
+        } else {
+            if (!fs::is_directory(activeHome))
+                throw_std_runtime_error("Directory '{}' does not exist", activeHome);
+            log_info("Using home directory '{}'", activeHome);
+        }
+        co->makeOption(activeHome, self);
+        ConfigPathSetup::Home = activeHome;
     }
 
     // checking database driver options
-    bool mysqlEn = false;
+    [[maybe_unused]] bool mysqlEn = false;
     bool sqlite3En = false;
 
-    co = ConfigDefinition::findConfigSetup(CFG_SERVER_STORAGE);
+    co = ConfigDefinition::findConfigSetup(ConfigVal::SERVER_STORAGE);
     co->getXmlElement(root); // fails if missing
-    setOption(root, CFG_SERVER_STORAGE_USE_TRANSACTIONS);
+    setOption(root, ConfigVal::SERVER_STORAGE_USE_TRANSACTIONS);
 
-    co = ConfigDefinition::findConfigSetup(CFG_SERVER_STORAGE_MYSQL);
+    co = ConfigDefinition::findConfigSetup(ConfigVal::SERVER_STORAGE_MYSQL);
     if (co->hasXmlElement(root)) {
-        mysqlEn = setOption(root, CFG_SERVER_STORAGE_MYSQL_ENABLED)->getBoolOption();
+        mysqlEn = setOption(root, ConfigVal::SERVER_STORAGE_MYSQL_ENABLED)->getBoolOption();
     }
 
-    co = ConfigDefinition::findConfigSetup(CFG_SERVER_STORAGE_SQLITE);
+    co = ConfigDefinition::findConfigSetup(ConfigVal::SERVER_STORAGE_SQLITE);
     if (co->hasXmlElement(root)) {
-        sqlite3En = setOption(root, CFG_SERVER_STORAGE_SQLITE_ENABLED)->getBoolOption();
+        sqlite3En = setOption(root, ConfigVal::SERVER_STORAGE_SQLITE_ENABLED)->getBoolOption();
     }
 
-    if (sqlite3En && mysqlEn)
-        throw_std_runtime_error("You enabled both, sqlite3 and mysql but "
-                                "only one database driver may be active at a time");
-
-    if (!sqlite3En && !mysqlEn)
-        throw_std_runtime_error("You disabled both sqlite3 and mysql but "
-                                "one database driver must be active");
-
     std::string dbDriver;
 
 #ifdef HAVE_MYSQL
     if (mysqlEn) {
         // read mysql options
-        setOption(root, CFG_SERVER_STORAGE_MYSQL_HOST);
-        setOption(root, CFG_SERVER_STORAGE_MYSQL_DATABASE);
-        setOption(root, CFG_SERVER_STORAGE_MYSQL_USERNAME);
-        setOption(root, CFG_SERVER_STORAGE_MYSQL_PORT);
-        setOption(root, CFG_SERVER_STORAGE_MYSQL_SOCKET);
-        setOption(root, CFG_SERVER_STORAGE_MYSQL_PASSWORD);
+        setOption(root, ConfigVal::SERVER_STORAGE_MYSQL_HOST);
+        setOption(root, ConfigVal::SERVER_STORAGE_MYSQL_DATABASE);
+        setOption(root, ConfigVal::SERVER_STORAGE_MYSQL_USERNAME);
+        setOption(root, ConfigVal::SERVER_STORAGE_MYSQL_PORT);
+        setOption(root, ConfigVal::SERVER_STORAGE_MYSQL_SOCKET);
+        setOption(root, ConfigVal::SERVER_STORAGE_MYSQL_PASSWORD);
 
-        co = ConfigDefinition::findConfigSetup(CFG_SERVER_STORAGE_MYSQL_INIT_SQL_FILE);
+        co = ConfigDefinition::findConfigSetup(ConfigVal::SERVER_STORAGE_MYSQL_INIT_SQL_FILE);
         co->setDefaultValue(dataDir / "mysql.sql");
         co->makeOption(root, self);
-        co = ConfigDefinition::findConfigSetup(CFG_SERVER_STORAGE_MYSQL_UPGRADE_FILE);
+        co = ConfigDefinition::findConfigSetup(ConfigVal::SERVER_STORAGE_MYSQL_UPGRADE_FILE);
         co->setDefaultValue(dataDir / "mysql-upgrade.xml");
         co->makeOption(root, self);
         dbDriver = "mysql";
     }
-#else
-    if (mysqlEn) {
-        throw_std_runtime_error("You enabled MySQL database in configuration, "
-                                "however this version of Gerbera was compiled "
-                                "without MySQL support!");
-    }
 #endif // HAVE_MYSQL
 
     if (sqlite3En) {
         // read sqlite options
-        setOption(root, CFG_SERVER_STORAGE_SQLITE_DATABASE_FILE);
-        setOption(root, CFG_SERVER_STORAGE_SQLITE_SYNCHRONOUS);
-        setOption(root, CFG_SERVER_STORAGE_SQLITE_RESTORE);
-        setOption(root, CFG_SERVER_STORAGE_SQLITE_BACKUP_ENABLED);
-        setOption(root, CFG_SERVER_STORAGE_SQLITE_BACKUP_INTERVAL);
+        setOption(root, ConfigVal::SERVER_STORAGE_SQLITE_DATABASE_FILE);
+        setOption(root, ConfigVal::SERVER_STORAGE_SQLITE_SYNCHRONOUS);
+        setOption(root, ConfigVal::SERVER_STORAGE_SQLITE_JOURNALMODE);
+        setOption(root, ConfigVal::SERVER_STORAGE_SQLITE_RESTORE);
+        setOption(root, ConfigVal::SERVER_STORAGE_SQLITE_BACKUP_ENABLED);
+        setOption(root, ConfigVal::SERVER_STORAGE_SQLITE_BACKUP_INTERVAL);
 
-        co = ConfigDefinition::findConfigSetup(CFG_SERVER_STORAGE_SQLITE_INIT_SQL_FILE);
+        co = ConfigDefinition::findConfigSetup(ConfigVal::SERVER_STORAGE_SQLITE_INIT_SQL_FILE);
         co->setDefaultValue(dataDir / "sqlite3.sql");
         co->makeOption(root, self);
-        co = ConfigDefinition::findConfigSetup(CFG_SERVER_STORAGE_SQLITE_UPGRADE_FILE);
+        co = ConfigDefinition::findConfigSetup(ConfigVal::SERVER_STORAGE_SQLITE_UPGRADE_FILE);
         co->setDefaultValue(dataDir / "sqlite3-upgrade.xml");
         co->makeOption(root, self);
         dbDriver = "sqlite3";
     }
 
-    co = ConfigDefinition::findConfigSetup(CFG_SERVER_STORAGE_DRIVER);
+    co = ConfigDefinition::findConfigSetup(ConfigVal::SERVER_STORAGE_DRIVER);
     co->makeOption(dbDriver, self);
 
-    // now go through the optional settings and fix them if anything is missing
-    auto defIpp = setOption(root, CFG_SERVER_UI_DEFAULT_ITEMS_PER_PAGE)->getIntOption();
+    bool multiValue = setOption(root, ConfigVal::UPNP_MULTI_VALUES_ENABLED)->getBoolOption();
+    co = ConfigDefinition::findConfigSetup(ConfigVal::A_CLIENTS_UPNP_MULTI_VALUE);
+    co->setDefaultValue(multiValue ? YES : NO);
 
-    // now get the option list for the drop-down menu
-    auto menuOpts = setOption(root, CFG_SERVER_UI_ITEMS_PER_PAGE_DROPDOWN)->getArrayOption();
-    if (std::find(menuOpts.begin(), menuOpts.end(), fmt::to_string(defIpp)) == menuOpts.end())
-        throw_std_runtime_error("Error in config file: at least one <option> "
-                                "under <items-per-page> must match the "
-                                "<items-per-page default=\"\" /> attribute");
-
-    bool clEn = setOption(root, CFG_CLIENTS_LIST_ENABLED)->getBoolOption();
+    bool clEn = setOption(root, ConfigVal::CLIENTS_LIST_ENABLED)->getBoolOption();
     args["isEnabled"] = clEn ? "true" : "false";
-    setOption(root, CFG_CLIENTS_LIST, &args);
+    setOption(root, ConfigVal::CLIENTS_LIST, &args);
     args.clear();
 
-    setOption(root, CFG_IMPORT_HIDDEN_FILES);
-    bool csens = setOption(root, CFG_IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_CASE_SENSITIVE)->getBoolOption();
+    setOption(root, ConfigVal::IMPORT_HIDDEN_FILES);
+    setOption(root, ConfigVal::IMPORT_FOLLOW_SYMLINKS);
+    bool csens = setOption(root, ConfigVal::IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_CASE_SENSITIVE)->getBoolOption();
     args["tolower"] = fmt::to_string(!csens);
-    setOption(root, CFG_IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_LIST, &args);
+    setOption(root, ConfigVal::IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_LIST, &args);
     args.clear();
 
     std::string defaultCharSet = DEFAULT_FILESYSTEM_CHARSET;
@@ -244,7 +233,7 @@ void ConfigManager::load(const fs::path& userHome)
     }
 #endif
     // check if the one we take as default is actually available
-    co = ConfigDefinition::findConfigSetup(CFG_IMPORT_FILESYSTEM_CHARSET);
+    co = ConfigDefinition::findConfigSetup(ConfigVal::IMPORT_FILESYSTEM_CHARSET);
     try {
         auto conv = std::make_unique<StringConverter>(defaultCharSet, DEFAULT_INTERNAL_CHARSET);
     } catch (const std::runtime_error&) {
@@ -252,209 +241,177 @@ void ConfigManager::load(const fs::path& userHome)
     }
     co->setDefaultValue(defaultCharSet);
 
-    std::string charset = co->getXmlContent(root);
-    try {
-        auto conv = std::make_unique<StringConverter>(charset, DEFAULT_INTERNAL_CHARSET);
-    } catch (const std::runtime_error& e) {
-        throw_std_runtime_error("Error in config file: unsupported filesystem-charset specified: {}\n{}", charset, e.what());
-    }
-    log_debug("Setting filesystem import charset to {}", charset);
-    co->makeOption(charset, self);
-
-    co = ConfigDefinition::findConfigSetup(CFG_IMPORT_METADATA_CHARSET);
+    co = ConfigDefinition::findConfigSetup(ConfigVal::IMPORT_METADATA_CHARSET);
     co->setDefaultValue(defaultCharSet);
-    charset = co->getXmlContent(root);
-    try {
-        auto conv = std::make_unique<StringConverter>(charset, DEFAULT_INTERNAL_CHARSET);
-    } catch (const std::runtime_error& e) {
-        throw_std_runtime_error("Error in config file: unsupported metadata-charset specified: {}\n{}", charset, e.what());
-    }
-    log_debug("Setting metadata import charset to {}", charset);
-    co->makeOption(charset, self);
 
-    // read playlist options
-    co = ConfigDefinition::findConfigSetup(CFG_IMPORT_PLAYLIST_CHARSET);
+    co = ConfigDefinition::findConfigSetup(ConfigVal::IMPORT_PLAYLIST_CHARSET);
     co->setDefaultValue(defaultCharSet);
-    charset = co->getXmlContent(root);
-    try {
-        auto conv = std::make_unique<StringConverter>(charset, DEFAULT_INTERNAL_CHARSET);
-    } catch (const std::runtime_error& e) {
-        throw_std_runtime_error("Error in config file: unsupported playlist-charset specified: {}\n{}", charset, e.what());
-    }
-    log_debug("Setting playlist charset to {}", charset);
-    co->makeOption(charset, self);
 
-    // read network options
-    co = ConfigDefinition::findConfigSetup(CFG_SERVER_NETWORK_INTERFACE);
-    co->makeOption((interface.empty()) ? co->getXmlContent(root) : interface, self);
+    co = ConfigDefinition::findConfigSetup(ConfigVal::A_DIRECTORIES_TWEAK_META_CHARSET);
+    co->setDefaultValue(defaultCharSet);
 
-    co = ConfigDefinition::findConfigSetup(CFG_SERVER_IP);
-    // bind to any IP address
-    co->makeOption(ip.empty() ? co->getXmlContent(root) : ip, self);
-
-    if (!getOption(CFG_SERVER_NETWORK_INTERFACE).empty() && !getOption(CFG_SERVER_IP).empty())
-        throw_std_runtime_error("Error in config file: you can not specify interface and ip at the same time");
-
-    // read server options
-    {
-        std::string temp = setOption(root, CFG_SERVER_APPEND_PRESENTATION_URL_TO)->getOption();
-        if ((temp == "ip" || temp == "port") && getOption(CFG_SERVER_PRESENTATION_URL).empty()) {
-            throw_std_runtime_error("Error in config file: \"append-to\" attribute "
-                                    "value in <presentationURL> tag is set to \"{}\""
-                                    "but no URL is specified",
-                temp);
-        }
-    }
 #ifdef HAVE_JS
     // read javascript options
-    co = ConfigDefinition::findConfigSetup(CFG_IMPORT_SCRIPTING_PLAYLIST_SCRIPT);
-    co->setDefaultValue(dataDir / DEFAULT_JS_DIR / DEFAULT_PLAYLISTS_SCRIPT);
+    co = ConfigDefinition::findConfigSetup(ConfigVal::IMPORT_SCRIPTING_COMMON_FOLDER);
+    co->setDefaultValue(dataDir / DEFAULT_JS_DIR);
     co->makeOption(root, self);
 
-    co = ConfigDefinition::findConfigSetup(CFG_IMPORT_SCRIPTING_COMMON_SCRIPT);
-    co->setDefaultValue(dataDir / DEFAULT_JS_DIR / DEFAULT_COMMON_SCRIPT);
-    co->makeOption(root, self);
-
-    co = ConfigDefinition::findConfigSetup(CFG_IMPORT_SCRIPTING_CUSTOM_SCRIPT);
-    args["resolveEmpty"] = "false";
-    co->makeOption(root, self, &args);
-    args.clear();
-#endif
-
-    auto layoutType = setOption(root, CFG_IMPORT_SCRIPTING_VIRTUAL_LAYOUT_TYPE)->getOption();
-
-#ifndef HAVE_JS
-    if (layoutType == "js")
-        throw_std_runtime_error("Gerbera was compiled without JS support, "
-                                "however you specified \"js\" to be used for the "
-                                "virtual-layout.");
-#else
     // read more javascript options
-    charset = setOption(root, CFG_IMPORT_SCRIPTING_CHARSET)->getOption();
-    if (layoutType == "js") {
-        try {
-            auto conv = std::make_unique<StringConverter>(charset, DEFAULT_INTERNAL_CHARSET);
-        } catch (const std::runtime_error& e) {
-            throw_std_runtime_error("Error in config file: unsupported import script charset specified: {}\n{}", charset, e.what());
-        }
-    }
-
-    co = ConfigDefinition::findConfigSetup(CFG_IMPORT_SCRIPTING_IMPORT_SCRIPT);
-    args["mustExist"] = fmt::to_string(layoutType == "js");
-    args["notEmpty"] = fmt::to_string(layoutType == "js");
-    co->setDefaultValue(dataDir / DEFAULT_JS_DIR / DEFAULT_IMPORT_SCRIPT);
-    co->makeOption(root, self, &args);
-    args.clear();
+    co = ConfigDefinition::findConfigSetup(ConfigVal::IMPORT_SCRIPTING_CHARSET);
+    co->setDefaultValue(defaultCharSet);
 #endif
 
-    co = ConfigDefinition::findConfigSetup(CFG_SERVER_PORT);
-    // 0 means, that the SDK will any free port itself
-    co->makeOption((port == 0) ? co->getXmlContent(root) : fmt::to_string(port), self);
-
-    args["hiddenFiles"] = getBoolOption(CFG_IMPORT_HIDDEN_FILES) ? "true" : "false";
-    setOption(root, CFG_IMPORT_AUTOSCAN_TIMED_LIST, &args);
+    args["hiddenFiles"] = getBoolOption(ConfigVal::IMPORT_HIDDEN_FILES) ? "true" : "false";
+    args["followSymlinks"] = getBoolOption(ConfigVal::IMPORT_FOLLOW_SYMLINKS) ? "true" : "false";
+    setOption(root, ConfigVal::IMPORT_AUTOSCAN_TIMED_LIST, &args);
 
 #ifdef HAVE_INOTIFY
-    auto useInotify = setOption(root, CFG_IMPORT_AUTOSCAN_USE_INOTIFY)->getBoolOption();
+    auto useInotify = setOption(root, ConfigVal::IMPORT_AUTOSCAN_USE_INOTIFY)->getBoolOption();
 
     if (useInotify) {
-        setOption(root, CFG_IMPORT_AUTOSCAN_INOTIFY_LIST, &args);
+        setOption(root, ConfigVal::IMPORT_AUTOSCAN_INOTIFY_LIST, &args);
     } else {
-        setOption({}, CFG_IMPORT_AUTOSCAN_INOTIFY_LIST); // set empty list
+        setOption({}, ConfigVal::IMPORT_AUTOSCAN_INOTIFY_LIST); // set empty list
     }
 #endif
     args.clear();
 
     // read transcoding options
-    auto trEn = setOption(root, CFG_TRANSCODING_TRANSCODING_ENABLED)->getBoolOption();
+    auto trEn = setOption(root, ConfigVal::TRANSCODING_TRANSCODING_ENABLED)->getBoolOption();
     args["isEnabled"] = trEn ? "true" : "false";
-    setOption(root, CFG_TRANSCODING_PROFILE_LIST, &args);
+    setOption(root, ConfigVal::TRANSCODING_PROFILE_LIST, &args);
     args.clear();
 
 #ifdef HAVE_CURL
     if (trEn) {
-        setOption(root, CFG_EXTERNAL_TRANSCODING_CURL_BUFFER_SIZE);
-        setOption(root, CFG_EXTERNAL_TRANSCODING_CURL_FILL_SIZE);
+        setOption(root, ConfigVal::EXTERNAL_TRANSCODING_CURL_BUFFER_SIZE);
+        setOption(root, ConfigVal::EXTERNAL_TRANSCODING_CURL_FILL_SIZE);
     }
 #endif // HAVE_CURL
 
     // read import options
     args["trim"] = "false";
-    setOption(root, CFG_IMPORT_LIBOPTS_ENTRY_SEP, &args);
-    setOption(root, CFG_UPNP_SEARCH_SEPARATOR, &args);
-    setOption(root, CFG_IMPORT_LIBOPTS_ENTRY_LEGACY_SEP, &args);
+    setOption(root, ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP, &args);
+    setOption(root, ConfigVal::UPNP_SEARCH_SEPARATOR, &args);
+    setOption(root, ConfigVal::IMPORT_LIBOPTS_ENTRY_LEGACY_SEP, &args);
     args.clear();
 
 #if defined(HAVE_FFMPEG) && defined(HAVE_FFMPEGTHUMBNAILER)
-    auto ffmpEn = setOption(root, CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_ENABLED)->getBoolOption();
+    auto ffmpEn = setOption(root, ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_ENABLED)->getBoolOption();
     if (ffmpEn) {
-        setOption(root, CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_THUMBSIZE);
-        setOption(root, CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_SEEK_PERCENTAGE);
-        setOption(root, CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_FILMSTRIP_OVERLAY);
-        setOption(root, CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_IMAGE_QUALITY);
-        setOption(root, CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_CACHE_DIR_ENABLED);
-        setOption(root, CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_CACHE_DIR);
+        setOption(root, ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_THUMBSIZE);
+        setOption(root, ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_SEEK_PERCENTAGE);
+        setOption(root, ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_FILMSTRIP_OVERLAY);
+        setOption(root, ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_IMAGE_QUALITY);
+        setOption(root, ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_CACHE_DIR_ENABLED);
+        setOption(root, ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_CACHE_DIR);
     }
 #endif
 
-    bool markingEnabled = setOption(root, CFG_SERVER_EXTOPTS_MARK_PLAYED_ITEMS_ENABLED)->getBoolOption();
-    bool contentArrayEmpty = setOption(root, CFG_SERVER_EXTOPTS_MARK_PLAYED_ITEMS_CONTENT_LIST)->getArrayOption().empty();
-    if (markingEnabled && contentArrayEmpty) {
-        throw_std_runtime_error("Error in config file: <mark-played-items>/<mark> tag must contain at least one <content> tag");
-    }
-
 #if defined(HAVE_LASTFMLIB)
-    auto lfmEn = setOption(root, CFG_SERVER_EXTOPTS_LASTFM_ENABLED)->getBoolOption();
+    auto lfmEn = setOption(root, ConfigVal::SERVER_EXTOPTS_LASTFM_ENABLED)->getBoolOption();
     if (lfmEn) {
-        setOption(root, CFG_SERVER_EXTOPTS_LASTFM_USERNAME);
-        setOption(root, CFG_SERVER_EXTOPTS_LASTFM_PASSWORD);
+        setOption(root, ConfigVal::SERVER_EXTOPTS_LASTFM_USERNAME);
+        setOption(root, ConfigVal::SERVER_EXTOPTS_LASTFM_PASSWORD);
     }
 #endif
 
-#ifdef HAVE_MAGIC
-    co = ConfigDefinition::findConfigSetup(CFG_IMPORT_MAGIC_FILE);
-    args["isFile"] = "true";
-    args["resolveEmpty"] = "false";
-    co->makeOption(!magicFile.empty() ? magicFile.string() : co->getXmlContent(root), self, &args);
-    args.clear();
-#endif
+    // read options that do not have special requirement and that are not yet loaded
+    for (auto&& optionKey : ConfigOptionIterator()) {
+        if (!options.at(to_underlying(optionKey)) && !ConfigDefinition::isDependent(optionKey)) {
+            setOption(root, optionKey);
+        }
+    }
+
+    log_info("Configuration load succeeded.");
+
+    std::ostringstream buf;
+    xmlDoc->print(buf, "  ");
+    log_debug("Config file dump after loading: {}", buf.str());
+
+    // now the XML is no longer needed we can destroy it
+    xmlDoc = nullptr;
+}
+
+/// \brief: Validate that correlated options have correct values
+bool ConfigManager::validate()
+{
+    log_info("Validating configuration...");
+    auto self = getSelf();
+
+    if (!getOption(ConfigVal::SERVER_NETWORK_INTERFACE).empty() && !getOption(ConfigVal::SERVER_IP).empty())
+        throw_std_runtime_error("Error in config file: you can not specify interface and ip at the same time");
+
+    // checking database driver options
+    bool mysqlEn = getBoolOption(ConfigVal::SERVER_STORAGE_MYSQL_ENABLED);
+    bool sqlite3En = getBoolOption(ConfigVal::SERVER_STORAGE_SQLITE_ENABLED);
+
+    if (sqlite3En && mysqlEn)
+        throw_std_runtime_error("You enabled both, sqlite3 and mysql but "
+                                "only one database driver may be active at a time");
+
+    if (!sqlite3En && !mysqlEn)
+        throw_std_runtime_error("You disabled both sqlite3 and mysql but "
+                                "one database driver must be active");
+
+#ifndef HAVE_MYSQL
+    if (mysqlEn) {
+        throw_std_runtime_error("You enabled MySQL database in configuration, "
+                                "however this version of Gerbera was compiled "
+                                "without MySQL support!");
+    }
+#endif // HAVE_MYSQL
+
+    auto appendto = EnumOption<UrlAppendMode>::getEnumOption(self, ConfigVal::SERVER_APPEND_PRESENTATION_URL_TO);
+    if ((appendto == UrlAppendMode::ip || appendto == UrlAppendMode::port) && getOption(ConfigVal::SERVER_PRESENTATION_URL).empty()) {
+        throw_std_runtime_error("Error in config file: \"append-to\" attribute "
+                                "value in <presentationURL> tag is set to \"{}\""
+                                "but no URL is specified",
+            appendto);
+    }
 
 #ifdef HAVE_INOTIFY
-    auto configTimedList = getAutoscanListOption(CFG_IMPORT_AUTOSCAN_TIMED_LIST);
-    auto configInotifyList = getAutoscanListOption(CFG_IMPORT_AUTOSCAN_INOTIFY_LIST);
+    auto configTimedList = getAutoscanListOption(ConfigVal::IMPORT_AUTOSCAN_TIMED_LIST);
+    auto configInotifyList = getAutoscanListOption(ConfigVal::IMPORT_AUTOSCAN_INOTIFY_LIST);
 
     for (const auto& iDir : configInotifyList) {
         for (const auto& tDir : configTimedList) {
-            if (iDir.getLocation() == tDir.getLocation())
+            if (iDir->getLocation() == tDir->getLocation())
                 throw_std_runtime_error("Error in config file: same path used in both inotify and timed scan modes");
         }
     }
 #endif
 
-    // read online content options
+    // now go through the optional settings and fix them if anything is missing
+    auto defIpp = getIntOption(ConfigVal::SERVER_UI_DEFAULT_ITEMS_PER_PAGE);
 
-#ifdef ATRAILERS
-    int atrailersRefresh = setOption(root, CFG_ONLINE_CONTENT_ATRAILERS_REFRESH)->getIntOption();
+    // now get the option list for the drop-down menu
+    auto menuOpts = getArrayOption(ConfigVal::SERVER_UI_ITEMS_PER_PAGE_DROPDOWN);
+    if (std::find(menuOpts.begin(), menuOpts.end(), fmt::to_string(defIpp)) == menuOpts.end())
+        throw_std_runtime_error("Error in config file: at least one <option> "
+                                "under <items-per-page> must match the "
+                                "<items-per-page default=\"\" /> attribute");
 
-    co = ConfigDefinition::findConfigSetup(CFG_ONLINE_CONTENT_ATRAILERS_PURGE_AFTER);
-    co->makeOption(fmt::to_string(atrailersRefresh), self);
-#endif
-
-    // read options that do not have special requirement and that are not yet loaded
-    for (auto&& optionKey : ConfigOptionIterator()) {
-        if (!options.at(optionKey) && !ConfigDefinition::isDependent(optionKey)) {
-            setOption(root, optionKey);
-        }
+    bool markingEnabled = getBoolOption(ConfigVal::SERVER_EXTOPTS_MARK_PLAYED_ITEMS_ENABLED);
+    bool contentArrayEmpty = getArrayOption(ConfigVal::SERVER_EXTOPTS_MARK_PLAYED_ITEMS_CONTENT_LIST).empty();
+    if (markingEnabled && contentArrayEmpty) {
+        throw_std_runtime_error("Error in config file: <mark-played-items>/<mark> tag must contain at least one <content> tag");
     }
 
+#ifndef HAVE_JS
+    auto layoutType = EnumOption<LayoutType>::getEnumOption(self, ConfigVal::IMPORT_SCRIPTING_VIRTUAL_LAYOUT_TYPE);
+
+    if (layoutType == LayoutType::Js)
+        throw_std_runtime_error("Gerbera was compiled without JS support, "
+                                "however you specified \"js\" to be used for the "
+                                "virtual-layout.");
+#endif
+    auto co = ConfigDefinition::findConfigSetup<ConfigBoxLayoutSetup>(ConfigVal::BOXLAYOUT_BOX);
+    if (!co->validate(getSelf(), getBoxLayoutListOption(ConfigVal::BOXLAYOUT_BOX)))
+        throw_std_runtime_error("Validation of {} failed", co->xpath);
+
     log_info("Configuration check succeeded.");
-
-    std::ostringstream buf;
-    xmlDoc->print(buf, "  ");
-    log_debug("Config file dump after validation: {}", buf.str());
-
-    // now the XML is no longer needed we can destroy it
-    xmlDoc = nullptr;
+    return true;
 }
 
 void ConfigManager::updateConfigFromDatabase(const std::shared_ptr<Database>& database)
@@ -474,11 +431,12 @@ void ConfigManager::updateConfigFromDatabase(const std::shared_ptr<Database>& da
                     cs->makeOption(cfgValue.value, self);
                 } else {
                     std::string parValue = cfgValue.value;
-                    if (cfgValue.status == STATUS_CHANGED || cfgValue.status == STATUS_UNCHANGED) {
+                    auto statusList = splitString(cfgValue.status, ',');
+                    if (statusList.back() == STATUS_CHANGED || statusList.back() == STATUS_UNCHANGED) {
                         if (!cs->updateDetail(cfgValue.item, parValue, self)) {
                             log_error("unhandled {} option {} != {}", cfgValue.status, cfgValue.item, cs->xpath);
                         }
-                    } else if (cfgValue.status == STATUS_REMOVED || cfgValue.status == STATUS_ADDED || cfgValue.status == STATUS_MANUAL) {
+                    } else if (statusList.back() == STATUS_REMOVED || statusList.back() == STATUS_ADDED || statusList.back() == STATUS_MANUAL) {
                         std::map<std::string, std::string> arguments { { "status", cfgValue.status } };
                         if (!cs->updateDetail(cfgValue.item, parValue, self, &arguments)) {
                             log_error("unhandled {} option {} != {}", cfgValue.status, cfgValue.item, cs->xpath);
@@ -505,105 +463,151 @@ void ConfigManager::setOrigValue(const std::string& item, bool value)
     origValues.try_emplace(item, value ? "true" : "false");
 }
 
-void ConfigManager::setOrigValue(const std::string& item, int value)
+void ConfigManager::setOrigValue(const std::string& item, IntOptionType value)
+{
+    origValues.try_emplace(item, fmt::to_string(value));
+}
+
+void ConfigManager::setOrigValue(const std::string& item, UIntOptionType value)
+{
+    origValues.try_emplace(item, fmt::to_string(value));
+}
+
+void ConfigManager::setOrigValue(const std::string& item, LongOptionType value)
 {
     origValues.try_emplace(item, fmt::to_string(value));
 }
 
 // The validate function ensures that the array is completely filled!
-std::string ConfigManager::getOption(config_option_t option) const
+std::string ConfigManager::getOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
     return optionValue->getOption();
 }
 
-int ConfigManager::getIntOption(config_option_t option) const
+IntOptionType ConfigManager::getIntOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
     return optionValue->getIntOption();
 }
 
-bool ConfigManager::getBoolOption(config_option_t option) const
+UIntOptionType ConfigManager::getUIntOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
+    if (!optionValue) {
+        throw_std_runtime_error("option {} not set", option);
+    }
+    return optionValue->getUIntOption();
+}
+
+LongOptionType ConfigManager::getLongOption(ConfigVal option) const
+{
+    auto optionValue = options.at(to_underlying(option));
+    if (!optionValue) {
+        throw_std_runtime_error("option {} not set", option);
+    }
+    return optionValue->getLongOption();
+}
+
+std::shared_ptr<ConfigOption> ConfigManager::getConfigOption(ConfigVal option) const
+{
+    auto optionValue = options.at(to_underlying(option));
+    if (!optionValue) {
+        throw_std_runtime_error("option {} not set", option);
+    }
+    return optionValue;
+}
+
+bool ConfigManager::getBoolOption(ConfigVal option) const
+{
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
     return optionValue->getBoolOption();
 }
 
-std::map<std::string, std::string> ConfigManager::getDictionaryOption(config_option_t option) const
+std::map<std::string, std::string> ConfigManager::getDictionaryOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
     return optionValue->getDictionaryOption();
 }
 
-std::vector<std::vector<std::pair<std::string, std::string>>> ConfigManager::getVectorOption(config_option_t option) const
+std::vector<std::vector<std::pair<std::string, std::string>>> ConfigManager::getVectorOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
     return optionValue->getVectorOption();
 }
 
-std::vector<std::string> ConfigManager::getArrayOption(config_option_t option) const
+std::vector<std::string> ConfigManager::getArrayOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
     return optionValue->getArrayOption();
 }
 
-std::vector<AutoscanDirectory> ConfigManager::getAutoscanListOption(config_option_t option) const
+std::vector<std::shared_ptr<AutoscanDirectory>> ConfigManager::getAutoscanListOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
     return optionValue->getAutoscanListOption();
 }
 
-std::shared_ptr<ClientConfigList> ConfigManager::getClientConfigListOption(config_option_t option) const
+std::shared_ptr<BoxLayoutList> ConfigManager::getBoxLayoutListOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
+    if (!optionValue) {
+        throw_std_runtime_error("option {} not set", option);
+    }
+    return optionValue->getBoxLayoutListOption();
+}
+
+std::shared_ptr<ClientConfigList> ConfigManager::getClientConfigListOption(ConfigVal option) const
+{
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
     return optionValue->getClientConfigListOption();
 }
 
-std::shared_ptr<DirectoryConfigList> ConfigManager::getDirectoryTweakOption(config_option_t option) const
+std::shared_ptr<DirectoryConfigList> ConfigManager::getDirectoryTweakOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
     return optionValue->getDirectoryTweakOption();
 }
 
-std::shared_ptr<DynamicContentList> ConfigManager::getDynamicContentListOption(config_option_t option) const
+std::shared_ptr<DynamicContentList> ConfigManager::getDynamicContentListOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
     return optionValue->getDynamicContentListOption();
 }
 
-std::shared_ptr<TranscodingProfileList> ConfigManager::getTranscodingProfileListOption(config_option_t option) const
+std::shared_ptr<TranscodingProfileList> ConfigManager::getTranscodingProfileListOption(ConfigVal option) const
 {
-    auto optionValue = options.at(option);
+    auto optionValue = options.at(to_underlying(option));
     if (!optionValue) {
         throw_std_runtime_error("option {} not set", option);
     }
