@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -33,17 +35,20 @@
 #ifndef __CONFIG_OPTIONS_H__
 #define __CONFIG_OPTIONS_H__
 
+#include <fmt/format.h>
 #include <map>
+#include <memory>
 #include <vector>
 
-#include "content/autoscan.h"
-#include "transcoding/transcoding.h"
+#include "config_int_types.h"
 
 // forward declaration
-class AutoscanList;
+class AutoscanDirectory;
+class BoxLayoutList;
 class ClientConfigList;
 class DirectoryConfigList;
 class DynamicContentList;
+class TranscodingProfileList;
 
 class ConfigOption {
 public:
@@ -54,10 +59,18 @@ public:
         throw std::runtime_error("Wrong option type string");
     }
 
-    virtual int getIntOption() const
+    virtual IntOptionType getIntOption() const
     {
         throw std::runtime_error("Wrong option type int");
     }
+    virtual UIntOptionType getUIntOption() const
+    {
+        throw std::runtime_error("Wrong option type uint");
+    }
+    virtual LongOptionType getLongOption() const
+    {
+        throw std::runtime_error("Wrong option type long");
+    }
 
     virtual bool getBoolOption() const
     {
@@ -66,42 +79,47 @@ public:
 
     virtual std::map<std::string, std::string> getDictionaryOption(bool forEdit = false) const
     {
-        throw std::runtime_error("Wrong option type dictionary");
+        throw std::runtime_error("Wrong option type Dictionary");
     }
 
     virtual std::vector<std::vector<std::pair<std::string, std::string>>> getVectorOption(bool forEdit = false) const
     {
-        throw std::runtime_error("Wrong option type array");
+        throw std::runtime_error("Wrong option type Vector");
     }
 
-    virtual std::vector<AutoscanDirectory> getAutoscanListOption() const
+    virtual std::vector<std::shared_ptr<AutoscanDirectory>> getAutoscanListOption() const
     {
-        throw std::runtime_error("Wrong option type autoscan list");
+        throw std::runtime_error("Wrong option type Autoscan list");
     }
 
     virtual std::shared_ptr<ClientConfigList> getClientConfigListOption() const
     {
-        throw std::runtime_error("Wrong option type client list");
+        throw std::runtime_error("Wrong option type Client list");
+    }
+
+    virtual std::shared_ptr<BoxLayoutList> getBoxLayoutListOption() const
+    {
+        throw std::runtime_error("Wrong option type BoxLayout list");
     }
 
     virtual std::vector<std::string> getArrayOption(bool forEdit = false) const
     {
-        throw std::runtime_error("Wrong option type array");
+        throw std::runtime_error("Wrong option type Array");
     }
 
     virtual std::shared_ptr<TranscodingProfileList> getTranscodingProfileListOption() const
     {
-        throw std::runtime_error("Wrong option type transcoding list");
+        throw std::runtime_error("Wrong option type Transcoding list");
     }
 
     virtual std::shared_ptr<DirectoryConfigList> getDirectoryTweakOption() const
     {
-        throw std::runtime_error("Wrong option type directory list");
+        throw std::runtime_error("Wrong option type Directory list");
     }
 
     virtual std::shared_ptr<DynamicContentList> getDynamicContentListOption() const
     {
-        throw std::runtime_error("Wrong option type dynamic content list");
+        throw std::runtime_error("Wrong option type DynamicContent list");
     }
 };
 
@@ -120,15 +138,53 @@ private:
 
 class IntOption : public ConfigOption {
 public:
-    explicit IntOption(int option)
+    explicit IntOption(IntOptionType option, std::string optionString = "")
         : option(option)
+        , optionString(std::move(optionString))
     {
     }
 
-    int getIntOption() const override { return option; }
+    std::string getOption() const override { return optionString.length() == 0 ? fmt::to_string(option) : optionString; }
+
+    IntOptionType getIntOption() const override { return option; }
 
 private:
-    int option;
+    IntOptionType option;
+    std::string optionString;
+};
+
+class UIntOption : public ConfigOption {
+public:
+    explicit UIntOption(UIntOptionType option, std::string optionString = "")
+        : option(option)
+        , optionString(std::move(optionString))
+    {
+    }
+
+    std::string getOption() const override { return optionString.length() == 0 ? fmt::to_string(option) : optionString; }
+
+    UIntOptionType getUIntOption() const override { return option; }
+
+private:
+    UIntOptionType option;
+    std::string optionString;
+};
+
+class LongOption : public ConfigOption {
+public:
+    explicit LongOption(LongOptionType option, std::string optionString = "")
+        : option(option)
+        , optionString(std::move(optionString))
+    {
+    }
+
+    std::string getOption() const override { return optionString.length() == 0 ? fmt::to_string(option) : optionString; }
+
+    LongOptionType getLongOption() const override { return option; }
+
+private:
+    LongOptionType option;
+    std::string optionString;
 };
 
 class BoolOption : public ConfigOption {
@@ -234,15 +290,27 @@ private:
 
 class AutoscanListOption : public ConfigOption {
 public:
-    explicit AutoscanListOption(std::vector<AutoscanDirectory> autoscans_)
+    explicit AutoscanListOption(std::vector<std::shared_ptr<AutoscanDirectory>> autoscans_)
         : autoscans(std::move(autoscans_))
     {
     }
 
-    [[nodiscard]] std::vector<AutoscanDirectory> getAutoscanListOption() const override { return autoscans; }
+    [[nodiscard]] std::vector<std::shared_ptr<AutoscanDirectory>> getAutoscanListOption() const override { return autoscans; }
 
 private:
-    std::vector<AutoscanDirectory> autoscans;
+    std::vector<std::shared_ptr<AutoscanDirectory>> autoscans;
+};
+
+class BoxLayoutListOption : public ConfigOption {
+public:
+    explicit BoxLayoutListOption(std::shared_ptr<BoxLayoutList> option)
+        : option(std::move(option))
+    {
+    }
+    std::shared_ptr<BoxLayoutList> getBoxLayoutListOption() const override { return option; }
+
+private:
+    std::shared_ptr<BoxLayoutList> option;
 };
 
 class ClientConfigListOption : public ConfigOption {
