@@ -4,7 +4,7 @@
 
     autoscan_list.cc - this file is part of Gerbera.
 
-    Copyright (C) 2021-2022 Gerbera Contributors
+    Copyright (C) 2021-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -22,10 +22,22 @@
 */
 
 /// \file autoscan_list.cc
+#define GRB_LOG_FAC GrbLogFacility::autoscan
+
 #include "autoscan_list.h"
 
-#include "autoscan.h"
+#include "config/result/autoscan.h"
+#include "content.h"
 #include "database/database.h"
+#include "exceptions.h"
+#include "util/timer.h"
+
+#ifdef HAVE_INOTIFY
+#include "content/inotify/autoscan_inotify.h"
+#include "content/inotify/mt_inotify.h"
+#endif
+
+#include <algorithm>
 
 void AutoscanList::updateLMinDB(Database& database)
 {
@@ -148,7 +160,7 @@ std::shared_ptr<AutoscanList> AutoscanList::removeIfSubdir(const fs::path& paren
     for (auto it = list.begin(); it != list.end(); /*++it*/) {
         auto dir = *it;
 
-        if (parent <= dir->getLocation()) {
+        if (isSubDir(dir->getLocation(), parent)) {
             if (dir->persistent() && !persistent) {
                 ++it;
                 continue;
@@ -176,8 +188,36 @@ void AutoscanList::notifyAll(Timer::Subscriber* sub)
     AutoLock lock(mutex);
 
     for (auto&& i : list) {
-        if (i->getScanMode() == AutoscanDirectory::ScanMode::Timed) {
+        if (i->getScanMode() == AutoscanScanMode::Timed) {
             sub->timerNotify(i->getTimerParameter());
         }
     }
 }
+
+void AutoscanList::initTimer(
+    std::shared_ptr<Content>& content,
+    std::shared_ptr<Timer>& timer
+#ifdef HAVE_INOTIFY
+    ,
+    bool doInotify, std::unique_ptr<AutoscanInotify>& inotify
+#endif
+)
+{
+    AutoLock lock(mutex);
+    for (auto&& autoScanDir : list) {
+#ifdef HAVE_INOTIFY
+        if (autoScanDir->getScanMode() == AutoscanScanMode::INotify) {
+            if (doInotify && inotify)
+                inotify->monitor(autoScanDir);
+            auto param = std::make_shared<Timer::Parameter>(Timer::TimerParamType::IDAutoscan, autoScanDir->getScanID());
+            log_debug("Adding one-shot inotify scan");
+            timer->addTimerSubscriber(content.get(), std::chrono::minutes(1), std::move(param), true);
+        }
+#endif
+        if (autoScanDir->getScanMode() == AutoscanScanMode::Timed) {
+            auto param = std::make_shared<Timer::Parameter>(Timer::TimerParamType::IDAutoscan, autoScanDir->getScanID());
+            log_debug("Adding timed scan with interval {}", autoScanDir->getInterval().count());
+            timer->addTimerSubscriber(content.get(), autoScanDir->getInterval(), std::move(param), false);
+        }
+    }
+}
