@@ -4,7 +4,7 @@
 
     autoscan_list.h - this file is part of Gerbera.
 
-    Copyright (C) 2021-2022 Gerbera Contributors
+    Copyright (C) 2021-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -29,8 +29,15 @@
 #include "util/timer.h"
 
 // forward declaration
-class Database;
 class AutoscanDirectory;
+class Content;
+class Context;
+class Database;
+class Timer;
+
+#ifdef HAVE_INOTIFY
+class AutoscanInotify;
+#endif
 
 class AutoscanList {
 public:
@@ -40,6 +47,7 @@ public:
     /// the index in the AutoscanList.
     ///
     /// \param dir AutoscanDirectory to add to the list.
+    /// \param index position of new entry
     /// \return scanID of the newly added AutoscanDirectory
     int add(const std::shared_ptr<AutoscanDirectory>& dir, std::size_t index = std::numeric_limits<std::size_t>::max());
 
@@ -73,6 +81,15 @@ public:
     /// \brief returns a copy of the autoscan list in the form of an array
     std::vector<std::shared_ptr<AutoscanDirectory>> getArrayCopy() const;
 
+    void initTimer(
+        std::shared_ptr<Content>& content,
+        std::shared_ptr<Timer>& timer
+#ifdef HAVE_INOTIFY
+        ,
+        bool doInotify, std::unique_ptr<AutoscanInotify>& inotify
+#endif
+    );
+
 protected:
     std::size_t origSize {};
     std::map<std::size_t, std::shared_ptr<AutoscanDirectory>> indexMap;
