@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -32,21 +34,35 @@
 #ifndef __JS_LAYOUT_H__
 #define __JS_LAYOUT_H__
 
+#include "layout.h"
+
 #include <memory>
 
-#include "content/scripting/import_script.h"
-#include "layout.h"
+class ImportScript;
+class ScriptingRuntime;
 
 class JSLayout : public Layout {
 protected:
     std::shared_ptr<ScriptingRuntime> runtime;
     std::unique_ptr<ImportScript> import_script;
 
-public:
-    JSLayout(const std::shared_ptr<ContentManager>& content,
-        const std::shared_ptr<ScriptingRuntime>& runtime);
+    std::vector<int> addVideo(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap) override;
+    std::vector<int> addImage(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap) override;
+    std::vector<int> addAudio(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap) override;
 
-    void processCdsObject(const std::shared_ptr<CdsObject>& obj, const fs::path& rootpath, const std::string& contentType) override;
+#ifdef ONLINE_SERVICES
+    std::vector<int> addOnlineItem(const std::shared_ptr<CdsObject>& obj, OnlineServiceType serviceType, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap) override;
+#endif
+
+public:
+    JSLayout(const std::shared_ptr<Content>& content, const std::string& parent);
+
+    void processCdsObject(const std::shared_ptr<CdsObject>& obj,
+        const std::shared_ptr<CdsContainer>& parent,
+        const fs::path& rootpath,
+        const std::string& contentType,
+        const std::map<AutoscanMediaMode, std::string>& containerMap,
+        std::vector<int>& refObjects) override;
 };
 
 #endif // __JS_LAYOUT_H__
