@@ -4,7 +4,7 @@
 
     curl_online_service.cc - this file is part of Gerbera.
 
-    Copyright (C) 2021-2022 Gerbera Contributors
+    Copyright (C) 2021-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -24,12 +24,21 @@
 /// \file curl_online_service.cc
 
 #ifdef HAVE_CURL
+#ifdef ONLINE_SERVICES
+#define GRB_LOG_FAC GrbLogFacility::online
 #include "curl_online_service.h" // API
 
+#include "cds/cds_item.h"
 #include "config/config.h"
-#include "content/content_manager.h"
+#include "config/config_val.h"
+#include "config/result/autoscan.h"
+#include "content/content.h"
+#include "content/layout/layout.h"
+#include "context.h"
 #include "database/database.h"
+#include "exceptions.h"
 #include "util/string_converter.h"
+#include "util/tools.h"
 #include "util/url.h"
 
 CurlContentHandler::CurlContentHandler(const std::shared_ptr<Context>& context)
@@ -38,10 +47,11 @@ CurlContentHandler::CurlContentHandler(const std::shared_ptr<Context>& context)
 {
 }
 
-CurlOnlineService::CurlOnlineService(const std::shared_ptr<ContentManager>& content, std::string serviceName)
+CurlOnlineService::CurlOnlineService(const std::shared_ptr<Content>& content, std::string serviceName)
     : OnlineService(content)
     , curl_handle(curl_easy_init())
     , serviceName(std::move(serviceName))
+    , converterManager(content->getContext()->getConverterManager())
 {
     if (!curl_handle)
         throw_std_runtime_error("failed to initialize curl");
@@ -61,14 +71,18 @@ std::string CurlOnlineService::getServiceName() const
 std::unique_ptr<pugi::xml_document> CurlOnlineService::getData()
 {
     long retcode;
-    auto sc = StringConverter::i2i(config);
+    auto sc = converterManager->i2i();
 
     std::string buffer;
 
     try {
         log_debug("DOWNLOADING URL: {}", service_url);
-        buffer = URL::download(service_url, &retcode,
-            curl_handle, false, true, true);
+#ifdef GRBDEBUG
+        bool verbose = GrbLogger::Logger.isDebugging(GRB_LOG_FAC) || GrbLogger::Logger.isDebugLogging();
+#else
+        bool verbose = false;
+#endif
+        buffer = URL(service_url, curl_handle).download(&retcode, false, verbose, true).second;
     } catch (const std::runtime_error& ex) {
         log_error("Failed to download {} XML data: {}", serviceName, ex.what());
         return nullptr;
@@ -80,7 +94,7 @@ std::unique_ptr<pugi::xml_document> CurlOnlineService::getData()
     if (retcode != 200)
         return nullptr;
 
-    log_debug("GOT BUFFER{}", buffer);
+    log_debug("GOT BUFFER {}", buffer);
     auto doc = std::make_unique<pugi::xml_document>();
     pugi::xml_parse_result result = doc->load_string(sc->convert(buffer).c_str());
     if (result.status != pugi::xml_parse_status::status_ok) {
@@ -116,13 +130,12 @@ bool CurlOnlineService::refreshServiceData(const std::shared_ptr<Layout>& layout
     auto sc = getContentHandler();
     sc->setServiceContent(std::move(reply));
 
+    auto mappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
     std::shared_ptr<CdsObject> obj;
     do {
-        /// \todo add try/catch here and a possibility do find out if we
-        /// may request more stuff or if we are at the end of the list
         obj = sc->getNextObject();
         if (!obj)
-            break;
+            continue;
 
         obj->setVirtual(true);
 
@@ -133,10 +146,13 @@ bool CurlOnlineService::refreshServiceData(const std::shared_ptr<Layout>& layout
 
             if (layout) {
                 std::string mimetype = item->getMimeType();
-                auto mappings = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
                 std::string contentType = getValueOrDefault(mappings, mimetype);
+                std::vector<int> refObjects;
 
-                layout->processCdsObject(obj, "", contentType);
+                layout->processCdsObject(obj, nullptr, "",
+                    contentType,
+                    AutoscanDirectory::ContainerTypesDefaults,
+                    refObjects);
             }
         } else {
             log_debug("Updating existing {} object", serviceName);
@@ -144,12 +160,10 @@ bool CurlOnlineService::refreshServiceData(const std::shared_ptr<Layout>& layout
             obj->setParentID(old->getParentID());
             content->updateObject(obj);
         }
-
-        //        if (server->getShutdownStatus())
-        //            return false;
-    } while (obj);
+    } while (obj || !sc->isEnd());
 
     return false;
 }
 
+#endif // ONLINE_SERVICES
 #endif // HAVE_CURL
