@@ -4,7 +4,7 @@
 
     curl_online_service.h - this file is part of Gerbera.
 
-    Copyright (C) 2021-2022 Gerbera Contributors
+    Copyright (C) 2021-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -25,19 +25,21 @@
 /// \brief Definition of the CurlContentHandler class.
 /// \brief Definition of the CurlOnlineService class.
 
-#ifdef HAVE_CURL
-
 #ifndef __CURL_ONLINE_SERVICE_H__
 #define __CURL_ONLINE_SERVICE_H__
 
+#ifdef HAVE_CURL
+#ifdef ONLINE_SERVICES
+
+#include "online_service.h"
+
 #include <curl/curl.h>
 #include <pugixml.hpp>
 
-#include "context.h"
-#include "online_service.h"
-
 // forward declaration
-class ContentManager;
+class Content;
+class Context;
+class ConverterManager;
 class CdsObject;
 
 /// \brief this class is responsible for creating objects from the CurlContentHandler
@@ -48,7 +50,7 @@ public:
     virtual ~CurlContentHandler() = default;
 
     /// \brief Sets the service XML from which we will extract the objects.
-    /// \return false if service XML contained an error status.
+    /// \return \c false if service XML contained an error status.
     virtual void setServiceContent(std::unique_ptr<pugi::xml_document> service) = 0;
 
     /// \brief retrieves an object from the service.
@@ -57,9 +59,12 @@ public:
     /// when the whole service XML is parsed and no more objects are left,
     /// this function will return nullptr.
     ///
-    /// \return CdsObject or nullptr if there are no more objects to parse.
+    /// \return \c CdsObject or \c nullptr if there are no more objects to parse.
     virtual std::shared_ptr<CdsObject> getNextObject() = 0;
 
+    /// \brief Checks whether we reached the end of the file
+    virtual bool isEnd() = 0;
+
 protected:
     std::shared_ptr<Config> config;
     std::shared_ptr<Database> database;
@@ -71,7 +76,7 @@ protected:
 /// handles adding/refreshing content in the database.
 class CurlOnlineService : public OnlineService {
 public:
-    explicit CurlOnlineService(const std::shared_ptr<ContentManager>& content, std::string serviceName);
+    explicit CurlOnlineService(const std::shared_ptr<Content>& content, std::string serviceName);
     ~CurlOnlineService() override;
 
     CurlOnlineService(const CurlOnlineService&) = delete;
@@ -96,8 +101,10 @@ protected:
 
     std::string service_url;
     std::string serviceName;
+    std::shared_ptr<ConverterManager> converterManager;
 };
 
-#endif //__CURL_ONLINE_SERVICE_H__
-
+#endif // ONLINE_SERVICES
 #endif // HAVE_CURL
+
+#endif //__CURL_ONLINE_SERVICE_H__
