@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -30,10 +32,13 @@
 /// \file lastfm_scrobbler.cc
 
 #ifdef HAVE_LASTFMLIB
+#define GRB_LOG_FAC GrbLogFacility::online
 #include "lastfm_scrobbler.h" // API
 
-#include "config/config_manager.h"
-#include "metadata/metadata_handler.h"
+#include "cds_item.h"
+#include "config/config.h"
+#include "context.h"
+#include "metadata/metadata_enums.h"
 #include "util/tools.h"
 
 LastFm::LastFm(const std::shared_ptr<Context>& context)
@@ -43,11 +48,11 @@ LastFm::LastFm(const std::shared_ptr<Context>& context)
 
 void LastFm::run()
 {
-    if (!config->getBoolOption(CFG_SERVER_EXTOPTS_LASTFM_ENABLED))
+    if (!config->getBoolOption(ConfigVal::SERVER_EXTOPTS_LASTFM_ENABLED))
         return;
 
-    std::string username = config->getOption(CFG_SERVER_EXTOPTS_LASTFM_USERNAME);
-    std::string password = config->getOption(CFG_SERVER_EXTOPTS_LASTFM_PASSWORD);
+    std::string username = config->getOption(ConfigVal::SERVER_EXTOPTS_LASTFM_USERNAME);
+    std::string password = config->getOption(ConfigVal::SERVER_EXTOPTS_LASTFM_PASSWORD);
 
     scrobbler = std::make_unique<LastFmScrobbler>(username, password, false, false);
     scrobbler->authenticate();
@@ -70,8 +75,8 @@ void LastFm::startedPlaying(const std::shared_ptr<CdsItem>& item)
 
     currentTrackId = item->getID();
 
-    std::string artist = item->getMetaData(M_ARTIST);
-    std::string title = item->getMetaData(M_TITLE);
+    std::string artist = item->getMetaData(MetadataField::M_ARTIST);
+    std::string title = item->getMetaData(MetadataField::M_TITLE);
 
     log_debug("Artist:\t{}", artist);
     log_debug("Title:\t{}", title);
@@ -84,17 +89,17 @@ void LastFm::startedPlaying(const std::shared_ptr<CdsItem>& item)
 
     auto info = SubmissionInfo(artist, title);
 
-    std::string album = item->getMetaData(M_ALBUM);
+    std::string album = item->getMetaData(MetadataField::M_ALBUM);
     if (!album.empty())
         info.setAlbum(album);
 
-    std::string trackNr = item->getMetaData(M_TRACKNUMBER);
+    std::string trackNr = item->getMetaData(MetadataField::M_TRACKNUMBER);
     if (!trackNr.empty())
         info.setTrackNr(std::stoi(trackNr));
 
     if (item->getResourceCount() > 0) {
         auto resource = item->getResource(ContentHandler::DEFAULT);
-        std::string duration = resource->getAttribute(CdsResource::Attribute::DURATION);
+        std::string duration = resource->getAttribute(ResourceAttribute::DURATION);
         info.setTrackLength(HMSFToMilliseconds(duration) / 1000);
     }
 
