@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -38,9 +40,9 @@
 #include <lastfmlib/lastfmscrobbler.h>
 #include <memory>
 
-#include "cds_objects.h"
-#include "config/config_manager.h"
-#include "context.h"
+class Config;
+class Context;
+class CdsItem;
 
 class LastFm {
 public:
