@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -30,48 +32,65 @@
 /// \file online_service.cc
 
 #ifdef ONLINE_SERVICES
-
+#define GRB_LOG_FAC GrbLogFacility::online
 #include "online_service.h" // API
 
-#include <array>
-
-#include "content/content_manager.h"
+#include "content/content.h"
+#include "context.h"
+#include "database/database.h"
+#include "exceptions.h"
 #include "util/tools.h"
 
 // DO NOT FORGET TO ADD SERVICE STORAGE PREFIXES TO THIS ARRAY WHEN ADDING
 // NEW SERVICES!
-static constexpr auto servicePrefixes = std::array { '\0', 'Y', 'S', 'W', 'T', '\0' };
+// keep old entries to avoid future collisions
+static const std::map<OnlineServiceType, char> servicePrefixes {
+    { OnlineServiceType::None, '\0' },
+#ifdef YOUTUBE
+    { OnlineServiceType::YouTube, 'Y' },
+#endif
+#ifdef SOPCAST
+    { OnlineServiceType::SopCast, 'S' },
+#endif
+#ifdef WEBORAMA
+    { OnlineServiceType::Weborama, 'W' },
+#endif
+#ifdef ATRAILERS
+    { OnlineServiceType::ATrailers, 'T' },
+#endif
+    { OnlineServiceType::Max, '\0' }
+};
 
 void OnlineServiceList::registerService(const std::shared_ptr<OnlineService>& service)
 {
     if (!service)
         return;
 
-    if (service->getServiceType() >= OS_Max) {
+    if (service->getServiceType() >= OnlineServiceType::Max) {
         throw_std_runtime_error("Requested service with illegal type");
     }
 
-    service_list.at(service->getServiceType()) = service;
+    service_list[service->getServiceType()] = service;
 }
 
-std::shared_ptr<OnlineService> OnlineServiceList::getService(service_type_t service) const
+std::shared_ptr<OnlineService> OnlineServiceList::getService(OnlineServiceType service) const
 {
-    if (service > OS_Max)
+    if (service > OnlineServiceType::Max)
         return nullptr;
 
     return service_list.at(service);
 }
 
-OnlineService::OnlineService(const std::shared_ptr<ContentManager>& content)
+OnlineService::OnlineService(const std::shared_ptr<Content>& content)
     : config(content->getContext()->getConfig())
     , database(content->getContext()->getDatabase())
     , content(content)
 {
 }
 
-char OnlineService::getDatabasePrefix(service_type_t service)
+char OnlineService::getDatabasePrefix(OnlineServiceType service)
 {
-    if (service >= OS_Max)
+    if (service >= OnlineServiceType::Max)
         throw_std_runtime_error("Illegal service requested");
 
     return servicePrefixes.at(service);
