@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -30,9 +32,11 @@
 /// \file online_service_helper.cc
 
 #ifdef ONLINE_SERVICES
+#define GRB_LOG_FAC GrbLogFacility::online
 #include "online_service_helper.h" // API
 
-#include "cds_objects.h"
+#include "cds/cds_item.h"
+#include "exceptions.h"
 #include "online_service.h"
 
 std::string OnlineServiceHelper::resolveURL(const std::shared_ptr<CdsItemExternalURL>& item)
@@ -40,15 +44,10 @@ std::string OnlineServiceHelper::resolveURL(const std::shared_ptr<CdsItemExterna
     if (!item->getFlag(OBJECT_FLAG_ONLINE_SERVICE))
         throw_std_runtime_error("The given item does not belong to an online service");
 
-    auto service = service_type_t(std::stoi(item->getAuxData(ONLINE_SERVICE_AUX_ID)));
-    if (service > OS_Max)
+    auto service = OnlineServiceType(std::stoi(item->getAuxData(ONLINE_SERVICE_AUX_ID)));
+    if (service > OnlineServiceType::Max)
         throw_std_runtime_error("Invalid service id");
 
-#ifdef ATRAILERS
-    if (service == OS_ATrailers) {
-        return item->getLocation();
-    }
-#endif
     throw_std_runtime_error("No handler for this service");
 }
 
