@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -30,23 +32,46 @@
 /// \file js_functions.cc
 
 #ifdef HAVE_JS
+#define GRB_LOG_FAC GrbLogFacility::script
 #include "js_functions.h" // API
 
-#include "content/content_manager.h"
+#include "cds/cds_objects.h"
+#include "content/content.h"
 #include "database/database.h"
-#include "metadata/metadata_handler.h"
+#include "exceptions.h"
 #include "script.h"
+#include "script_property.h"
 #include "util/string_converter.h"
+#include "util/tools.h"
 
-// extern "C" {
+duk_ret_t js_print2(duk_context* ctx)
+{
+    std::string mode = toUpper(duk_get_string(ctx, 0));
+    duk_push_string(ctx, " ");
+    duk_replace(ctx, 0);
+    duk_join(ctx, duk_get_top(ctx) - 1);
+    std::string message = duk_get_string(ctx, 0);
+    if (mode == "ERROR")
+        log_error("{}", message);
+    else if (mode == "WARNING")
+        log_warning("{}", message);
+    else if (mode == "DEBUG")
+        log_debug("{}", message);
+    else
+        log_info("{}", message);
+    duk_push_string(ctx, fmt::format("{}: {}", mode, message).c_str());
+    return 1;
+}
 
 duk_ret_t js_print(duk_context* ctx)
 {
     duk_push_string(ctx, " ");
     duk_insert(ctx, 0);
     duk_join(ctx, duk_get_top(ctx) - 1);
-    log_js("{}", duk_get_string(ctx, 0));
-    return 0;
+    auto message = duk_get_string(ctx, 0);
+    log_info("{}", message);
+    duk_push_string(ctx, message);
+    return 1;
 }
 
 duk_ret_t js_copyObject(duk_context* ctx)
@@ -64,51 +89,63 @@ duk_ret_t js_addContainerTree(duk_context* ctx)
     auto self = Script::getContextScript(ctx);
 
     if (!duk_is_array(ctx, 0)) {
-        log_js("js_addContainerTree: No Array");
+        log_debug("No Array");
         return 0;
     }
 
-    std::vector<std::shared_ptr<CdsObject>> result;
+    std::vector<std::shared_ptr<CdsObject>> chain;
     auto length = duk_get_length(ctx, -1);
 
     for (duk_uarridx_t i = 0; i < length; i++) {
         if (duk_get_prop_index(ctx, -1, i)) {
             if (!duk_is_object(ctx, -1)) {
                 duk_pop(ctx);
-                log_js("js_addContainerTree: no object at {}", i);
+                log_debug("no object at {}", i);
                 break;
             }
             duk_to_object(ctx, -1);
             auto cdsObj = self->dukObject2cdsObject(nullptr);
             if (cdsObj) {
-                result.push_back(std::move(cdsObj));
+                chain.push_back(std::move(cdsObj));
             } else {
-                log_js("js_addContainerTree: no CdsObject at {}", i);
+                log_debug("no CdsObject at {}", i);
             }
         }
         duk_pop(ctx);
     }
 
-    if (!result.empty()) {
+    if (!chain.empty()) {
+        auto lastItem = chain.at(chain.size() - 1);
+        if (lastItem->isContainer()) {
+            log_debug("lastContainer {}, resSize {}, refID {}", lastItem->getTitle(), lastItem->getResourceCount(), lastItem->getRefID());
+            if (lastItem->getResourceCount() == 0 && lastItem->getRefID() >= CDS_ID_ROOT) {
+                lastItem = self->getDatabase()->loadObject(lastItem->getRefID());
+                log_debug("lastItem from RefID {}, resSize {}, loc {}", lastItem->getTitle(), lastItem->getResourceCount(), lastItem->getLocation().string());
+            }
+        } else
+            log_debug("lastItem {}, resSize {}, refID {}", lastItem->getTitle(), lastItem->getResourceCount(), lastItem->getRefID());
         auto cm = self->getContent();
-        auto [containerId, containerStatus] = cm->addContainerTree(result);
+        auto [containerId, containerStatus] = cm->addContainerTree(chain, lastItem);
         if (containerId != INVALID_OBJECT_ID) {
             /* setting last container ID as return value */
-            auto tmp = fmt::to_string(containerId);
-            duk_push_string(ctx, tmp.c_str());
+            log_debug("container {}", containerId);
+            duk_push_string(ctx, fmt::to_string(containerId).c_str());
             return 1;
         }
     }
+    log_debug("Chain empty");
 
     return 0;
 }
 
 duk_ret_t js_addCdsObject(duk_context* ctx)
 {
-    auto* self = Script::getContextScript(ctx);
+    auto self = Script::getContextScript(ctx);
 
-    if (!duk_is_object(ctx, 0))
+    if (!duk_is_object(ctx, 0)) {
+        log_debug("No object argument");
         return 0;
+    }
     duk_to_object(ctx, 0);
     // stack: js_cds_obj
     const char* containerId = duk_to_string(ctx, 1);
@@ -117,82 +154,34 @@ duk_ret_t js_addCdsObject(duk_context* ctx)
     // stack: js_cds_obj containerId
 
     try {
-        std::unique_ptr<StringConverter> p2i;
-        std::unique_ptr<StringConverter> i2i;
+        std::string rootPath = ScriptGlobalProperty(ctx, "object_script_path").getStringValue();
 
-        auto config = self->getConfig();
-        if (self->whoami() == S_PLAYLIST) {
-            p2i = StringConverter::p2i(config);
-        } else {
-            i2i = StringConverter::i2i(config);
-        }
-        duk_get_global_string(ctx, "object_script_path");
-        auto rp = duk_get_string(ctx, -1);
-        duk_pop(ctx);
-        std::string rootPath = rp ? rp : "";
-
-        if (self->whoami() == S_PLAYLIST)
-            duk_get_global_string(ctx, "playlist");
-        else if (self->whoami() == S_IMPORT)
-            duk_get_global_string(ctx, "orig");
-        else
+        if (self->getOrigName().empty())
             duk_push_undefined(ctx);
+        else
+            duk_get_global_string(ctx, self->getOrigName().c_str());
         // stack: js_cds_obj containerId js_orig_obj
 
         if (duk_is_undefined(ctx, -1)) {
-            log_debug("Could not retrieve global 'orig'/'playlist' object");
+            log_debug("Could not retrieve global {} object", self->getOrigName());
             return 0;
         }
 
         auto origObject = self->dukObject2cdsObject(self->getProcessedObject());
-        if (!origObject)
-            return 0;
-
-        std::shared_ptr<CdsObject> cdsObj;
-        auto cm = self->getContent();
-        int pcdId = INVALID_OBJECT_ID;
-
-        duk_swap_top(ctx, 0);
-        // stack: js_orig_obj containerId js_cds_obj
-        if (self->whoami() == S_PLAYLIST) {
-            int otype = self->getIntProperty("objectType", -1);
-            if (otype == -1) {
-                log_error("missing objectType property");
-                return 0;
-            }
-
-            if (!IS_CDS_ITEM_EXTERNAL_URL(otype)) {
-                fs::path loc = self->getProperty("location");
-                std::error_code ec;
-                auto dirEnt = fs::directory_entry(loc, ec);
-                if (!ec) {
-                    AutoScanSetting asSetting;
-                    asSetting.followSymlinks = config->getBoolOption(CFG_IMPORT_FOLLOW_SYMLINKS);
-                    asSetting.recursive = false;
-                    asSetting.hidden = config->getBoolOption(CFG_IMPORT_HIDDEN_FILES);
-                    asSetting.rescanResource = false;
-                    asSetting.mergeOptions(config, loc);
-
-                    pcdId = cm->addFile(dirEnt, rootPath, asSetting, false);
-                    if (pcdId == INVALID_OBJECT_ID) {
-                        log_error("Failed to add object {}", dirEnt.path().string());
-                        return 0;
-                    }
-                    auto mainObj = self->getDatabase()->loadObject(pcdId);
-                    cdsObj = self->dukObject2cdsObject(mainObj);
-                } else {
-                    log_error("Failed to read {}: {}", loc.c_str(), ec.message());
-                }
-            } else
-                cdsObj = self->dukObject2cdsObject(origObject);
-        } else
-            cdsObj = self->dukObject2cdsObject(origObject);
-
-        if (!cdsObj) {
+        if (!origObject) {
+            log_debug("No orig object");
             return 0;
         }
 
-        auto [parentId, parentStatus] = std::pair(stoiString(containerId), false);
+        duk_swap_top(ctx, 0);
+        // stack: js_orig_obj containerId js_cds_obj
+        auto [cdsObj, pcdId] = self->createObject2cdsObject(origObject, rootPath);
+        if (!cdsObj) {
+            log_debug("No content object");
+            return 0;
+        }
+
+        auto parentId = stoiString(containerId);
 
         if (parentId <= 0) {
             log_error("Invalid parent id passed to addCdsObject: {}", parentId);
@@ -200,31 +189,20 @@ duk_ret_t js_addCdsObject(duk_context* ctx)
         }
 
         cdsObj->setParentID(parentId);
-        if (!cdsObj->isExternalItem()) {
-            /// \todo get hidden file setting from config manager?
-            /// what about same stuff in content manager, why is it not used
-            /// there?
-
-            if (self->whoami() == S_PLAYLIST) {
-                if (pcdId == INVALID_OBJECT_ID) {
-                    return 0;
-                }
-                cdsObj->setRefID(pcdId);
-            } else
-                cdsObj->setRefID(origObject->getID());
-
-            cdsObj->setFlag(OBJECT_FLAG_USE_RESOURCE_REF);
-        } else if (cdsObj->isExternalItem() && (self->whoami() == S_PLAYLIST) && self->getConfig()->getBoolOption(CFG_IMPORT_SCRIPTING_PLAYLIST_SCRIPT_LINK_OBJECTS)) {
-            cdsObj->setFlag(OBJECT_FLAG_PLAYLIST_REF);
-            cdsObj->setRefID(origObject->getID());
+        if (!self->setRefId(cdsObj, origObject, pcdId)) {
+            log_debug("No ref object");
+            return 0;
         }
 
+        if (self->isHiddenFile(cdsObj, rootPath)) {
+            log_debug("Hidden file {} cannot be added", cdsObj->getLocation().c_str());
+            return 0;
+        }
         cdsObj->setID(INVALID_OBJECT_ID);
-        cm->addObject(cdsObj, parentStatus);
+        self->getContent()->addObject(cdsObj, false);
 
         /* setting object ID as return value */
-        auto tmp = fmt::to_string(parentId);
-        duk_push_string(ctx, tmp.c_str());
+        duk_push_string(ctx, fmt::to_string(cdsObj->getID()).c_str());
         return 1;
     } catch (const ServerShutdownException&) {
         log_warning("Aborting script execution due to server shutdown.");
@@ -278,6 +256,4 @@ duk_ret_t js_j2i(duk_context* ctx)
     return convertCharsetGeneric(ctx, J2I);
 }
 
-//} // extern "C"
-
 #endif // HAVE_JS
