@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    runtime.cc - this file is part of MediaTomb.
+    scripting_runtime.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -30,8 +32,12 @@
 /// \file scripting_runtime.cc
 
 #ifdef HAVE_JS
+#define GRB_LOG_FAC GrbLogFacility::script
+
 #include "scripting_runtime.h" // API
 
+#include "util/logger.h"
+
 ScriptingRuntime::ScriptingRuntime()
     : ctx(duk_create_heap(nullptr, nullptr, nullptr, nullptr, [](auto, auto msg) { log_error("Fatal Duktape error: {}", msg ? msg : "no message"); std::abort(); }))
 {
