@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -28,6 +30,7 @@
 */
 
 /// \file update_manager.cc
+#define GRB_LOG_FAC GrbLogFacility::update
 
 #include "update_manager.h" // API
 
@@ -35,6 +38,9 @@
 
 #include "database/database.h"
 #include "server.h"
+#include "upnp/upnp_common.h"
+#include "util/grb_net.h"
+#include "util/grb_time.h"
 #include "util/tools.h"
 
 static constexpr auto specInterval = std::chrono::seconds(2);
@@ -206,7 +212,7 @@ void UpdateManager::threadProc()
                 if (!updateString.empty()) {
                     try {
                         log_debug("updates sent: \"{}\"", updateString);
-                        server->sendCDSSubscriptionUpdate(updateString);
+                        server->sendSubscriptionUpdate(updateString, UPNP_DESC_CDS_SERVICE_ID);
                         lastUpdate = currentTimeMS();
                     } catch (const std::runtime_error& e) {
                         log_error("Fatal error when sending updates: {}", e.what());
