@@ -4,7 +4,7 @@
 
     context.cc - this file is part of Gerbera.
 
-    Copyright (C) 2021-2022 Gerbera Contributors
+    Copyright (C) 2021-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -22,18 +22,20 @@
 */
 
 /// \file context.cc
-
+#define GRB_LOG_FAC GrbLogFacility::content
 #include "context.h" // API
 
 Context::Context(std::shared_ptr<Config> config,
     std::shared_ptr<ClientManager> clients,
     std::shared_ptr<Mime> mime,
     std::shared_ptr<Database> database,
-    std::shared_ptr<Web::SessionManager> sessionManager)
+    std::shared_ptr<Web::SessionManager> sessionManager,
+    std::shared_ptr<ConverterManager> converterManager)
     : config(std::move(config))
     , clients(std::move(clients))
     , mime(std::move(mime))
     , database(std::move(database))
     , session_manager(std::move(sessionManager))
+    , converterManager(std::move(converterManager))
 {
 }
