@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -29,24 +31,26 @@
 
 /// \file database.h
 
-#ifndef __STORAGE_H__
-#define __STORAGE_H__
+#ifndef __GRB_DATABASE_H__
+#define __GRB_DATABASE_H__
+
+#include "util/grb_fs.h"
 
 #include <map>
 #include <unordered_set>
 #include <vector>
 
-#include "content/autoscan.h"
-#include "util/grb_fs.h"
-
-// forward declaration
+// forward declarations
+class AutoscanDirectory;
 class AutoscanList;
+enum class AutoscanScanMode;
 class CdsContainer;
 class CdsObject;
-struct ClientCacheEntry;
+struct ClientObservation;
 class ClientStatusDetail;
 class Config;
 class ConfigValue;
+class ConverterManager;
 class Mime;
 class Timer;
 
@@ -80,7 +84,7 @@ public:
 
     unsigned int getFlag(unsigned int mask) const { return flags & mask; }
     void setFlag(unsigned int mask) { flags |= mask; }
-    void clearFlag(unsigned int mask) { flags &= !mask; }
+    void clearFlag(unsigned int mask) { flags &= ~mask; }
 
     std::shared_ptr<CdsObject> getObject() const { return object; }
 
@@ -136,10 +140,14 @@ protected:
     int requestedCount;
     bool searchableContainers;
     std::string group;
+    bool searchContainers;
+    bool searchItems;
 
 public:
     SearchParam(std::string containerID, std::string searchCriteria, std::string sortCriteria, int startingIndex,
-        int requestedCount, bool searchableContainers, std::string group)
+        int requestedCount, bool searchableContainers, std::string group,
+        bool searchContainers = true,
+        bool searchItems = true)
         : containerID(std::move(containerID))
         , searchCrit(std::move(searchCriteria))
         , sortCrit(std::move(sortCriteria))
@@ -147,15 +155,54 @@ public:
         , requestedCount(requestedCount)
         , searchableContainers(searchableContainers)
         , group(std::move(group))
+        , searchContainers(searchContainers)
+        , searchItems(searchItems)
     {
     }
-    const std::string& searchCriteria() const { return searchCrit; }
+    const std::string& getSearchCriteria() const { return searchCrit; }
     const std::string& getContainerID() const { return containerID; }
     bool getSearchableContainers() const { return searchableContainers; }
     int getStartingIndex() const { return startingIndex; }
     int getRequestedCount() const { return requestedCount; }
     const std::string& getSortCriteria() const { return sortCrit; }
     const std::string& getGroup() const { return group; }
+    bool getContainers() const { return searchContainers; }
+    bool getItems() const { return searchItems; }
+};
+
+class StatsParam {
+public:
+    enum class StatsMode {
+        Count,
+        Size,
+    };
+
+protected:
+    std::string mimeType;
+    std::string upnpClass;
+    StatsMode mode { StatsMode::Count };
+    bool isVirtual { false };
+
+public:
+    StatsParam(StatsMode mode, std::string mimeType, std::string upnpClass, bool isVirtual)
+        : mimeType(std::move(mimeType))
+        , upnpClass(std::move(upnpClass))
+        , mode(mode)
+        , isVirtual(isVirtual)
+    {
+    }
+    const std::string& getMimeType() const { return mimeType; }
+    const std::string& getUpnpClass() const { return upnpClass; }
+    StatsMode getMode() const { return mode; }
+    bool getVirtual() const { return isVirtual; }
+};
+
+enum class DbFileType {
+    Auto,
+    Directory,
+    File,
+    Virtual,
+    Any,
 };
 
 class Database {
@@ -170,15 +217,11 @@ public:
     virtual void addObject(const std::shared_ptr<CdsObject>& object, int* changedContainer) = 0;
 
     /// \brief Adds a virtual container chain specified by path.
-    /// \param path container path separated by '/'. Slashes in container
-    /// titles must be escaped.
-    /// \param lastClass upnp:class of the last container in the chain, it
-    /// is only set when the container is created for the first time.
-    /// \param lastRefID reference id of the last container in the chain,
-    /// INVALID_OBJECT_ID indicates that the id will not be set.
+    /// \param parentContainerId the id of the parent container
+    /// \param virtualPath container path separated by '/'
+    /// \param cont reference of the object to copy data from. Slashes in container
+    /// title must be escaped.
     /// \param containerID will be filled in by the function
-    /// \param updateID will be filled in by the function only if it is set to INVALID_OBJECT_ID
-    /// and it is necessary to update a container. Otherwise it will be left unchanged.
     ///
     /// The function gets a path (i.e. "/Audio/All Music/") and will create
     /// the container path if needed. The container ID will be filled in with
@@ -207,20 +250,19 @@ public:
     /// \brief Loads a given (pc directory) object, identified by the given path
     /// from the database
     /// \param path the path of the object; object is interpreted as directory
-    /// \param wasRegularFile was a regular file before file was moved, now fs::is_regular_file returns false (used for inotify events)
+    /// \param fileType type of the database entry to search
     /// \return the CdsObject
-    virtual std::shared_ptr<CdsObject> findObjectByPath(const fs::path& path, bool wasRegularFile = false) = 0;
+    virtual std::shared_ptr<CdsObject> findObjectByPath(const fs::path& path, DbFileType fileType = DbFileType::Auto) = 0;
 
     /// \brief checks for a given (pc directory) object, identified by the given path
     /// from the database
-    /// \param path the path of the object; object is interpreted as directory
-    /// \param wasRegularFile was a regular file before file was moved, now fs::is_regular_file returns false (used for inotify events)
+    /// \param fullpath the path of the object; object is interpreted as directory
+    /// \param fileType type of the database entry to search
     /// \return the obejectID
-    virtual int findObjectIDByPath(const fs::path& fullpath, bool wasRegularFile = false) = 0;
+    virtual int findObjectIDByPath(const fs::path& fullpath, DbFileType fileType = DbFileType::Auto) = 0;
 
     /// \brief increments the updateIDs for the given objectIDs
     /// \param ids pointer to the array of ids
-    /// \param size number of entries in the given array
     /// \return a String for UPnP: a CSV list; for every existing object:
     ///  "id,update_id"
     virtual std::string incrementUpdateIDs(const std::unordered_set<int>& ids) = 0;
@@ -243,17 +285,20 @@ public:
     /// the object is a reference to another object, the "all" flag
     /// determines, if the main object will be removed too.
     /// \param objectID the object id of the object to remove
+    /// \param path delete resource references to this path
     /// \param all if true and the object to be removed is a reference
-    /// \param objectType pointer to an int; will be filled with the objectType of
-    /// the removed object, if not NULL
     /// \return changed container ids
-    virtual std::unique_ptr<ChangedContainers> removeObject(int objectID, bool all) = 0;
+    virtual std::unique_ptr<ChangedContainers> removeObject(int objectID, const fs::path& path, bool all) = 0;
 
     /// \brief Get all objects under the given parentID.
     /// \param parentID parent container
     /// \param withoutContainer if false: all children are returned; if true: only items are returned
-    /// \return DBHash containing the objectID's - nullptr if there are none!
-    virtual std::unordered_set<int> getObjects(int parentID, bool withoutContainer) = 0;
+    /// \param ret list of matching objects
+    /// \param full do full hierarchy search
+    /// \return number of objects found
+    virtual std::size_t getObjects(int parentID, bool withoutContainer, std::unordered_set<int>& ret, bool full) = 0;
+    virtual std::vector<int> getRefObjects(int objectId) = 0;
+    virtual std::unordered_set<int> getUnreferencedObjects() = 0;
 
     /// \brief Remove all objects found in list
     /// \param list a DBHash containing objectIDs that have to be removed
@@ -270,15 +315,16 @@ public:
     virtual std::vector<int> getServiceObjectIDs(char servicePrefix) = 0;
 
     /* accounting methods */
-    virtual int getTotalFiles(bool isVirtual = false, const std::string& mimeType = "", const std::string& upnpClass = "") = 0;
+    virtual long long getFileStats(const StatsParam& stats) = 0;
+    virtual std::map<std::string, long long> getGroupStats(const StatsParam& stats) = 0;
 
     /* internal setting methods */
     virtual std::string getInternalSetting(const std::string& key) = 0;
     virtual void storeInternalSetting(const std::string& key, const std::string& value) = 0;
 
     /* autoscan methods */
-    virtual std::shared_ptr<AutoscanList> getAutoscanList(AutoscanDirectory::ScanMode scanode) = 0;
-    virtual void updateAutoscanList(AutoscanDirectory::ScanMode scanmode, const std::shared_ptr<AutoscanList>& list) = 0;
+    virtual std::shared_ptr<AutoscanList> getAutoscanList(AutoscanScanMode scanode) = 0;
+    virtual void updateAutoscanList(AutoscanScanMode scanmode, const std::shared_ptr<AutoscanList>& list) = 0;
 
     /* config methods */
     virtual std::vector<ConfigValue> getConfigValues() = 0;
@@ -286,8 +332,8 @@ public:
     virtual void updateConfigValue(const std::string& key, const std::string& item, const std::string& value, const std::string& status = "unchanged") = 0;
 
     /* clients methods */
-    virtual std::vector<ClientCacheEntry> getClients() = 0;
-    virtual void saveClients(const std::vector<ClientCacheEntry>& cache) = 0;
+    virtual std::vector<ClientObservation> getClients() = 0;
+    virtual void saveClients(const std::vector<ClientObservation>& cache) = 0;
     virtual std::shared_ptr<ClientStatusDetail> getPlayStatus(const std::string& group, int objectId) = 0;
     virtual void savePlayStatus(const std::shared_ptr<ClientStatusDetail>& detail) = 0;
     virtual std::vector<std::shared_ptr<ClientStatusDetail>> getPlayStatusList(int objectId) = 0;
@@ -310,7 +356,8 @@ public:
     /// present in the database. If it does not exist it will be created, but
     /// it's content will not be added.
     ///
-    /// \param *changedContainer returns the ID for the UpdateManager
+    /// \param path location of the container to handle
+    /// \param changedContainer returns the ID for the UpdateManager
     /// \return objectID of the container given by path
     virtual int ensurePathExistence(const fs::path& path, int* changedContainer) = 0;
 
@@ -318,7 +365,10 @@ public:
     virtual bool threadCleanupRequired() const = 0;
 
 protected:
-    static std::shared_ptr<Database> createInstance(const std::shared_ptr<Config>& config, const std::shared_ptr<Mime>& mime, const std::shared_ptr<Timer>& timer);
+    static std::shared_ptr<Database> createInstance(const std::shared_ptr<Config>& config,
+        const std::shared_ptr<Mime>& mime,
+        const std::shared_ptr<ConverterManager>& converterManager,
+        const std::shared_ptr<Timer>& timer);
     friend class Server;
 
     virtual std::shared_ptr<Database> getSelf() = 0;
@@ -326,4 +376,4 @@ protected:
     std::shared_ptr<Config> config;
 };
 
-#endif // __STORAGE_H__
+#endif // __GRB_DATABASE_H__
