@@ -3,7 +3,7 @@
 
   search_handler.cc - this file is part of Gerbera.
 
-  Copyright (C) 2018-2022 Gerbera Contributors
+  Copyright (C) 2018-2024 Gerbera Contributors
 
   Gerbera is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2
@@ -22,15 +22,19 @@
 
 /// \file search_handler.cc
 
+#define GRB_LOG_FAC GrbLogFacility::sqldatabase
 #include "search_handler.h" // API
 
+#include "config/config_val.h"
+#include "database/sql_database.h"
+#include "exceptions.h"
+#include "metadata/metadata_enums.h"
+#include "util/grb_time.h"
+#include "util/tools.h"
+
 #include <algorithm>
 #include <stack>
 
-#include "database/sql_database.h"
-#include "metadata/metadata_handler.h"
-#include "util/tools.h"
-
 static const std::unordered_map<std::string_view, TokenType> tokenTypes {
     { "(", TokenType::LPAREN },
     { ")", TokenType::RPAREN },
@@ -153,7 +157,7 @@ std::string SearchLexer::nextStringToken(const std::string& input)
             break;
     }
     if (startPos == currentPos)
-        throw_std_runtime_error("String expected, found '{}' instead", input[currentPos]);
+        throw SearchParseException(fmt::format("String expected, found '{}' instead", input[currentPos]), LINE_MESSAGE);
     return input.substr(startPos, currentPos - startPos);
 }
 
@@ -180,7 +184,7 @@ void SearchParser::getNextToken()
 std::unique_ptr<ASTNode> SearchParser::parse()
 {
     getNextToken();
-    if (currentToken->getType() == TokenType::ASTERISK)
+    if (currentToken && currentToken->getType() == TokenType::ASTERISK)
         return std::make_unique<ASTAsterisk>(sqlEmitter, currentToken->getValue());
 
     return parseSearchExpression();
@@ -197,18 +201,18 @@ std::unique_ptr<ASTNode> SearchParser::parseSearchExpression()
             auto expressionNode = parseRelationshipExpression();
             if (currentOperator == TokenType::AND) {
                 if (!nodeStack.top())
-                    throw_std_runtime_error("Cannot construct ASTAndOperator without lhs");
+                    throw SearchParseException("Cannot construct ASTAndOperator without lhs", LINE_MESSAGE);
                 if (!expressionNode)
-                    throw_std_runtime_error("Cannot construct ASTAndOperator without rhs");
+                    throw SearchParseException("Cannot construct ASTAndOperator without rhs", LINE_MESSAGE);
                 auto lhs = std::move(nodeStack.top());
                 nodeStack.pop();
                 nodeStack.push(std::make_unique<ASTAndOperator>(sqlEmitter, std::move(lhs), std::move(expressionNode)));
                 operatorStack.pop();
             } else if (currentOperator == TokenType::OR) {
                 if (!nodeStack.top())
-                    throw_std_runtime_error("Cannot construct ASTOrOperator without lhs");
+                    throw SearchParseException("Cannot construct ASTOrOperator without lhs", LINE_MESSAGE);
                 if (!expressionNode)
-                    throw_std_runtime_error("Cannot construct ASTOrOperator without rhs");
+                    throw SearchParseException("Cannot construct ASTOrOperator without rhs", LINE_MESSAGE);
                 auto lhs = std::move(nodeStack.top());
                 nodeStack.pop();
                 nodeStack.push(std::make_unique<ASTOrOperator>(sqlEmitter, std::move(lhs), std::move(expressionNode)));
@@ -242,7 +246,7 @@ std::unique_ptr<ASTNode> SearchParser::parseSearchExpression()
                 else
                     root = std::make_unique<ASTOrOperator>(sqlEmitter, std::move(lhs), std::move(root));
             } else
-                throw_std_runtime_error("Cannot construct ASTOrOperator/ASTAndOperator without rhs");
+                throw SearchParseException("Cannot construct ASTOrOperator/ASTAndOperator without rhs", LINE_MESSAGE);
         }
     }
     return root;
@@ -251,7 +255,7 @@ std::unique_ptr<ASTNode> SearchParser::parseSearchExpression()
 std::unique_ptr<ASTNode> SearchParser::parseParenthesis()
 {
     if (currentToken->getType() != TokenType::LPAREN)
-        throw_std_runtime_error("Failed to parse search criteria - expecting a ')'");
+        throw SearchParseException("Failed to parse search criteria - expecting a ')'", LINE_MESSAGE);
 
     std::unique_ptr<ASTNode> currentNode;
     getNextToken();
@@ -272,7 +276,7 @@ std::unique_ptr<ASTNode> SearchParser::parseParenthesis()
             else if (tokenType == TokenType::OR)
                 currentNode = std::make_unique<ASTOrOperator>(sqlEmitter, std::move(lhsNode), std::move(rhsNode));
             else
-                throw_std_runtime_error("Failed to parse search criteria - expected and/or");
+                throw SearchParseException("Failed to parse search criteria - expected and/or", LINE_MESSAGE);
 
             getNextToken();
         } else if (currentToken->getType() == TokenType::LPAREN) {
@@ -281,58 +285,58 @@ std::unique_ptr<ASTNode> SearchParser::parseParenthesis()
         }
     }
     if (!currentNode)
-        throw_std_runtime_error("Failed to parse search criteria - bad expression between parenthesis");
+        throw SearchParseException("Failed to parse search criteria - bad expression between parenthesis", LINE_MESSAGE);
 
     return std::make_unique<ASTParenthesis>(sqlEmitter, std::move(currentNode));
 }
 
 std::unique_ptr<ASTNode> SearchParser::parseRelationshipExpression()
 {
-    if (currentToken->getType() != TokenType::PROPERTY)
+    if (!currentToken || currentToken->getType() != TokenType::PROPERTY)
         throw_std_runtime_error("Failed to parse search criteria - expecting a property name");
 
     auto property = std::make_unique<ASTProperty>(sqlEmitter, currentToken->getValue());
 
     getNextToken();
-    if (currentToken->getType() == TokenType::COMPAREOP) {
+    if (currentToken && currentToken->getType() == TokenType::COMPAREOP) {
         auto operatr = std::make_unique<ASTCompareOperator>(sqlEmitter, currentToken->getValue());
         getNextToken();
         auto quotedString = parseQuotedString();
         return std::make_unique<ASTCompareExpression>(sqlEmitter, std::move(property), std::move(operatr), std::move(quotedString));
     }
 
-    if (currentToken->getType() == TokenType::STRINGOP) {
+    if (currentToken && currentToken->getType() == TokenType::STRINGOP) {
         auto operatr = std::make_unique<ASTStringOperator>(sqlEmitter, currentToken->getValue());
         getNextToken();
         auto quotedString = parseQuotedString();
         return std::make_unique<ASTStringExpression>(sqlEmitter, std::move(property), std::move(operatr), std::move(quotedString));
     }
 
-    if (currentToken->getType() == TokenType::EXISTS) {
+    if (currentToken && currentToken->getType() == TokenType::EXISTS) {
         auto operatr = std::make_unique<ASTExistsOperator>(sqlEmitter, currentToken->getValue());
         getNextToken();
         auto booleanValue = std::make_unique<ASTBoolean>(sqlEmitter, currentToken->getValue());
         return std::make_unique<ASTExistsExpression>(sqlEmitter, std::move(property), std::move(operatr), std::move(booleanValue));
     }
 
-    throw_std_runtime_error("Failed to parse search criteria - expecting a comparison, exists, or string operator");
+    throw SearchParseException("Failed to parse search criteria - expecting a comparison, exists, or string operator", LINE_MESSAGE);
 }
 
 std::unique_ptr<ASTQuotedString> SearchParser::parseQuotedString()
 {
-    if (currentToken->getType() != TokenType::DQUOTE)
+    if (!currentToken || currentToken->getType() != TokenType::DQUOTE)
         throw_std_runtime_error("Failed to parse search criteria - expecting a double-quote");
     auto openQuote = std::make_unique<ASTDQuote>(sqlEmitter, currentToken->getValue());
     getNextToken();
 
-    if (currentToken->getType() != TokenType::ESCAPEDSTRING)
-        throw_std_runtime_error("Failed to parse search criteria - expecting an escaped string value");
+    if (!currentToken || currentToken->getType() != TokenType::ESCAPEDSTRING)
+        throw SearchParseException("Failed to parse search criteria - expecting an escaped string value", LINE_MESSAGE);
 
     auto escapedString = std::make_unique<ASTEscapedString>(sqlEmitter, currentToken->getValue());
     getNextToken();
 
-    if (currentToken->getType() != TokenType::DQUOTE)
-        throw_std_runtime_error("Failed to parse search criteria - expecting a double-quote");
+    if (!currentToken || currentToken->getType() != TokenType::DQUOTE)
+        throw SearchParseException("Failed to parse search criteria - expecting a double-quote", LINE_MESSAGE);
     auto closeQuote = std::make_unique<ASTDQuote>(sqlEmitter, currentToken->getValue());
 
     return std::make_unique<ASTQuotedString>(sqlEmitter, std::move(openQuote), std::move(escapedString), std::move(closeQuote));
@@ -539,10 +543,11 @@ std::string ASTOrOperator::emit() const
     return sqlEmitter.emit(this, lhs->emit(), rhs->emit());
 }
 
-DefaultSQLEmitter::DefaultSQLEmitter(std::shared_ptr<ColumnMapper> colMapper, std::shared_ptr<ColumnMapper> metaMapper, std::shared_ptr<ColumnMapper> resMapper)
+DefaultSQLEmitter::DefaultSQLEmitter(std::shared_ptr<ColumnMapper> colMapper, std::shared_ptr<ColumnMapper> metaMapper, std::shared_ptr<ColumnMapper> resMapper, std::shared_ptr<ColumnMapper> plyMapper)
     : colMapper(std::move(colMapper))
     , metaMapper(std::move(metaMapper))
     , resMapper(std::move(resMapper))
+    , plyMapper(std::move(plyMapper))
 {
 }
 
@@ -578,21 +583,25 @@ static const std::map<std::string, std::string> logicOperator {
     { "@compare", "({2}LOWER('{3}'))" }, // lower
 };
 
-std::pair<std::string, std::string> DefaultSQLEmitter::getPropertyStatement(const std::string& property) const
+std::tuple<std::string, std::string, FieldType> DefaultSQLEmitter::getPropertyStatement(const std::string& property) const
 {
     if (colMapper && colMapper->hasEntry(property)) {
-        return { colMapper->mapQuoted(property), colMapper->mapQuotedLower(property) };
+        return { colMapper->mapQuoted(property), colMapper->mapQuotedLower(property), colMapper->getFieldType(property) };
     }
     if (resMapper && resMapper->hasEntry(property)) {
-        return { resMapper->mapQuoted(property), resMapper->mapQuotedLower(property) };
+        return { resMapper->mapQuoted(property), resMapper->mapQuotedLower(property), resMapper->getFieldType(property) };
+    }
+    if (plyMapper && plyMapper->hasEntry(property)) {
+        return { plyMapper->mapQuoted(property), plyMapper->mapQuotedLower(property), plyMapper->getFieldType(property) };
     }
     if (metaMapper) {
         return {
             fmt::format("{0}='{2}' AND {1}", metaMapper->mapQuoted(META_NAME), metaMapper->mapQuoted(META_VALUE), property),
-            fmt::format("{0}='{2}' AND {1}", metaMapper->mapQuoted(META_NAME), metaMapper->mapQuotedLower(META_VALUE), property)
+            fmt::format("{0}='{2}' AND {1}", metaMapper->mapQuoted(META_NAME), metaMapper->mapQuotedLower(META_VALUE), property),
+            metaMapper->getFieldType(property)
         };
     }
-    log_info("Property {} not yet supported. Search may return no result!", property);
+    log_warning("Property {} not yet supported. Search may return no result!", property);
     return {};
 }
 
@@ -600,17 +609,29 @@ std::string DefaultSQLEmitter::emit(const ASTCompareOperator* node, const std::s
     const std::string& value) const
 {
     auto operatr = node->getValue();
-    auto [prpUpper, prpLower] = getPropertyStatement(property);
+    auto [prpUpper, prpLower, prpType] = getPropertyStatement(property);
 
-    if ((operatr == ">" || operatr == ">=") && startswith(value, "@last")) {
-        auto dateVal = currentTime() - std::chrono::hours(24 * stoiString(value.substr(5)));
-        return fmt::format(logicOperator.at("newer"), "", fmt::format("{} {}", prpUpper, operatr), fmt::format("{} {}", prpLower, operatr), dateVal.count());
+    if (prpType == FieldType::Date) {
+        if ((operatr == ">" || operatr == ">=") && startswith(value, "@last")) {
+            auto dateVal = currentTime() - std::chrono::hours(24 * stoiString(value.substr(5)));
+            return fmt::format(logicOperator.at("newer"), "", fmt::format("{} {}", prpUpper, operatr), fmt::format("{} {}", prpLower, operatr), dateVal.count());
+        }
+        if (operatr == ">" || operatr == ">=" || operatr == "<" || operatr == "<=") {
+            std::chrono::seconds dateVal(0);
+            if (parseSimpleDate(value, dateVal)) {
+                log_debug("parseSimpleDate {} -> {}", value, dateVal.count());
+                return fmt::format(logicOperator.at("newer"), "", fmt::format("{} {}", prpUpper, operatr), fmt::format("{} {}", prpLower, operatr), dateVal.count());
+            }
+        }
+    }
+    if (prpType == FieldType::Integer && (operatr == ">" || operatr == ">=" || operatr == "<" || operatr == "<=")) {
+        return fmt::format("{} {} {}", prpUpper, operatr, stoiString(value));
     }
 
     if (operatr != "=")
-        throw_std_runtime_error("Operator '{}' not yet supported", operatr);
+        throw_std_runtime_error("Operation '{}' '{}' not yet supported", operatr, value);
 
-    auto [clsUpper, clcLower] = getPropertyStatement(UPNP_SEARCH_CLASS);
+    auto [clsUpper, clsLower, clsType] = getPropertyStatement(UPNP_SEARCH_CLASS);
     return fmt::format(logicOperator.at((property[0] == '@') ? "@compare" : "compare"), clsUpper,
         fmt::format("{}{}", prpUpper, operatr), fmt::format("{}{}", prpLower, operatr), value);
 }
@@ -619,10 +640,10 @@ std::string DefaultSQLEmitter::emit(const ASTStringOperator* node, const std::st
 {
     auto stringOperator = toLower(node->getValue());
     if (logicOperator.find(stringOperator) == logicOperator.end()) {
-        throw_std_runtime_error("Operation '{}' not yet supported", stringOperator);
+        throw SearchParseException(fmt::format("Operation '{}' not yet supported", stringOperator), LINE_MESSAGE);
     }
-    auto [prpUpper, prpLower] = getPropertyStatement(property);
-    auto [clsUpper, clsLower] = getPropertyStatement(UPNP_SEARCH_CLASS);
+    auto [prpUpper, prpLower, prpType] = getPropertyStatement(property);
+    auto clsUpper = std::get<0>(getPropertyStatement(UPNP_SEARCH_CLASS));
     return fmt::format(logicOperator.at(stringOperator), clsUpper, prpUpper, prpLower, value);
 }
 
@@ -634,10 +655,10 @@ std::string DefaultSQLEmitter::emit(const ASTExistsOperator* node, const std::st
     } else if (value == "false") {
         exists = "NULL";
     } else {
-        throw_std_runtime_error("Invalid value '{}' on rhs of 'exists' operator", value);
+        throw SearchParseException(fmt::format("Invalid value '{}' on rhs of 'exists' operator", value), LINE_MESSAGE);
     }
-    auto [prpUpper, prpLower] = getPropertyStatement(property);
-    auto [clsUpper, clsLower] = getPropertyStatement(UPNP_SEARCH_CLASS);
+    auto [prpUpper, prpLower, prpType] = getPropertyStatement(property);
+    auto [clsUpper, clsLower, clsType] = getPropertyStatement(UPNP_SEARCH_CLASS);
     return fmt::format(logicOperator.at((property[0] == '@') ? "@exists" : "exists"), clsUpper, prpUpper, prpLower, exists);
 }
 
@@ -653,8 +674,9 @@ std::string DefaultSQLEmitter::emit(const ASTOrOperator* node, const std::string
     return fmt::format("{} OR {}", lhs, rhs);
 }
 
-SortParser::SortParser(std::shared_ptr<ColumnMapper> colMapper, std::shared_ptr<ColumnMapper> metaMapper, std::string sortCriteria)
+SortParser::SortParser(std::shared_ptr<ColumnMapper> colMapper, std::shared_ptr<ColumnMapper> plyMapper, std::shared_ptr<ColumnMapper> metaMapper, std::string sortCriteria)
     : colMapper(std::move(colMapper))
+    , plyMapper(std::move(plyMapper))
     , metaMapper(std::move(metaMapper))
     , sortCrit(std::move(sortCriteria))
 {
@@ -662,9 +684,9 @@ SortParser::SortParser(std::shared_ptr<ColumnMapper> colMapper, std::shared_ptr<
 
 std::string SortParser::parse(std::string& addColumns, std::string& addJoin)
 {
-    if (sortCrit.empty()) {
+    if (sortCrit.empty() || !colMapper || !metaMapper)
         return {};
-    }
+
     std::vector<std::string> sort;
     std::vector<std::string> colBuf;
     std::vector<std::string> joinBuf;
@@ -677,10 +699,10 @@ std::string SortParser::parse(std::string& addColumns, std::string& addJoin)
         } else {
             log_warning("Unknown sort direction '{}' in '{}'", seg, sortCrit);
         }
-        if (!colMapper || !colMapper->mapQuotedList(sort, seg, (desc ? "DESC" : "ASC"))) {
+        if (!colMapper->mapQuotedList(sort, seg, (desc ? "DESC" : "ASC")) && !plyMapper->mapQuotedList(sort, seg, (desc ? "DESC" : "ASC"))) {
             std::string sortSql;
             for (auto&& metaId : MetadataIterator()) {
-                auto&& metaName = MetadataHandler::getMetaFieldName(metaId);
+                auto&& metaName = MetaEnumMapper::getMetaFieldName(metaId);
                 if (metaName == seg) {
                     sortSql = metaName;
                 }
