@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -32,22 +34,23 @@
 #ifndef __SQL_STORAGE_H__
 #define __SQL_STORAGE_H__
 
+#include "config/config.h"
+#include "config/config_val.h"
+#include "database.h"
+#include "sql_format.h"
+
 #include <array>
 #include <mutex>
 #include <unordered_set>
 #include <utility>
 
-#include "config/config.h"
-#include "database.h"
-#include "sql_format.h"
-
 // forward declarations
 class CdsContainer;
 class CdsResource;
 class SQLResult;
 class SQLEmitter;
 
-#define DBVERSION 19
+#define DBVERSION 22
 
 #define CDS_OBJECT_TABLE "mt_cds_object"
 #define INTERNAL_SETTINGS_TABLE "mt_internal_setting"
@@ -83,6 +86,14 @@ public:
             return null_value;
         return std::atoi(c);
     }
+    /// \brief Return the value of column index as an integer value
+    long long col_long(int index, long long null_value) const
+    {
+        const char* c = col_c_str(index);
+        if (!c || *c == '\0')
+            return null_value;
+        return std::atoll(c);
+    }
     virtual char* col_c_str(int index) const = 0;
 };
 
@@ -113,7 +124,10 @@ public:
     virtual void rollback(std::string_view tName) { }
     virtual void commit(std::string_view tName) { }
 
+    virtual void del(std::string_view tableName, const std::string& clause, const std::vector<int>& ids) = 0;
+    virtual void exec(std::string_view tableName, const std::string& query, int objId) = 0;
     virtual int exec(const std::string& query, bool getLastInsertId = false) = 0;
+    virtual void execOnly(const std::string& query) = 0;
     virtual std::shared_ptr<SQLResult> select(const std::string& query) = 0;
 
     void addObject(const std::shared_ptr<CdsObject>& obj, int* changedContainer) override;
@@ -124,16 +138,19 @@ public:
     int getChildCount(int contId, bool containers, bool items, bool hideFsRoot) override;
     std::map<int, int> getChildCounts(const std::vector<int>& contId, bool containers, bool items, bool hideFsRoot) override;
 
-    std::unordered_set<int> getObjects(int parentID, bool withoutContainer) override;
+    std::size_t getObjects(int parentID, bool withoutContainer, std::unordered_set<int>& ret, bool full) override;
+    std::vector<int> getRefObjects(int objectId) override;
+    std::unordered_set<int> getUnreferencedObjects() override;
 
-    std::unique_ptr<ChangedContainers> removeObject(int objectID, bool all) override;
+    std::unique_ptr<ChangedContainers> removeObject(int objectID, const fs::path& path, bool all) override;
     std::unique_ptr<ChangedContainers> removeObjects(const std::unordered_set<int>& list, bool all = false) override;
 
     std::shared_ptr<CdsObject> loadObjectByServiceID(const std::string& serviceID) override;
     std::vector<int> getServiceObjectIDs(char servicePrefix) override;
 
     /* accounting methods */
-    int getTotalFiles(bool isVirtual = false, const std::string& mimeType = "", const std::string& upnpClass = "") override;
+    long long getFileStats(const StatsParam& stats) override;
+    std::map<std::string, long long> getGroupStats(const StatsParam& stats) override;
 
     std::vector<std::shared_ptr<CdsObject>> browse(BrowseParam& param) override;
     std::vector<std::shared_ptr<CdsObject>> search(const SearchParam& param, int* numMatches) override;
@@ -142,8 +159,8 @@ public:
 
     std::vector<std::shared_ptr<CdsObject>> findObjectByContentClass(const std::string& contentClass) override;
     // virtual std::shared_ptr<CdsObject> findObjectByTitle(std::string title, int parentID);
-    std::shared_ptr<CdsObject> findObjectByPath(const fs::path& fullpath, bool wasRegularFile = false) override;
-    int findObjectIDByPath(const fs::path& fullpath, bool wasRegularFile = false) override;
+    std::shared_ptr<CdsObject> findObjectByPath(const fs::path& fullpath, DbFileType fileType = DbFileType::Auto) override;
+    int findObjectIDByPath(const fs::path& fullpath, DbFileType fileType = DbFileType::Auto) override;
     std::string incrementUpdateIDs(const std::unordered_set<int>& ids) override;
 
     fs::path buildContainerPath(int parentID, const std::string& title) override;
@@ -151,8 +168,8 @@ public:
     std::string getInternalSetting(const std::string& key) override;
     void storeInternalSetting(const std::string& key, const std::string& value) override = 0;
 
-    std::shared_ptr<AutoscanList> getAutoscanList(AutoscanDirectory::ScanMode scanmode) override;
-    void updateAutoscanList(AutoscanDirectory::ScanMode scanmode, const std::shared_ptr<AutoscanList>& list) override;
+    std::shared_ptr<AutoscanList> getAutoscanList(AutoscanScanMode scanmode) override;
+    void updateAutoscanList(AutoscanScanMode scanmode, const std::shared_ptr<AutoscanList>& list) override;
 
     std::shared_ptr<AutoscanDirectory> getAutoscanDirectory(int objectID) override;
     void addAutoscanDirectory(const std::shared_ptr<AutoscanDirectory>& adir) override;
@@ -166,8 +183,8 @@ public:
     void updateConfigValue(const std::string& key, const std::string& item, const std::string& value, const std::string& status = "unchanged") override;
 
     /* clients methods */
-    std::vector<ClientCacheEntry> getClients() override;
-    void saveClients(const std::vector<ClientCacheEntry>& cache) override;
+    std::vector<ClientObservation> getClients() override;
+    void saveClients(const std::vector<ClientObservation>& cache) override;
     std::shared_ptr<ClientStatusDetail> getPlayStatus(const std::string& group, int objectId) override;
     std::vector<std::shared_ptr<ClientStatusDetail>> getPlayStatusList(int objectId) override;
     void savePlayStatus(const std::shared_ptr<ClientStatusDetail>& detail) override;
@@ -185,17 +202,17 @@ public:
 
     unsigned int getHash(std::size_t index) const { return index < DBVERSION ? hashies.at(index) : 0; }
 
-    int insert(std::string_view tableName, const std::vector<SQLIdentifier>& fields, const std::vector<std::string>& values, bool getLastInsertId = false);
+    int insert(std::string_view tableName, const std::vector<SQLIdentifier>& fields, const std::vector<std::string>& values, bool getLastInsertId = false, bool warnOnly = false);
     void insertMultipleRows(std::string_view tableName, const std::vector<SQLIdentifier>& fields, const std::vector<std::vector<std::string>>& valuesets);
     template <typename T>
     void updateRow(std::string_view tableName, const std::vector<ColumnUpdate>& values, std::string_view key, const T& value);
     void deleteAll(std::string_view tableName);
     template <typename T>
-    void deleteRow(std::string_view tableName, std::string_view key, const T& value);
-    void deleteRows(std::string_view tableName, std::string_view key, const std::vector<int>& values);
+    void deleteRow(const std::string& tableName, const std::string& key, const T& value);
+    void deleteRows(std::string_view tableName, const std::string& key, const std::vector<int>& values);
 
 protected:
-    explicit SQLDatabase(const std::shared_ptr<Config>& config, std::shared_ptr<Mime> mime);
+    explicit SQLDatabase(const std::shared_ptr<Config>& config, std::shared_ptr<Mime> mime, std::shared_ptr<ConverterManager> converterManager);
     void init() override;
 
     /// \brief migrate metadata from mt_cds_objects to mt_metadata before removing the column (DBVERSION 12)
@@ -210,9 +227,10 @@ protected:
     void migrateResources(int objectId, const std::string& resourcesStr);
 
     /// \brief returns a fmt-printable identifier name
-    SQLIdentifier identifier(std::string_view name) const { return { name, table_quote_begin, table_quote_end }; }
+    SQLIdentifier identifier(const std::string& name) const { return { name, table_quote_begin, table_quote_end }; }
 
     std::shared_ptr<Mime> mime;
+    std::shared_ptr<ConverterManager> converterManager;
 
     char table_quote_begin { '\0' };
     char table_quote_end { '\0' };
@@ -222,7 +240,7 @@ protected:
     using SqlAutoLock = std::scoped_lock<decltype(sqlMutex)>;
     std::map<int, std::shared_ptr<CdsContainer>> dynamicContainers;
 
-    void upgradeDatabase(unsigned int dbVersion, const std::array<unsigned int, DBVERSION>& hashies, config_option_t upgradeOption, std::string_view updateVersionCommand, std::string_view addResourceColumnCmd);
+    void upgradeDatabase(unsigned int dbVersion, const std::array<unsigned int, DBVERSION>& hashies, ConfigVal upgradeOption, std::string_view updateVersionCommand, std::string_view addResourceColumnCmd);
     virtual void _exec(const std::string& query) = 0;
 
 private:
@@ -301,7 +319,7 @@ private:
 
     std::shared_ptr<CdsObject> checkRefID(const std::shared_ptr<CdsObject>& obj);
     int createContainer(int parentID, const std::string& name, const std::string& virtualPath, int flags, bool isVirtual, const std::string& upnpClass, int refID,
-        const std::vector<std::pair<std::string, std::string>>& itemMetadata);
+        const std::vector<std::pair<std::string, std::string>>& itemMetadata, const std::vector<std::shared_ptr<CdsResource>>& itemResources = {});
 
     static bool remapBool(const std::string& field) { return field == "1"; }
     static bool remapBool(int field) { return field == 1; }
@@ -314,11 +332,11 @@ private:
 template <typename T>
 void SQLDatabase::updateRow(std::string_view tableName, const std::vector<ColumnUpdate>& values, std::string_view key, const T& value)
 {
-    exec(fmt::format("UPDATE {} SET {} WHERE {} = {}", identifier(tableName), fmt::join(values, ", "), identifier(key), quote(value)));
+    exec(fmt::format("UPDATE {} SET {} WHERE {} = {}", identifier(std::string(tableName)), fmt::join(values, ", "), identifier(std::string(key)), quote(value)));
 }
 
 template <typename T>
-void SQLDatabase::deleteRow(std::string_view tableName, std::string_view key, const T& value)
+void SQLDatabase::deleteRow(const std::string& tableName, const std::string& key, const T& value)
 {
     exec(fmt::format("DELETE FROM {} WHERE {} = {}", identifier(tableName), identifier(key), quote(value)));
 }
@@ -326,7 +344,7 @@ void SQLDatabase::deleteRow(std::string_view tableName, std::string_view key, co
 class SqlWithTransactions {
 protected:
     explicit SqlWithTransactions(const std::shared_ptr<Config>& config)
-        : use_transaction(config->getBoolOption(CFG_SERVER_STORAGE_USE_TRANSACTIONS))
+        : use_transaction(config->getBoolOption(ConfigVal::SERVER_STORAGE_USE_TRANSACTIONS))
     {
     }
 
