@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    sqlite3_database.cc - this file is part of MediaTomb.
+    sqlite_database.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,37 +29,42 @@
     $Id$
 */
 
-/// \file sqlite3_database.cc
+/// \file sqlite_database.cc
+#define GRB_LOG_FAC GrbLogFacility::sqlite3
 
 #include "sqlite_database.h" // API
 
-#include "config/config_manager.h"
+#include "config/config.h"
+#include "config/config_val.h"
+#include "exceptions.h"
+#include "sl_task.h"
 
-#define DB_BACKUP_FORMAT "{}.backup"
-
-#define SQLITE3_SET_VERSION "INSERT INTO \"mt_internal_setting\" VALUES('db_version', '{}')"
 static constexpr auto sqlite3UpdateVersion = std::string_view(R"(UPDATE "mt_internal_setting" SET "value"='{}' WHERE "key"='db_version' AND "value"='{}')");
 static constexpr auto sqlite3AddResourceAttr = std::string_view(R"(ALTER TABLE "grb_cds_resource" ADD COLUMN "{}" varchar(255) default NULL)");
 
-Sqlite3Database::Sqlite3Database(const std::shared_ptr<Config>& config, const std::shared_ptr<Mime>& mime, std::shared_ptr<Timer> timer)
-    : SQLDatabase(config, mime)
+#define DELETE_CACHE_MAX_TIME 60 // drop cache if last delete was more than 60 secs ago
+#define DELETE_CACHE_RED_SIZE 0.2 // reduce cache to 80% of max entries
+
+Sqlite3Database::Sqlite3Database(const std::shared_ptr<Config>& config, const std::shared_ptr<Mime>& mime, const std::shared_ptr<ConverterManager>& converterManager, std::shared_ptr<Timer> timer)
+    : SQLDatabase(config, mime, converterManager)
     , timer(std::move(timer))
 {
     table_quote_begin = '"';
     table_quote_end = '"';
 
     // if sqlite3.sql or sqlite3-upgrade.xml is changed hashies have to be updated
-    hashies = { 3585221191, // index 0 is used for create script sqlite3.sql
-        778996897, 3362507034, 853149842, 4035419264, 3497064885, 974692115, 119767663, 3167732653, 2427825904, 3305506356, // upgrade 1-10
-        43189396, 2767540493, 2512852146, 1273710965, 319062951, 3593597366, 1028160353, 2644417837 };
+    hashies = { 220562690, // index 0 is used for create script sqlite3.sql = Version 1
+        778996897, 3362507034, 853149842, 4035419264, 3497064885, 974692115, 119767663, 3167732653, 2427825904, 3305506356, // upgrade 2-11
+        43189396, 2767540493, 2512852146, 1273710965, 319062951, 3593597366, 1028160353, 881071639, 1989518047, 3743992560, // upgrade 12-21
+        3135921396 };
 }
 
 void Sqlite3Database::prepare()
 {
     _exec("PRAGMA locking_mode = EXCLUSIVE");
     _exec("PRAGMA foreign_keys = ON");
-    _exec("PRAGMA journal_mode = WAL");
-    exec(fmt::format("PRAGMA synchronous = {}", config->getIntOption(CFG_SERVER_STORAGE_SQLITE_SYNCHRONOUS)));
+    _exec(fmt::format("PRAGMA journal_mode = {}", config->getOption(ConfigVal::SERVER_STORAGE_SQLITE_JOURNALMODE)));
+    exec(fmt::format("PRAGMA synchronous = {}", config->getIntOption(ConfigVal::SERVER_STORAGE_SQLITE_SYNCHRONOUS)));
 }
 
 void Sqlite3Database::init()
@@ -65,7 +72,7 @@ void Sqlite3Database::init()
     dbInitDone = false;
     SQLDatabase::init();
 
-    fs::path dbFilePath = config->getOption(CFG_SERVER_STORAGE_SQLITE_DATABASE_FILE);
+    fs::path dbFilePath = config->getOption(ConfigVal::SERVER_STORAGE_SQLITE_DATABASE_FILE);
     auto dbFile = GrbFile(dbFilePath);
     log_debug("SQLite path: {}", dbFilePath.c_str());
 
@@ -97,14 +104,14 @@ void Sqlite3Database::init()
     }
 
     std::string dbVersion;
-    fs::path dbFilePathbackup = fmt::format(DB_BACKUP_FORMAT, dbFilePath.c_str());
+    fs::path dbFilePathbackup = fmt::format(SQLITE3_BACKUP_FORMAT, dbFilePath.c_str());
     auto dbBackupFile = GrbFile(dbFilePathbackup);
     try {
         dbVersion = getInternalSetting("db_version");
     } catch (const std::runtime_error&) {
         log_warning("Sqlite3 database seems to be corrupt or doesn't exist yet.");
         // database seems to be corrupt or nonexistent
-        if (config->getBoolOption(CFG_SERVER_STORAGE_SQLITE_RESTORE) && sqliteStatus == SQLITE_OK) {
+        if (config->getBoolOption(ConfigVal::SERVER_STORAGE_SQLITE_RESTORE) && sqliteStatus == SQLITE_OK) {
             // try to restore database
 
             // checking for backup file
@@ -148,15 +155,15 @@ void Sqlite3Database::init()
     }
 
     try {
-        upgradeDatabase(std::stoul(dbVersion), hashies, CFG_SERVER_STORAGE_SQLITE_UPGRADE_FILE, sqlite3UpdateVersion, sqlite3AddResourceAttr);
-        if (config->getBoolOption(CFG_SERVER_STORAGE_SQLITE_BACKUP_ENABLED) && timer) {
+        upgradeDatabase(std::stoul(dbVersion), hashies, ConfigVal::SERVER_STORAGE_SQLITE_UPGRADE_FILE, sqlite3UpdateVersion, sqlite3AddResourceAttr);
+        if (config->getBoolOption(ConfigVal::SERVER_STORAGE_SQLITE_BACKUP_ENABLED) && timer) {
             // do a backup now
             auto btask = std::make_shared<SLBackupTask>(config, false);
             this->addTask(btask);
             btask->waitForTask();
 
             // add timer for backups
-            auto backupInterval = std::chrono::seconds(config->getIntOption(CFG_SERVER_STORAGE_SQLITE_BACKUP_INTERVAL));
+            auto backupInterval = std::chrono::seconds(config->getIntOption(ConfigVal::SERVER_STORAGE_SQLITE_BACKUP_INTERVAL));
             timer->addTimerSubscriber(this, backupInterval, nullptr);
             hasBackupTimer = true;
         }
@@ -189,15 +196,15 @@ std::string Sqlite3Database::quote(const std::string& value) const
     return ret;
 }
 
-std::string Sqlite3Database::getError(const std::string& query, const std::string& error, sqlite3* db, int errorCode)
+std::string Sqlite3Database::handleError(const std::string& query, const std::string& error, sqlite3* db, int errorCode)
 {
     sqliteStatus = (errorCode == SQLITE_BUSY || errorCode == SQLITE_LOCKED) ? SQLITE_LOCKED : SQLITE_OK;
-    return fmt::format("SQLITE3: ({}, : {}) {}\nQuery: {}\nerror: {}", sqlite3_errcode(db), sqlite3_extended_errcode(db), sqlite3_errmsg(db), query.empty() ? "unknown" : query, error.empty() ? "unknown" : error);
+    return fmt::format("SQLITE3 ({}: {}): {}\n       Query: {}\n       Error: {}", sqlite3_errcode(db), sqlite3_extended_errcode(db), sqlite3_errmsg(db), query.empty() ? "unknown" : query, error.empty() ? "unknown" : error);
 }
 
-Sqlite3DatabaseWithTransactions::Sqlite3DatabaseWithTransactions(const std::shared_ptr<Config>& config, const std::shared_ptr<Mime>& mime, const std::shared_ptr<Timer>& timer)
+Sqlite3DatabaseWithTransactions::Sqlite3DatabaseWithTransactions(const std::shared_ptr<Config>& config, const std::shared_ptr<Mime>& mime, const std::shared_ptr<ConverterManager>& converterManager, const std::shared_ptr<Timer>& timer)
     : SqlWithTransactions(config)
-    , Sqlite3Database(config, mime, timer)
+    , Sqlite3Database(config, mime, converterManager, timer)
 {
 }
 
@@ -240,12 +247,65 @@ std::shared_ptr<SQLResult> Sqlite3Database::select(const std::string& query)
         stask->waitForTask();
         return stask->getResult();
     } catch (const std::runtime_error& e) {
-        if (dbInitDone) {
-            log_error("prematurely shutting down.");
+        if (!dbInitDone)
+            throw_std_runtime_error(e.what());
+        if (!shutdownFlag) {
+            log_error("Prematurely shutting down.\n{}", e.what());
             rollback("");
             shutdown();
         }
-        throw_std_runtime_error(e.what());
+        log_error("Already shutting down.\n{}", e.what());
+        return {};
+    }
+}
+
+void Sqlite3Database::del(std::string_view tableName, const std::string& clause, const std::vector<int>& ids)
+{
+    auto query = clause.empty() //
+        ? fmt::format("DELETE FROM {}", identifier(std::string(tableName))) //
+        : fmt::format("DELETE FROM {} WHERE {}", identifier(std::string(tableName)), clause);
+    try {
+        log_debug("Adding delete to Queue: {}", query);
+        {
+            DelAutoLock del_lock(del_mutex);
+            maxDeleteCount = maxDeleteCount > ids.size() * 10 ? maxDeleteCount : ids.size() * 10;
+            for (auto&& id : ids) {
+                deletedEntries.push_back(fmt::format("{}_{}", tableName, id));
+            }
+            lastDelete = currentTime();
+        }
+        auto etask = std::make_shared<SLExecTask>(query, false);
+        addTask(etask);
+        etask->waitForTask();
+    } catch (const std::runtime_error& e) {
+        if (!dbInitDone)
+            throw_std_runtime_error(e.what());
+        if (!shutdownFlag) {
+            log_error("Prematurely shutting down.\n{}", e.what());
+            rollback("");
+            shutdown();
+        }
+        log_error("Already shutting down.\n{}", e.what());
+    }
+}
+
+void Sqlite3Database::exec(std::string_view tableName, const std::string& query, int objId)
+{
+    try {
+        log_debug("Adding query to Queue: {}", query);
+        auto eKey = fmt::format("{}_{}", tableName, objId);
+        auto etask = std::make_shared<SLExecTask>(query, eKey);
+        addTask(etask);
+        etask->waitForTask();
+    } catch (const std::runtime_error& e) {
+        if (!dbInitDone)
+            throw_std_runtime_error(e.what());
+        if (!shutdownFlag) {
+            log_error("Prematurely shutting down.\n{}", e.what());
+            rollback("");
+            shutdown();
+        }
+        log_error("Already shutting down.\n{}", e.what());
     }
 }
 
@@ -258,12 +318,27 @@ int Sqlite3Database::exec(const std::string& query, bool getLastInsertId)
         etask->waitForTask();
         return getLastInsertId ? etask->getLastInsertId() : -1;
     } catch (const std::runtime_error& e) {
-        if (dbInitDone) {
-            log_error("prematurely shutting down.");
+        if (!dbInitDone)
+            throw_std_runtime_error(e.what());
+        if (!shutdownFlag) {
+            log_error("Prematurely shutting down.\n{}", e.what());
             rollback("");
             shutdown();
         }
-        throw_std_runtime_error(e.what());
+        log_error("Already shutting down.\n{}", e.what());
+        return -1;
+    }
+}
+
+void Sqlite3Database::execOnly(const std::string& query)
+{
+    try {
+        log_debug("Adding query to Queue: {}", query);
+        auto etask = std::make_shared<SLExecTask>(query, false, false);
+        addTask(etask);
+        etask->waitForTask();
+    } catch (const std::runtime_error& e) {
+        log_error("Failed to execute {}\n{}", query, e.what());
     }
 }
 
@@ -273,7 +348,7 @@ void Sqlite3Database::threadProc()
     try {
         sqlite3* db;
 
-        std::string dbFilePath = config->getOption(CFG_SERVER_STORAGE_SQLITE_DATABASE_FILE);
+        std::string dbFilePath = config->getOption(ConfigVal::SERVER_STORAGE_SQLITE_DATABASE_FILE);
 
         int res = sqlite3_open(dbFilePath.c_str(), &db);
         if (res != SQLITE_OK) {
@@ -286,6 +361,14 @@ void Sqlite3Database::threadProc()
         // tell init() that we are ready
         threadRunner->setReady();
 
+        auto throwOnError = [&](const std::shared_ptr<SLTask>& task) {
+            for (auto&& ent : deletedEntries) {
+                if (!task->checkKey(ent))
+                    return false;
+            }
+            return task->getThrowOnError();
+        };
+
         while (!shutdownFlag) {
             while (!taskQueue.empty()) {
                 auto task = std::move(taskQueue.front());
@@ -293,7 +376,7 @@ void Sqlite3Database::threadProc()
 
                 lock.unlock();
                 try {
-                    task->run(db, this);
+                    task->run(db, this, throwOnError(task));
                     if (task->didContamination())
                         dirty = true;
                     else if (task->didDecontamination())
@@ -306,6 +389,15 @@ void Sqlite3Database::threadProc()
             }
 
             /* if nothing to do, sleep until awakened */
+            auto now = currentTime();
+            if (now.count() - lastDelete.count() > DELETE_CACHE_MAX_TIME) { // drop cache if last delete was more than 60 secs ago
+                DelAutoLock del_lock(del_mutex);
+                maxDeleteCount = DELETE_CACHE_MAX_SIZE;
+                deletedEntries.clear();
+            } else if (deletedEntries.size() > maxDeleteCount) { // dynamically increase if large DELETESs happen
+                DelAutoLock del_lock(del_mutex);
+                deletedEntries.erase(deletedEntries.begin(), deletedEntries.begin() + maxDeleteCount * DELETE_CACHE_RED_SIZE);
+            }
             threadRunner->wait(lock);
         }
         log_debug("Exiting");
@@ -326,11 +418,19 @@ void Sqlite3Database::threadProc()
             }
             db = nullptr;
         }
-    } catch (const std::runtime_error&) {
-        log_error("Aborting thread");
+    } catch (const std::runtime_error& e) {
+        log_error("Aborting thread {}", e.what());
     }
 }
 
+/* Sqlite3BackupTimerSubscriber */
+
+void Sqlite3Database::timerNotify(const std::shared_ptr<Timer::Parameter>& param)
+{
+    auto btask = std::make_shared<SLBackupTask>(config, false);
+    addTask(btask, true);
+}
+
 void Sqlite3Database::addTask(const std::shared_ptr<SLTask>& task, bool onlyIfDirty)
 {
     if (!taskQueueOpen) {
@@ -373,197 +473,6 @@ void Sqlite3Database::storeInternalSetting(const std::string& key, const std::st
     exec(command);
 }
 
-bool SLTask::is_running() const
-{
-    return running;
-}
-
-void SLTask::sendSignal()
-{
-    if (is_running()) { // we check before we lock first, because there is no need to lock then
-        std::scoped_lock<decltype(mutex)> lock(mutex);
-        running = false;
-        cond.notify_one();
-    }
-}
-
-void SLTask::sendSignal(std::string error)
-{
-    this->error = std::move(error);
-    sendSignal();
-}
-
-void SLTask::waitForTask()
-{
-    if (is_running()) { // we check before we lock first, because there is no need to lock then
-        std::unique_lock<decltype(mutex)> lock(mutex);
-        if (is_running()) { // we check it a second time after locking to ensure we didn't miss the pthread_cond_signal
-            cond.wait(lock); // waiting for the task to complete
-        }
-    }
-
-    if (!getError().empty()) {
-        log_debug("{}", getError());
-        throw_std_runtime_error(getError());
-    }
-}
-
-/* SLInitTask */
-SLInitTask::SLInitTask(std::shared_ptr<Config> config, unsigned int hashie)
-    : config(std::move(config))
-    , hashie(hashie)
-{
-}
-
-void SLInitTask::run(sqlite3*& db, Sqlite3Database* sl)
-{
-    log_debug("Running: init");
-    std::string dbFilePath = config->getOption(CFG_SERVER_STORAGE_SQLITE_DATABASE_FILE);
-
-    sqlite3_close(db);
-
-    int res = sqlite3_open(dbFilePath.c_str(), &db);
-    if (res != SQLITE_OK)
-        throw DatabaseException("", "SQLite: Failed to create new database");
-
-    auto sqlFilePath = fs::path(config->getOption(CFG_SERVER_STORAGE_SQLITE_INIT_SQL_FILE));
-    log_debug("Loading initialisation SQL from: {}", sqlFilePath.c_str());
-    auto sql = GrbFile(std::move(sqlFilePath)).readTextFile();
-    auto&& myHash = stringHash(sql);
-
-    if (myHash == hashie) {
-        sql += fmt::format("\n" SQLITE3_SET_VERSION ";", DBVERSION);
-
-        char* err = nullptr;
-        int ret = sqlite3_exec(
-            db,
-            sql.c_str(),
-            nullptr,
-            nullptr,
-            &err);
-        std::string error;
-        if (err) {
-            error = err;
-            sqlite3_free(err);
-        }
-        if (ret != SQLITE_OK) {
-            throw DatabaseException("", sl->getError(sql, error, db, ret));
-        }
-        contamination = true;
-    } else {
-        log_warning("Wrong hash for create script {}: {} != {}", DBVERSION, myHash, hashie);
-    }
-}
-
-/* SLSelectTask */
-SLSelectTask::SLSelectTask(const std::string& query)
-    : query(query.c_str())
-{
-}
-
-void SLSelectTask::run(sqlite3*& db, Sqlite3Database* sl)
-{
-    log_debug("Running: {}", query);
-    pres = std::make_shared<Sqlite3Result>();
-
-    char* err = nullptr;
-    int ret = sqlite3_get_table(
-        db,
-        query,
-        &pres->table,
-        &pres->nrow,
-        &pres->ncolumn,
-        &err);
-    std::string error;
-    if (err) {
-        log_debug(err);
-        error = err;
-        sqlite3_free(err);
-    }
-    if (ret != SQLITE_OK) {
-        throw DatabaseException("", sl->getError(query, error, db, ret));
-    }
-
-    pres->row = pres->table;
-    pres->cur_row = 0;
-}
-
-/* SLExecTask */
-
-SLExecTask::SLExecTask(const std::string& query, bool getLastInsertId)
-    : query(query.c_str())
-    , getLastInsertIdFlag(getLastInsertId)
-{
-}
-
-void SLExecTask::run(sqlite3*& db, Sqlite3Database* sl)
-{
-    log_debug("Running: {}", query);
-    char* err;
-    int ret = sqlite3_exec(
-        db,
-        query,
-        nullptr,
-        nullptr,
-        &err);
-    std::string error;
-    if (err) {
-        error = err;
-        sqlite3_free(err);
-    }
-    if (ret != SQLITE_OK) {
-        throw DatabaseException("", sl->getError(query, error, db, ret));
-    }
-    if (getLastInsertIdFlag)
-        lastInsertId = sqlite3_last_insert_rowid(db);
-    contamination = true;
-}
-
-/* SLBackupTask */
-SLBackupTask::SLBackupTask(std::shared_ptr<Config> config, bool restore)
-    : config(std::move(config))
-    , restore(restore)
-    , dbFile(this->config->getOption(CFG_SERVER_STORAGE_SQLITE_DATABASE_FILE))
-    , dbBackupFile(fmt::format(DB_BACKUP_FORMAT, this->config->getOption(CFG_SERVER_STORAGE_SQLITE_DATABASE_FILE)))
-{
-}
-
-void SLBackupTask::run(sqlite3*& db, Sqlite3Database* sl)
-{
-    log_debug("Running: backup");
-
-    if (!restore) {
-        try {
-            fs::copy(
-                dbFile.getPath(),
-                dbBackupFile.getPath(),
-                fs::copy_options::overwrite_existing);
-            log_debug("sqlite3 backup successful");
-            dbBackupFile.setPermissions();
-            decontamination = true;
-        } catch (const std::runtime_error& e) {
-            log_error("error while making sqlite3 backup: {}", e.what());
-        }
-    } else {
-        log_info("trying to restore sqlite3 database from backup...");
-        sqlite3_close(db);
-        try {
-            fs::copy(
-                dbBackupFile.getPath(),
-                dbFile.getPath(),
-                fs::copy_options::overwrite_existing);
-            dbFile.setPermissions();
-        } catch (const std::runtime_error& e) {
-            throw DatabaseException(fmt::format("Error while restoring sqlite3 backup: {}", e.what()), fmt::format("Error while restoring sqlite3 backup: {}", e.what()));
-        }
-        int res = sqlite3_open(dbFile.getPath().c_str(), &db);
-        if (res != SQLITE_OK) {
-            throw DatabaseException("", "error while restoring sqlite3 backup: could not reopen sqlite3 database after restore");
-        }
-        log_info("sqlite3 database successfully restored from backup.");
-    }
-}
-
 /* Sqlite3Row */
 
 Sqlite3Row::Sqlite3Row(char** row)
@@ -597,11 +506,3 @@ std::unique_ptr<SQLRow> Sqlite3Result::nextRow()
     }
     return nullptr;
 }
-
-/* Sqlite3BackupTimerSubscriber */
-
-void Sqlite3Database::timerNotify(const std::shared_ptr<Timer::Parameter>& param)
-{
-    auto btask = std::make_shared<SLBackupTask>(config, false);
-    this->addTask(btask, true);
-}
