@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -32,9 +34,11 @@
 #ifndef __EXCEPTIONS_H__
 #define __EXCEPTIONS_H__
 
+#include <fmt/core.h>
 #include <stdexcept>
 #include <string>
 
+#define LINE_MESSAGE fmt::format("[{}:{}] {}", __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define throw_std_runtime_error(...) throw std::runtime_error(fmt::format("[{}:{}] {} Error: {}", __FILE__, __LINE__, __PRETTY_FUNCTION__, fmt::format(__VA_ARGS__)))
 #define throw_fmt_system_error(...) throw fmt::system_error(errno, "[{}:{}] {} Error: {}", __FILE__, __LINE__, __PRETTY_FUNCTION__, fmt::format(__VA_ARGS__))
 
@@ -60,6 +64,10 @@ public:
     std::string getUserMessage() const { return (!userMessage.empty() ? userMessage : what()); }
 };
 
+class SearchParseException : public DatabaseException {
+    using DatabaseException::DatabaseException;
+};
+
 class ObjectNotFoundException : public DatabaseException {
 public:
     explicit ObjectNotFoundException(const std::string& message);
