@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -28,10 +30,12 @@
 */
 
 /// \file buffered_io_handler.cc
+#define GRB_LOG_FAC GrbLogFacility::iohandler
 
 #include "buffered_io_handler.h" // API
 
-#include "util/tools.h"
+#include "exceptions.h"
+#include "util/grb_time.h"
 
 class Config;
 
@@ -49,6 +53,11 @@ BufferedIOHandler::BufferedIOHandler(const std::shared_ptr<Config>& config, std:
     // seekEnabled = true;
 }
 
+BufferedIOHandler::~BufferedIOHandler()
+{
+    BufferedIOHandler::close();
+}
+
 void BufferedIOHandler::open(enum UpnpOpenFileMode mode)
 {
     // do the open here instead of threadProc() because it may throw an exception
@@ -58,7 +67,8 @@ void BufferedIOHandler::open(enum UpnpOpenFileMode mode)
 
 void BufferedIOHandler::close()
 {
-    IOHandlerBufferHelper::close();
+    if (isOpen)
+        IOHandlerBufferHelper::close();
     // do the close here instead of threadProc() because it may throw an exception
     underlyingHandler->close();
 }
@@ -78,15 +88,14 @@ void BufferedIOHandler::threadProc()
     do {
 #ifdef TOMBDEBUG
         if (firstLog || getDeltaMillis(lastLog) > std::chrono::milliseconds(100)) {
-            if (firstLog)
-                firstLog = false;
+            firstLog = false;
             lastLog = currentTimeMS();
             [[maybe_unused]] float percentFillLevel = 0;
             if (!empty) {
-                auto currentFillSize = int(b - a);
+                auto currentFillSize = static_cast<int>(b - a);
                 if (currentFillSize <= 0)
                     currentFillSize += bufSize;
-                percentFillLevel = (float(currentFillSize) / float(bufSize)) * 100;
+                percentFillLevel = (static_cast<float>(currentFillSize) / static_cast<float>(bufSize)) * 100;
             }
             log_debug("buffer fill level: {:03.2f}%  (bufSize: {}; a: {}; b: {})", percentFillLevel, bufSize, a, b);
         }
@@ -95,11 +104,11 @@ void BufferedIOHandler::threadProc()
             a = b = 0;
 
         if (doSeek && !empty && (seekWhence == SEEK_SET || (seekWhence == SEEK_CUR && seekOffset > 0))) {
-            auto currentFillSize = int(b - a);
+            auto currentFillSize = static_cast<int>(b - a);
             if (currentFillSize <= 0)
                 currentFillSize += bufSize;
 
-            auto relSeek = int(seekOffset);
+            auto relSeek = static_cast<int>(seekOffset);
             if (seekWhence == SEEK_SET)
                 relSeek -= posRead;
 
@@ -145,7 +154,7 @@ void BufferedIOHandler::threadProc()
         } else {
             lock.unlock();
             std::size_t chunkSize = (maxChunkSize > maxWrite ? maxWrite : maxChunkSize);
-            readBytes = underlyingHandler->read(buffer + int(b), chunkSize);
+            readBytes = underlyingHandler->read(buffer + static_cast<int>(b), chunkSize);
             lock.lock();
             if (readBytes > 0) {
                 b += readBytes;
@@ -157,10 +166,10 @@ void BufferedIOHandler::threadProc()
                     threadRunner->notify();
                 }
                 if (waitForInitialFillSize) {
-                    auto currentFillSize = int(b - a);
+                    auto currentFillSize = static_cast<int>(b - a);
                     if (currentFillSize <= 0)
                         currentFillSize += bufSize;
-                    if (std::size_t(currentFillSize) >= initialFillSize) {
+                    if (static_cast<std::size_t>(currentFillSize) >= initialFillSize) {
                         log_debug("buffer: initial fillsize reached");
                         waitForInitialFillSize = false;
                         threadRunner->notify();
