@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -32,18 +34,18 @@
 #ifndef __BUFFERED_IO_HANDLER_H__
 #define __BUFFERED_IO_HANDLER_H__
 
+#include "io_handler_buffer_helper.h"
+
 #include <memory>
 #include <upnp.h>
 
-#include "common.h"
-#include "io_handler_buffer_helper.h"
-
 /// \brief a IOHandler with buffer support
 /// the buffer is only for read(). write() is not supported
 /// the public functions of this class are *not* thread safe!
 class BufferedIOHandler : public IOHandlerBufferHelper {
 public:
     /// \brief get an instance of a BufferedIOHandler
+    /// \param config Access configuration
     /// \param underlyingHandler the IOHandler from which the buffer should read
     /// \param bufSize the size of the buffer in bytes
     /// \param maxChunkSize the maximum size of the chunks which are read by the buffer
@@ -51,6 +53,7 @@ public:
     /// before the first read at the very beginning or after a seek returns;
     /// 0 disables the delay
     BufferedIOHandler(const std::shared_ptr<Config>& config, std::unique_ptr<IOHandler> underlyingHandler, std::size_t bufSize, std::size_t maxChunkSize, std::size_t initialFillSize);
+    ~BufferedIOHandler() noexcept override;
 
     void open(enum UpnpOpenFileMode mode) override;
     void close() override;
