@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -28,32 +30,37 @@
 */
 
 /// \file curl_io_handler.cc
+#define GRB_LOG_FAC GrbLogFacility::curl
 
 #ifdef HAVE_CURL
 #include "curl_io_handler.h" // API
 
-#include "config/config_manager.h"
+#include "config/config.h"
+#include "exceptions.h"
 #include "util/tools.h"
 
-CurlIOHandler::CurlIOHandler(const std::shared_ptr<Config>& config, const std::string& url, CURL* curlHandle, std::size_t bufSize, std::size_t initialFillSize)
+CurlIOHandler::CurlIOHandler(const std::shared_ptr<Config>& config, const std::string& url, std::size_t bufSize, std::size_t initialFillSize)
     : IOHandlerBufferHelper(config, bufSize, initialFillSize)
+    , URL(url)
+    , external_curl_handle(false)
+    , curl_handle(nullptr)
 {
-    if (url.empty())
+    if (this->URL.empty())
         throw_std_runtime_error("URL has not been set correctly");
     if (bufSize < CURL_MAX_WRITE_SIZE)
         throw_std_runtime_error("bufSize must be at least CURL_MAX_WRITE_SIZE({})", CURL_MAX_WRITE_SIZE);
 
-    this->URL = url;
-    this->external_curl_handle = curlHandle;
-    this->curl_handle = curlHandle;
-    // bytesCurl = 0;
     signalAfterEveryRead = true;
 
-    // still todo:
-    // * optimize seek if data already in buffer
+    // still todo: optimize seek if data already in buffer
     seekEnabled = true;
 }
 
+CurlIOHandler::~CurlIOHandler()
+{
+    CurlIOHandler::close();
+}
+
 void CurlIOHandler::open(enum UpnpOpenFileMode mode)
 {
     if (!curl_handle) {
@@ -68,9 +75,10 @@ void CurlIOHandler::open(enum UpnpOpenFileMode mode)
 
 void CurlIOHandler::close()
 {
-    IOHandlerBufferHelper::close();
+    if (isOpen)
+        IOHandlerBufferHelper::close();
 
-    if (external_curl_handle && curl_handle)
+    if (!external_curl_handle && curl_handle)
         curl_easy_cleanup(curl_handle);
 }
 
@@ -82,24 +90,27 @@ void CurlIOHandler::threadProc()
     assert(curl_handle);
     assert(!URL.empty());
 
-    // char error_buffer[CURL_ERROR_SIZE] = {'\0'};
-    // curl_easy_setopt(curl_handle, CURLOPT_ERRORBUFFER, error_buffer);
-
     curl_easy_setopt(curl_handle, CURLOPT_URL, URL.c_str());
     curl_easy_setopt(curl_handle, CURLOPT_NOSIGNAL, 1);
     curl_easy_setopt(curl_handle, CURLOPT_FOLLOWLOCATION, 1);
     curl_easy_setopt(curl_handle, CURLOPT_MAXREDIRS, -1);
 
-    bool logEnabled;
-#ifdef TOMBDEBUG
-    logEnabled = !ConfigManager::isDebugLogging();
+#ifdef GRBDEBUG
+    bool logEnabled = GrbLogger::Logger.isDebugging(GRB_LOG_FAC) || GrbLogger::Logger.isDebugLogging();
 #else
-    logEnabled = ConfigManager::isDebugLogging();
+#ifdef TOMBDEBUG
+    bool logEnabled = !GrbLogger::Logger.isDebugLogging();
+#else
+    bool logEnabled = GrbLogger::Logger.isDebugLogging();
 #endif
-    if (logEnabled)
+#endif
+    char errorBuffer[CURL_ERROR_SIZE] = { '\0' };
+    if (logEnabled) {
+        curl_easy_setopt(curl_handle, CURLOPT_ERRORBUFFER, errorBuffer);
         curl_easy_setopt(curl_handle, CURLOPT_VERBOSE, 1);
+    }
 
-    // curl_easy_setopt(curl_handle, CURLOPT_CONNECTTIMEOUT,
+    curl_easy_setopt(curl_handle, CURLOPT_CONNECTTIMEOUT, 20); // seconds
 
     // proxy..
 
@@ -133,23 +144,29 @@ void CurlIOHandler::threadProc()
         res = curl_easy_perform(curl_handle);
     } while (doSeek);
 
-    if (res != CURLE_OK)
+    if (res != CURLE_OK) {
         readError = true;
-    else
+        if (logEnabled) {
+            std::size_t len = strlen(errorBuffer);
+            if (len) {
+                log_error("libcurl (error {}): {}", res, errorBuffer);
+            } else
+                log_error("libcurl (error {}): {}", res, curl_easy_strerror(res));
+        }
+    } else
         eof = true;
 
     threadRunner->notify();
 }
 
-std::size_t CurlIOHandler::curlCallback(void* ptr, std::size_t size, std::size_t nmemb, void* data)
+std::size_t CurlIOHandler::curlCallback(void* ptr, std::size_t size, std::size_t nmemb, CurlIOHandler* ego)
 {
-    auto ego = static_cast<CurlIOHandler*>(data);
     std::size_t wantWrite = size * nmemb;
 
     assert(wantWrite <= ego->bufSize);
-    auto& threadRunner = ego->threadRunner;
+    auto&& threadRunner = ego->threadRunner;
 
-    // log_debug("URL: {}; size: {}; nmemb: {}; wantWrite: {}", ego->URL.c_str(), size, nmemb, wantWrite);
+    // log_debug "URL: {}; size: {}; nmemb: {}; wantWrite: {}", ego->URL.c_str(), size, nmemb, wantWrite
 
     auto lock = threadRunner->uniqueLock();
 
@@ -158,11 +175,11 @@ std::size_t CurlIOHandler::curlCallback(void* ptr, std::size_t size, std::size_t
     int bufFree;
     do {
         if (ego->doSeek && !ego->empty && (ego->seekWhence == SEEK_SET || (ego->seekWhence == SEEK_CUR && ego->seekOffset > 0))) {
-            auto currentFillSize = int(ego->b - ego->a);
+            auto currentFillSize = static_cast<int>(ego->b - ego->a);
             if (currentFillSize <= 0)
                 currentFillSize += ego->bufSize;
 
-            auto relSeek = int(ego->seekOffset);
+            auto relSeek = static_cast<int>(ego->seekOffset);
             if (ego->seekWhence == SEEK_SET)
                 relSeek -= ego->posRead;
 
@@ -210,7 +227,7 @@ std::size_t CurlIOHandler::curlCallback(void* ptr, std::size_t size, std::size_t
             if (bufFree < 0)
                 bufFree += ego->bufSize;
         }
-    } while (std::size_t(bufFree) < wantWrite);
+    } while (static_cast<std::size_t>(bufFree) < wantWrite);
 
     std::size_t maxWrite = (ego->empty ? ego->bufSize : (ego->a < ego->b ? ego->bufSize - ego->b : ego->a - ego->b));
     std::size_t write1 = (wantWrite > maxWrite ? maxWrite : wantWrite);
@@ -226,7 +243,9 @@ std::size_t CurlIOHandler::curlCallback(void* ptr, std::size_t size, std::size_t
 
     lock.lock();
 
-    // ego->bytesCurl += wantWrite;
+#if 0
+    ego->bytesCurl += wantWrite;
+#endif
     ego->b += wantWrite;
     if (ego->b >= ego->bufSize)
         ego->b -= ego->bufSize;
@@ -235,10 +254,10 @@ std::size_t CurlIOHandler::curlCallback(void* ptr, std::size_t size, std::size_t
         threadRunner->notify();
     }
     if (ego->waitForInitialFillSize) {
-        auto currentFillSize = int(ego->b - ego->a);
+        auto currentFillSize = static_cast<int>(ego->b - ego->a);
         if (currentFillSize <= 0)
             currentFillSize += ego->bufSize;
-        if (std::size_t(currentFillSize) >= ego->initialFillSize) {
+        if (static_cast<std::size_t>(currentFillSize) >= ego->initialFillSize) {
             log_debug("buffer: initial fillsize reached");
             ego->waitForInitialFillSize = false;
             threadRunner->notify();
