@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -28,10 +30,11 @@
 */
 
 /// \file file_io_handler.cc
+#define GRB_LOG_FAC GrbLogFacility::iohandler
 
 #include "file_io_handler.h" // API
 
-#include "cds_objects.h"
+#include "exceptions.h"
 
 FileIOHandler::FileIOHandler(const fs::path& filename)
     : file(filename)
@@ -46,7 +49,7 @@ void FileIOHandler::open(enum UpnpOpenFileMode mode)
     f = file.open("rb");
 }
 
-std::size_t FileIOHandler::read(std::byte* buf, std::size_t length)
+grb_read_t FileIOHandler::read(std::byte* buf, std::size_t length)
 {
     std::size_t ret = std::fread(buf, sizeof(std::byte), length, f);
 
