@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -32,9 +34,10 @@
 #ifndef __MEM_IO_HANDLER_H__
 #define __MEM_IO_HANDLER_H__
 
-#include "common.h"
 #include "io_handler.h"
 
+#include <string>
+
 /// \brief Allows the web server to read from a memory buffer instead of a file.
 class MemIOHandler : public IOHandler {
 protected:
@@ -48,6 +51,7 @@ protected:
 public:
     /// \brief Initializes the internal buffer.
     /// \param buffer all operations will be done on this buffer.
+    /// \param length size of the buffer
     MemIOHandler(const void* buffer, int length);
     explicit MemIOHandler(const std::string& str);
     ~MemIOHandler() override;
@@ -57,7 +61,7 @@ public:
 
     ///
     void open(enum UpnpOpenFileMode mode) override;
-    std::size_t read(std::byte* buf, std::size_t length) override;
+    grb_read_t read(std::byte* buf, std::size_t length) override;
     void seek(off_t offset, int whence) override;
     off_t tell() override;
     void close() override;
