@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -28,16 +30,19 @@
 */
 
 /// \file process_io_handler.cc
+#define GRB_LOG_FAC GrbLogFacility::iohandler
 
 #include "process_io_handler.h" // API
 
+#include "content/content.h"
+#include "exceptions.h"
+
 #include <csignal>
 #include <utility>
 
 #include <fcntl.h>
 #include <sys/select.h>
-
-#include "content/content_manager.h"
+#include <unistd.h>
 
 // after MAX_TIMEOUTS we will tell libupnp to check the socket,
 // this will make sure that we do not block the read and allow libupnp to
@@ -101,12 +106,12 @@ void ProcessIOHandler::unregisterAll()
     }
 }
 
-ProcessIOHandler::ProcessIOHandler(std::shared_ptr<ContentManager> content,
+ProcessIOHandler::ProcessIOHandler(const std::shared_ptr<Content>& content,
     fs::path filename,
     std::shared_ptr<Executor> mainProc,
     std::vector<std::unique_ptr<ProcListItem>> procList,
     bool ignoreSeek)
-    : content(std::move(content))
+    : content(content)
     , procList(std::move(procList))
     , mainProc(std::move(mainProc))
     , path(std::move(filename))
@@ -154,11 +159,10 @@ void ProcessIOHandler::open(enum UpnpOpenFileMode mode)
     }
 }
 
-std::size_t ProcessIOHandler::read(std::byte* buf, std::size_t length)
+grb_read_t ProcessIOHandler::read(std::byte* buf, std::size_t length)
 {
     fd_set readSet;
     struct timespec timeout = { FIFO_READ_TIMEOUT, 0 };
-    ssize_t bytesRead;
     std::size_t numBytes = 0;
     auto pBuffer = buf;
     int exitStatus = EXIT_SUCCESS;
@@ -202,7 +206,7 @@ std::size_t ProcessIOHandler::read(std::byte* buf, std::size_t length)
 
         if (FD_ISSET(fd, &readSet)) {
             timeoutCount = 0;
-            bytesRead = ::read(fd, pBuffer, length);
+            ssize_t bytesRead = ::read(fd, pBuffer, length);
             if (bytesRead == 0)
                 break;
 
