@@ -4,7 +4,7 @@ Gerbera - https://gerbera.io/
 
     ffmpeg_thumbnailer_handler.cc - this file is part of Gerbera.
 
-    Copyright (C) 2022 Gerbera Contributors
+    Copyright (C) 2022-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -22,20 +22,30 @@ Gerbera - https://gerbera.io/
 /// \file ffmpeg_thumbnailer_handler.cc
 
 #ifdef HAVE_FFMPEGTHUMBNAILER
+#define GRB_LOG_FAC GrbLogFacility::ffmpeg
 
-#include "ffmpeg_thumbnailer_handler.h"
+#include "ffmpeg_thumbnailer_handler.h" // API
+
+#include "cds/cds_item.h"
+#include "config/config_val.h"
+#include "iohandler/mem_io_handler.h"
 #include "resolution.h"
+#include "util/tools.h"
+
 #include <libffmpegthumbnailer/filmstripfilter.h>
 #include <libffmpegthumbnailer/videothumbnailer.h>
 
-fs::path FfmpegThumbnailerHandler::getThumbnailCacheBasePath() const
+static void ffmpegThLogger(ThumbnailerLogLevel logLevel, const std::string message)
 {
-    auto configuredDir = config->getOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_CACHE_DIR);
-    if (!configuredDir.empty())
-        return configuredDir;
-
-    auto home = config->getOption(CFG_SERVER_HOME);
-    return fs::path(home) / "cache-dir";
+    switch (logLevel) {
+    case ThumbnailerLogLevelError:
+        log_error("FfmpegThumbnailerHandler: {}", message);
+        break;
+    case ThumbnailerLogLevelInfo:
+    default:
+        log_debug("FfmpegThumbnailerHandler: {}", message);
+        break;
+    }
 }
 
 fs::path FfmpegThumbnailerHandler::getThumbnailCachePath(const fs::path& base, const fs::path& movie)
@@ -64,16 +74,16 @@ void FfmpegThumbnailerHandler::writeThumbnailCacheFile(const fs::path& movieFile
     }
 }
 
-std::unique_ptr<IOHandler> FfmpegThumbnailerHandler::serveContent(const std::shared_ptr<CdsObject>& obj, int resNum)
+std::unique_ptr<IOHandler> FfmpegThumbnailerHandler::serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource)
 {
-    if (!enabled)
+    if (!isEnabled)
         return nullptr;
 
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
     if (!item)
         return nullptr;
 
-    auto cacheEnabled = config->getBoolOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_CACHE_DIR_ENABLED);
+    auto cacheEnabled = config->getBoolOption(ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_CACHE_DIR_ENABLED);
     if (cacheEnabled) {
         auto data = readThumbnailCacheFile(item->getLocation());
         if (data) {
@@ -85,15 +95,16 @@ std::unique_ptr<IOHandler> FfmpegThumbnailerHandler::serveContent(const std::sha
     try {
         auto thumbLock = std::scoped_lock<std::mutex>(thumb_mutex);
 
-        auto th = ffmpegthumbnailer::VideoThumbnailer(config->getIntOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_THUMBSIZE), false, true, config->getIntOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_IMAGE_QUALITY), false);
-        std::vector<uint8_t> img;
+        auto th = ffmpegthumbnailer::VideoThumbnailer(config->getIntOption(ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_THUMBSIZE), false, true, config->getIntOption(ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_IMAGE_QUALITY), false);
 
-        th.setSeekPercentage(config->getIntOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_SEEK_PERCENTAGE));
-        if (config->getBoolOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_FILMSTRIP_OVERLAY))
+        th.setLogCallback(ffmpegThLogger);
+        th.setSeekPercentage(config->getIntOption(ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_SEEK_PERCENTAGE));
+        if (config->getBoolOption(ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_FILMSTRIP_OVERLAY))
             th.addFilter(new ffmpegthumbnailer::FilmStripFilter());
 
         log_debug("Generating thumbnail for file: {}", item->getLocation().c_str());
 
+        std::vector<uint8_t> img;
         th.generateThumbnail(item->getLocation().c_str(), Jpeg, img);
         if (cacheEnabled) {
             writeThumbnailCacheFile(item->getLocation(), reinterpret_cast<std::byte*>(img.data()), img.size());
@@ -108,7 +119,7 @@ std::unique_ptr<IOHandler> FfmpegThumbnailerHandler::serveContent(const std::sha
 
 void FfmpegThumbnailerHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 {
-    if (!enabled)
+    if (!isEnabled)
         return;
 
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
@@ -116,21 +127,21 @@ void FfmpegThumbnailerHandler::fillMetadata(const std::shared_ptr<CdsObject>& ob
         return;
 
     try {
-        std::string videoResolution = item->getResource(ContentHandler::DEFAULT)->getAttribute(CdsResource::Attribute::RESOLUTION);
+        std::string videoResolution = item->getResource(ContentHandler::DEFAULT)->getAttribute(ResourceAttribute::RESOLUTION);
         if (videoResolution.empty())
             return;
         auto resolution = Resolution(videoResolution);
-        auto mappings = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
+        auto mappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
 
         auto it = mappings.find(CONTENT_TYPE_JPG);
         std::string thumbMimetype = it != mappings.end() && !it->second.empty() ? it->second : "image/jpeg";
 
-        auto thumbResource = std::make_shared<CdsResource>(ContentHandler::FFTH, CdsResource::Purpose::Thumbnail);
-        thumbResource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo(thumbMimetype));
+        auto thumbResource = std::make_shared<CdsResource>(ContentHandler::FFTH, ResourcePurpose::Thumbnail);
+        thumbResource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo(thumbMimetype));
 
-        auto y = config->getIntOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_THUMBSIZE) * resolution.y() / resolution.x();
-        auto x = config->getIntOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_THUMBSIZE);
-        thumbResource->addAttribute(CdsResource::Attribute::RESOLUTION, fmt::format("{}x{}", x, y));
+        auto y = config->getIntOption(ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_THUMBSIZE) * resolution.y() / resolution.x();
+        auto x = config->getIntOption(ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_THUMBSIZE);
+        thumbResource->addAttribute(ResourceAttribute::RESOLUTION, fmt::format("{}x{}", x, y));
         item->addResource(thumbResource);
         log_debug("Adding resource for video thumbnail");
 
@@ -139,10 +150,17 @@ void FfmpegThumbnailerHandler::fillMetadata(const std::shared_ptr<CdsObject>& ob
     }
 }
 
-FfmpegThumbnailerHandler::FfmpegThumbnailerHandler(const std::shared_ptr<Context>& context, config_option_t checkOption)
-    : MetadataHandler(context)
-    , enabled(config->getBoolOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_ENABLED) && config->getBoolOption(checkOption))
+FfmpegThumbnailerHandler::FfmpegThumbnailerHandler(const std::shared_ptr<Context>& context, ConfigVal checkOption)
+    : MediaMetadataHandler(context, ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_ENABLED)
 {
+    isEnabled = this->isEnabled && config->getBoolOption(checkOption);
+    auto configuredDir = config->getOption(ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_CACHE_DIR);
+    if (!configuredDir.empty()) {
+        cachePath = configuredDir;
+    } else {
+        auto home = config->getOption(ConfigVal::SERVER_HOME);
+        cachePath = fs::path(home) / "cache-dir";
+    }
 }
 
 #endif
