@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -30,33 +32,36 @@
 /// \file libexif_handler.cc
 
 #ifdef HAVE_LIBEXIF
+#define GRB_LOG_FAC GrbLogFacility::exif
 #include "libexif_handler.h" // API
 
-#include <array>
-
-#include "cds_objects.h"
+#include "cds/cds_item.h"
+#include "config/config_val.h"
+#include "exceptions.h"
 #include "iohandler/file_io_handler.h"
 #include "iohandler/mem_io_handler.h"
 #include "util/jpeg_resolution.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
 
+#include <array>
+
 /// \brief Sets resolution for a given resource index, item must be a JPEG image
-static void setJpegResolutionResource(const std::shared_ptr<CdsItem>& item, std::size_t resNum)
+static void setJpegResolutionResource(const std::shared_ptr<CdsItem>& item, std::size_t resNum = 0)
 {
+    if (resNum >= item->getResourceCount()) {
+        log_warning("Invalid resource {} index {}", item->getLocation().c_str(), resNum);
+        return;
+    }
     try {
         auto fioH = FileIOHandler(item->getLocation());
         fioH.open(UPNP_READ);
         auto resolution = getJpegResolution(fioH);
 
-        if (resNum >= item->getResourceCount()) {
-            log_warning("Invalid resource index");
-            return;
-        }
-        item->getResource(resNum)->addAttribute(CdsResource::Attribute::RESOLUTION, resolution.string());
+        item->getResource(resNum)->addAttribute(ResourceAttribute::RESOLUTION, resolution.string());
 
     } catch (const std::runtime_error& e) {
-        log_error("Exception! {}", e.what());
+        log_error("Failed to parse EXIF {} resolution: {}", item->getLocation().c_str(), e.what());
     }
 }
 
@@ -152,9 +157,9 @@ static const std::map<std::string, int> exifTagMap {
     { "EXIF_TAG_SCENE_TYPE", EXIF_TAG_SCENE_TYPE },
     { "EXIF_TAG_NEW_CFA_PATTERN", EXIF_TAG_NEW_CFA_PATTERN },
     { "EXIF_TAG_CUSTOM_RENDERED", EXIF_TAG_CUSTOM_RENDERED },
-    { "EXIF_TAG_EXPOSURE_MODE", EXIF_TAG_CUSTOM_RENDERED },
-    { "EXIF_TAG_WHITE_BALANCE", EXIF_TAG_CUSTOM_RENDERED },
-    { "EXIF_TAG_DIGITAL_ZOOM_RATIO", EXIF_TAG_CUSTOM_RENDERED },
+    { "EXIF_TAG_EXPOSURE_MODE", EXIF_TAG_EXPOSURE_MODE },
+    { "EXIF_TAG_WHITE_BALANCE", EXIF_TAG_WHITE_BALANCE },
+    { "EXIF_TAG_DIGITAL_ZOOM_RATIO", EXIF_TAG_DIGITAL_ZOOM_RATIO },
     { "EXIF_TAG_FOCAL_LENGTH_IN_35MM_FILM", EXIF_TAG_FOCAL_LENGTH_IN_35MM_FILM },
     { "EXIF_TAG_SCENE_CAPTURE_TYPE", EXIF_TAG_SCENE_CAPTURE_TYPE },
     { "EXIF_TAG_GAIN_CONTROL", EXIF_TAG_GAIN_CONTROL },
@@ -162,7 +167,7 @@ static const std::map<std::string, int> exifTagMap {
     { "EXIF_TAG_SATURATION", EXIF_TAG_SATURATION },
     { "EXIF_TAG_SHARPNESS", EXIF_TAG_SHARPNESS },
     { "EXIF_TAG_DEVICE_SETTING_DESCRIPTION", EXIF_TAG_DEVICE_SETTING_DESCRIPTION },
-    { "EXIF_TAG_SUBJECT_DISTANCE_RANGE", EXIF_TAG_DEVICE_SETTING_DESCRIPTION },
+    { "EXIF_TAG_SUBJECT_DISTANCE_RANGE", EXIF_TAG_SUBJECT_DISTANCE_RANGE },
     { "EXIF_TAG_IMAGE_UNIQUE_ID", EXIF_TAG_IMAGE_UNIQUE_ID },
 };
 
@@ -175,8 +180,33 @@ static int getTagFromString(const std::string& tag)
     return result;
 }
 
+static void logfunc(ExifLog* log, ExifLogCode code, const char* domain, const char* format, va_list args, void* data)
+{
+    va_list args_copy;
+    va_copy(args_copy, args);
+    size_t len = std::vsnprintf(nullptr, 0, format, args_copy);
+    std::vector<char> buf(len + 1); // need space for NUL
+    std::vsnprintf(buf.data(), len + 1, format, args);
+    std::string message = buf.data();
+    log_debug("Exif: {}-{}, domain {}: {}", exif_log_code_get_title(code), exif_log_code_get_message(code), domain, message);
+}
+
+LibExifHandler::LibExifHandler(const std::shared_ptr<Context>& context)
+    : MediaMetadataHandler(context, ConfigVal::IMPORT_LIBOPTS_EXIF_ENABLED, ConfigVal::IMPORT_LIBOPTS_EXIF_METADATA_TAGS_LIST, ConfigVal::IMPORT_LIBOPTS_EXIF_AUXDATA_TAGS_LIST)
+{
+    log = exif_log_new();
+    exif_log_set_func(log, logfunc, nullptr);
+}
+
+LibExifHandler::~LibExifHandler()
+{
+    exif_log_free(log);
+}
+
 void LibExifHandler::process_ifd(const ExifContent* content, const std::shared_ptr<CdsItem>& item,
-    const std::unique_ptr<StringConverter>& sc, const std::vector<std::string>& auxtags, const std::map<std::string, std::string>& metatags)
+    const std::shared_ptr<StringConverter>& sc,
+    std::string& imageX, std::string& imageY,
+    const std::vector<std::string>& auxtags, const std::map<std::string, std::string>& metatags)
 {
     constexpr auto BUFLEN = 4096;
     std::array<char, BUFLEN> exif_entry_buffer;
@@ -194,9 +224,9 @@ void LibExifHandler::process_ifd(const ExifContent* content, const std::shared_p
                 // convert date to ISO 8601 as required in the UPnP spec
                 // from YYYY:MM:DD to YYYY-MM-DD
                 if (value.length() >= 19) {
-                    item->addMetaData(M_DATE, fmt::format("{}-{}-{}T{}", value.substr(0, 4), value.substr(5, 2), value.substr(8, 2), value.substr(11, 8)));
+                    item->addMetaData(MetadataFields::M_DATE, fmt::format("{}-{}-{}T{}", value.substr(0, 4), value.substr(5, 2), value.substr(8, 2), value.substr(11, 8)));
                 } else if (value.length() >= 11) {
-                    item->addMetaData(M_DATE, fmt::format("{}-{}-{}", value.substr(0, 4), value.substr(5, 2), value.substr(8, 2)));
+                    item->addMetaData(MetadataFields::M_DATE, fmt::format("{}-{}-{}", value.substr(0, 4), value.substr(5, 2), value.substr(8, 2)));
                 }
             }
             break;
@@ -205,7 +235,7 @@ void LibExifHandler::process_ifd(const ExifContent* content, const std::shared_p
             auto value = trimString(exif_egv(entry));
             log_debug("Found exif comment: {}", value);
             if (!value.empty()) {
-                item->addMetaData(M_DESCRIPTION, sc->convert(value));
+                item->addMetaData(MetadataFields::M_DESCRIPTION, sc->convert(value));
             }
             break;
         }
@@ -225,6 +255,13 @@ void LibExifHandler::process_ifd(const ExifContent* content, const std::shared_p
             }
             break;
         }
+        case EXIF_TAG_ORIENTATION: {
+            auto o = exif_data_get_byte_order(entry->parent->parent);
+            auto value = exif_get_short(entry->data, o);
+            log_debug("Found exif orientation {}", value);
+            item->getResource(ContentHandler::DEFAULT)->addAttribute(ResourceAttribute::ORIENTATION, fmt::format("{}", value));
+            break;
+        }
         default:
             break;
         }
@@ -257,72 +294,77 @@ void LibExifHandler::process_ifd(const ExifContent* content, const std::shared_p
 void LibExifHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 {
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
-    if (!item)
+    if (!item || !isEnabled)
         return;
 
-    auto sc = StringConverter::m2i(CFG_IMPORT_LIBOPTS_EXIF_CHARSET, item->getLocation(), config);
+    auto sc = converterManager->m2i(ConfigVal::IMPORT_LIBOPTS_EXIF_CHARSET, item->getLocation());
     ExifData* exifData = exif_data_new_from_file(item->getLocation().c_str());
 
-    if (exifData == nullptr) {
+    if (!exifData) {
         log_debug("Exif data not found, attempting to set resolution internally...");
-        setJpegResolutionResource(item, 0);
+        setJpegResolutionResource(item);
         return;
     }
 
-    auto aux = config->getArrayOption(CFG_IMPORT_LIBOPTS_EXIF_AUXDATA_TAGS_LIST);
-    auto meta = config->getDictionaryOption(CFG_IMPORT_LIBOPTS_EXIF_METADATA_TAGS_LIST);
+    // image resolution in pixels
+    // the problem is that I do not know when I encounter the
+    // tags for X and Y, so I have to save the information
+    // and in the very end - when I have both values - add the
+    // appropriate resource
+    std::string imageX;
+    std::string imageY;
+
     for (auto* exifContent : exifData->ifd) {
         if (exifContent != nullptr) {
-            process_ifd(exifContent, item, sc, aux, meta);
+            process_ifd(exifContent, item, sc, imageX, imageY, auxTags, metaTags);
         }
     }
 
     // we got the image resolution so we can add our resource
     if (!imageX.empty() && !imageY.empty()) {
-        item->getResource(ContentHandler::DEFAULT)->addAttribute(CdsResource::Attribute::RESOLUTION, fmt::format("{}x{}", imageX, imageY));
+        item->getResource(ContentHandler::DEFAULT)->addAttribute(ResourceAttribute::RESOLUTION, fmt::format("{}x{}", imageX, imageY));
     } else {
         log_debug("EXIF resolution not found, falling back to parsing directly: {}", item->getLocation().c_str());
-        setJpegResolutionResource(item, 0);
+        setJpegResolutionResource(item);
     }
 
     if (exifData->size != 0U) {
+        auto resource = std::make_shared<CdsResource>(ContentHandler::LIBEXIF, ResourcePurpose::Thumbnail);
+        resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo(item->getMimeType()));
+        item->addResource(resource);
         try {
             auto ioH = MemIOHandler(exifData->data, exifData->size);
             ioH.open(UPNP_READ);
             auto thResolution = getJpegResolution(ioH);
             log_debug("EXIF Thumb Resolution: {}", thResolution.string());
-            auto resource = std::make_shared<CdsResource>(ContentHandler::LIBEXIF, CdsResource::Purpose::Thumbnail);
-            resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo(item->getMimeType()));
-            resource->addAttribute(CdsResource::Attribute::RESOLUTION, thResolution.string());
-            item->addResource(resource);
-
+            resource->addAttribute(ResourceAttribute::RESOLUTION, thResolution.string());
         } catch (const std::runtime_error& e) {
-            log_error("Failed to parse EXIF Thumbnail details: {}", e.what());
+            log_error("Failed to parse EXIF Thumbnail {} details: {}", item->getLocation().c_str(), e.what());
         }
-    } // (ed->size)
+    } // (exifData->size)
     exif_data_unref(exifData);
 }
 
-std::unique_ptr<IOHandler> LibExifHandler::serveContent(const std::shared_ptr<CdsObject>& obj, int resNum)
+std::unique_ptr<IOHandler> LibExifHandler::serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource)
 {
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
     if (!item)
         return nullptr;
 
-    auto res = item->getResource(resNum);
+    if (resource->getPurpose() != ResourcePurpose::Thumbnail)
+        throw_std_runtime_error("Resource {} is not a Thumbnail", resource->getPurpose());
 
-    if (res->getPurpose() != CdsResource::Purpose::Thumbnail)
-        throw_std_runtime_error("Resource {} is not a Thumbnail", resNum);
+    ExifData* exifData = exif_data_new_from_file(item->getLocation().c_str());
+    if (!exifData)
+        throw_std_runtime_error("Resource {} has no exif information", item->getLocation().string());
 
-    ExifData* ed = exif_data_new_from_file(item->getLocation().c_str());
-    if (!ed)
-        throw_std_runtime_error("Resource {} has no exif information", resNum);
+    if (exifData->size == 0U) {
+        exif_data_unref(exifData);
+        throw_std_runtime_error("Resource {} has no exif thumbnail", item->getLocation().string());
+    }
 
-    if (!(ed->size))
-        throw_std_runtime_error("Resource {} has no exif thumbnail", resNum);
-
-    auto ioHandler = std::make_unique<MemIOHandler>(ed->data, ed->size);
-    exif_data_unref(ed);
+    auto ioHandler = std::make_unique<MemIOHandler>(exifData->data, exifData->size);
+    exif_data_unref(exifData);
     return ioHandler;
 }
 #endif // HAVE_LIBEXIF
