@@ -4,7 +4,7 @@ Gerbera - https://gerbera.io/
 
     resolution.h - this file is part of Gerbera.
 
-    Copyright (C) 2022 Gerbera Contributors
+    Copyright (C) 2022-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -36,8 +36,8 @@ public:
     [[nodiscard]] std::string string() const;
 
 private:
-    std::uint64_t _x;
-    std::uint64_t _y;
+    std::uint64_t _x {};
+    std::uint64_t _y {};
 };
 
 #endif // GERBERA_RESOLUTION_H
