@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -30,8 +32,21 @@
 /// \file taglib_handler.cc
 
 #ifdef HAVE_TAGLIB
+#define GRB_LOG_FAC GrbLogFacility::taglib
+
 #include "taglib_handler.h" // API
 
+#include "cds/cds_item.h"
+#include "config/config.h"
+#include "config/config_val.h"
+#include "exceptions.h"
+#include "iohandler/mem_io_handler.h"
+#include "metadata_enums.h"
+#include "util/grb_time.h"
+#include "util/mime.h"
+#include "util/string_converter.h"
+#include "util/tools.h"
+
 #include <aifffile.h>
 #include <apefile.h>
 #include <asffile.h>
@@ -43,6 +58,7 @@
 #include <oggflacfile.h>
 #include <opusfile.h>
 #include <speexfile.h>
+#include <tdebuglistener.h>
 #include <textidentificationframe.h>
 #include <tfilestream.h>
 #include <tiostream.h>
@@ -50,68 +66,83 @@
 #include <vorbisfile.h>
 #include <wavpackfile.h>
 
-#include "cds_objects.h"
-#include "config/config_manager.h"
-#include "iohandler/mem_io_handler.h"
-#include "util/mime.h"
-#include "util/tools.h"
+#if TAGLIB_MAJOR_VERSION >= 2
+#include <mp4itemfactory.h>
+#endif
+
+class GerberaTagLibDebugListener : public TagLib::DebugListener {
+private:
+    GerberaTagLibDebugListener()
+    {
+        // Install listener.
+        TagLib::setDebugListener(this);
+    }
+
+    virtual void printMessage(const TagLib::String& msg) override
+    {
+        // Remove trailing newlines.
+        log_debug("{}", msg.stripWhiteSpace().to8Bit(true));
+    }
+    static GerberaTagLibDebugListener grbListener;
+};
+
+GerberaTagLibDebugListener GerberaTagLibDebugListener::grbListener;
 
 TagLibHandler::TagLibHandler(const std::shared_ptr<Context>& context)
-    : MetadataHandler(context)
+    : MediaMetadataHandler(context, ConfigVal::IMPORT_LIBOPTS_ID3_ENABLED, ConfigVal::IMPORT_LIBOPTS_ID3_METADATA_TAGS_LIST, ConfigVal::IMPORT_LIBOPTS_ID3_AUXDATA_TAGS_LIST)
 {
-    entrySeparator = this->config->getOption(CFG_IMPORT_LIBOPTS_ENTRY_SEP);
-    legacyEntrySeparator = this->config->getOption(CFG_IMPORT_LIBOPTS_ENTRY_LEGACY_SEP);
+    entrySeparator = this->config->getOption(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP);
+    legacyEntrySeparator = this->config->getOption(ConfigVal::IMPORT_LIBOPTS_ENTRY_LEGACY_SEP);
     trimStringInPlace(legacyEntrySeparator);
     if (legacyEntrySeparator.empty())
         legacyEntrySeparator = "\n";
-    specialPropertyMap = this->config->getDictionaryOption(CFG_IMPORT_LIBOPTS_ID3_METADATA_TAGS_LIST);
 }
 
-void TagLibHandler::addField(metadata_fields_t field, const TagLib::File& file, const TagLib::Tag* tag, const std::shared_ptr<CdsItem>& item) const
+void TagLibHandler::addField(MetadataFields field, const TagLib::File& file, const TagLib::Tag* tag, const std::shared_ptr<CdsItem>& item) const
 {
     if (!tag || tag->isEmpty())
         return;
 
-    auto sc = StringConverter::i2i(config); // sure is sure
+    auto sc = converterManager->i2i(); // sure is sure
 
     std::vector<std::string> value;
 
-    auto propertyMap = std::map<metadata_fields_t, std::string> {
-        { M_ARTIST, "ARTIST" },
-        { M_ALBUMARTIST, "ALBUMARTIST" },
-        { M_COMPOSER, "COMPOSER" },
-        { M_CONDUCTOR, "CONDUCTOR" },
-        { M_GENRE, "GENRE" },
-        { M_ORCHESTRA, "ORCHESTRA" },
+    auto propertyMap = std::map<MetadataFields, std::string> {
+        { MetadataFields::M_ARTIST, "ARTIST" },
+        { MetadataFields::M_ALBUMARTIST, "ALBUMARTIST" },
+        { MetadataFields::M_COMPOSER, "COMPOSER" },
+        { MetadataFields::M_CONDUCTOR, "CONDUCTOR" },
+        { MetadataFields::M_GENRE, "GENRE" },
+        { MetadataFields::M_ORCHESTRA, "ORCHESTRA" },
     };
 
     switch (field) {
-    case M_TITLE:
+    case MetadataFields::M_TITLE:
         value.push_back(tag->title().to8Bit(true));
         break;
-    case M_ALBUM:
+    case MetadataFields::M_ALBUM:
         value.push_back(tag->album().to8Bit(true));
         break;
-    case M_DATE:
-    case M_UPNP_DATE: {
+    case MetadataFields::M_DATE:
+    case MetadataFields::M_UPNP_DATE: {
         unsigned int i = tag->year();
         if (i == 0)
             return;
         value.push_back(fmt::format("{}-01-01", i));
         break;
     }
-    case M_DESCRIPTION:
+    case MetadataFields::M_DESCRIPTION:
         value.push_back(tag->comment().to8Bit(true));
         break;
-    case M_TRACKNUMBER: {
+    case MetadataFields::M_TRACKNUMBER: {
         std::uint32_t i = tag->track();
-        if (i == 0 || i > std::uint32_t(std::numeric_limits<int>::max()))
+        if (i == 0 || i > static_cast<std::uint32_t>(std::numeric_limits<int>::max()))
             return;
         value.push_back(fmt::to_string(i));
         item->setTrackNumber(i);
         break;
     }
-    case M_PARTNUMBER: {
+    case MetadataFields::M_PARTNUMBER: {
         auto list = file.properties()["DISCNUMBER"];
         if (!list.isEmpty()) {
             value.push_back(list[0].to8Bit(true));
@@ -156,9 +187,9 @@ void TagLibHandler::addSpecialFields(const TagLib::File& file, const TagLib::Tag
     if (!tag || tag->isEmpty())
         return;
 
-    auto sc = StringConverter::i2i(config); // sure is sure
+    auto sc = converterManager->i2i(); // sure is sure
 
-    for (auto&& [key, meta] : specialPropertyMap) {
+    for (auto&& [key, meta] : metaTags) {
         auto list = file.properties()[key];
         if (list.isEmpty())
             return;
@@ -179,7 +210,7 @@ void TagLibHandler::populateGenericTags(const std::shared_ptr<CdsItem>& item, co
         return;
 
     const TagLib::Tag* tag = file.tag();
-    for (auto&& [field, key] : mt_keys)
+    for (auto&& [field, key] : MetaEnumMapper::mt_keys)
         addField(field, file, tag, item);
 
     addSpecialFields(file, tag, item);
@@ -193,29 +224,28 @@ void TagLibHandler::populateGenericTags(const std::shared_ptr<CdsItem>& item, co
     // UPnP bitrate is in bytes/second
     int temp = audioProps->bitrate() * 1024 / 8; // kbit/second -> byte/second
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITRATE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITRATE, fmt::to_string(temp));
     }
 
     temp = audioProps->lengthInMilliseconds();
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::DURATION, millisecondsToHMSF(temp));
+        res->addAttribute(ResourceAttribute::DURATION, millisecondsToHMSF(temp));
     }
 
     temp = audioProps->sampleRate();
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::SAMPLEFREQUENCY, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::SAMPLEFREQUENCY, fmt::to_string(temp));
     }
 
     temp = audioProps->channels();
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::NRAUDIOCHANNELS, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::NRAUDIOCHANNELS, fmt::to_string(temp));
     }
 }
 
-void TagLibHandler::populateAuxTags(const std::shared_ptr<CdsItem>& item, const TagLib::PropertyMap& propertyMap, const std::unique_ptr<StringConverter>& sc) const
+void TagLibHandler::populateAuxTags(const std::shared_ptr<CdsItem>& item, const TagLib::PropertyMap& propertyMap, const std::shared_ptr<StringConverter>& sc) const
 {
-    auto auxTagsList = config->getArrayOption(CFG_IMPORT_LIBOPTS_ID3_AUXDATA_TAGS_LIST);
-    for (auto&& desiredTag : auxTagsList) {
+    for (auto&& desiredTag : auxTags) {
         if (desiredTag.empty()) {
             continue;
         }
@@ -241,10 +271,10 @@ void TagLibHandler::populateAuxTags(const std::shared_ptr<CdsItem>& item, const
 void TagLibHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 {
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
-    if (!item)
+    if (!item || !isEnabled)
         return;
 
-    auto mappings = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
+    auto mappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
     std::string contentType = getValueOrDefault(mappings, item->getMimeType());
 
     auto fs = TagLib::FileStream(item->getLocation().c_str(), true); // true = Read only
@@ -295,40 +325,45 @@ void TagLibHandler::addArtworkResource(const std::shared_ptr<CdsItem>& item, con
     log_debug("Found artwork of type {} in file {}", artMimetype.c_str(), item->getLocation().c_str());
 
     if (artMimetype != MIMETYPE_DEFAULT) {
-        auto resource = std::make_shared<CdsResource>(ContentHandler::ID3, CdsResource::Purpose::Thumbnail);
-        resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo(artMimetype));
+        auto resource = std::make_shared<CdsResource>(ContentHandler::ID3, ResourcePurpose::Thumbnail);
+        resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo(artMimetype));
         item->addResource(resource);
     }
 }
 
 /// \brief stream content of object or resource to client
 /// \param obj Object to stream
-/// \param resNum number of resource
+/// \param resource the resource
 /// \return iohandler to stream to client
-std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<CdsObject>& obj, int resNum)
+std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource)
 {
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
     if (!item) // not streamable
         return nullptr;
 
-    auto mappings = config->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
+    auto mappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
     std::string contentType = getValueOrDefault(mappings, item->getMimeType());
+    auto itemLocation = item->getLocation().c_str();
 
-    auto roStream = TagLib::FileStream(item->getLocation().c_str(), true); // Open read only
+    auto roStream = TagLib::FileStream(itemLocation, true); // Open read only
 
     if (contentType == CONTENT_TYPE_MP3) {
         // stream album art from MP3 file
+#if TAGLIB_MAJOR_VERSION >= 2
+        auto f = TagLib::MPEG::File(&roStream, true, TagLib::MPEG::Properties::Average, TagLib::ID3v2::FrameFactory::instance());
+#else
         auto f = TagLib::MPEG::File(&roStream, TagLib::ID3v2::FrameFactory::instance());
+#endif
 
         if (!f.isValid())
-            throw_std_runtime_error("Could not open file: {}", item->getLocation().c_str());
+            throw_std_runtime_error("Could not open file {} as MP3", itemLocation);
 
         if (!f.ID3v2Tag())
-            throw_std_runtime_error("resource has no album information");
+            throw_std_runtime_error("MP3 resource {} has no ID3 information", itemLocation);
 
         auto&& list = f.ID3v2Tag()->frameList("APIC");
         if (list.isEmpty())
-            throw_std_runtime_error("resource has no album information");
+            throw_std_runtime_error("MP3 resource {} has no album information", itemLocation);
 
         auto art = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame*>(list.front());
         if (!art) {
@@ -339,13 +374,17 @@ std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<Cds
     }
     if (contentType == CONTENT_TYPE_FLAC) {
         // stream album art from FLAC file
+#if TAGLIB_MAJOR_VERSION >= 2
+        auto f = TagLib::FLAC::File(&roStream, true, TagLib::MPEG::Properties::Average, TagLib::ID3v2::FrameFactory::instance());
+#else
         auto f = TagLib::FLAC::File(&roStream, TagLib::ID3v2::FrameFactory::instance());
+#endif
 
         if (!f.isValid())
-            throw_std_runtime_error("Could not open flac file: {}", item->getLocation().c_str());
+            throw_std_runtime_error("Could not open file {} as flac", itemLocation);
 
         if (f.pictureList().isEmpty())
-            throw_std_runtime_error("flac resource has no picture information");
+            throw_std_runtime_error("Flac resource {} has empty picture information", itemLocation);
 
         const TagLib::FLAC::Picture* pic = f.pictureList().front();
         const TagLib::ByteVector& data = pic->data();
@@ -357,23 +396,23 @@ std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<Cds
         auto f = TagLib::MP4::File(&roStream);
 
         if (!f.isValid()) {
-            throw_std_runtime_error("Could not open mp4 file: {}", item->getLocation().c_str());
+            throw_std_runtime_error("Could not open file {} as MP4", itemLocation);
         }
 
         if (!f.hasMP4Tag()) {
-            throw_std_runtime_error("mp4 resource has no tag information");
+            throw_std_runtime_error("MP4 resource {} has no tag information", itemLocation);
         }
 
         auto tag = f.tag();
 
         if (!tag->contains("covr")) {
-            throw_std_runtime_error("mp4 file has no 'covr' item");
+            throw_std_runtime_error("MP4 resource {} has no 'covr' item", itemLocation);
         }
 
         auto coverItem = tag->item("covr");
         TagLib::MP4::CoverArtList coverArtList = coverItem.toCoverArtList();
         if (coverArtList.isEmpty()) {
-            throw_std_runtime_error("mp4 resource has no picture information");
+            throw_std_runtime_error("MP4 resource {} has empty 'covr' information", itemLocation);
         }
 
         const TagLib::MP4::CoverArt& coverArt = coverArtList.front();
@@ -386,19 +425,19 @@ std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<Cds
         auto f = TagLib::ASF::File(&roStream);
 
         if (!f.isValid())
-            throw_std_runtime_error("Could not open wma file: {}", item->getLocation().c_str());
+            throw_std_runtime_error("Could not open file {} as WMA", itemLocation);
 
         const TagLib::ASF::AttributeListMap& attrListMap = f.tag()->attributeListMap();
         if (!attrListMap.contains("WM/Picture"))
-            throw_std_runtime_error("wma file has no picture information");
+            throw_std_runtime_error("WMA resource {} has no picture information", itemLocation);
 
         const TagLib::ASF::AttributeList& attrList = attrListMap["WM/Picture"];
         if (attrList.isEmpty())
-            throw_std_runtime_error("wma list has no picture information");
+            throw_std_runtime_error("WMA resource {} has empty picture information", itemLocation);
 
         const TagLib::ASF::Picture& wmpic = attrList[0].toPicture();
         if (!wmpic.isValid())
-            throw_std_runtime_error("wma pic not valid");
+            throw_std_runtime_error("WMA resource {} has invalid picture", itemLocation);
 
         const TagLib::ByteVector& data = wmpic.picture();
 
@@ -409,11 +448,11 @@ std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<Cds
         auto f = getOggFile(roStream);
 
         if (!f || !f->isValid() || !f->tag())
-            throw_std_runtime_error("Could not open OGG file: {}", item->getLocation().c_str());
+            throw_std_runtime_error("Could not open file {} as OGG", itemLocation);
 
-        const TagLib::List<TagLib::FLAC::Picture*> picList = reinterpret_cast<TagLib::Ogg::XiphComment*>(f->tag())->pictureList();
+        const auto picList = dynamic_cast<TagLib::Ogg::XiphComment*>(f->tag())->pictureList();
         if (picList.isEmpty())
-            throw_std_runtime_error("OGG file has no picture information");
+            throw_std_runtime_error("OGG resource {} has empty picture information", itemLocation);
 
         const TagLib::FLAC::Picture* pic = picList.front();
         const TagLib::ByteVector& data = pic->data();
@@ -421,12 +460,16 @@ std::unique_ptr<IOHandler> TagLibHandler::serveContent(const std::shared_ptr<Cds
         return std::make_unique<MemIOHandler>(data.data(), data.size());
     }
 
-    throw_std_runtime_error("Unsupported content_type: {}", contentType.c_str());
+    throw_std_runtime_error("File {} has unsupported content type {}", itemLocation, contentType);
 }
 
 void TagLibHandler::extractMP3(TagLib::IOStream& roStream, const std::shared_ptr<CdsItem>& item) const
 {
+#if TAGLIB_MAJOR_VERSION >= 2
+    auto mp3 = TagLib::MPEG::File(&roStream, true, TagLib::MPEG::Properties::Average, TagLib::ID3v2::FrameFactory::instance());
+#else
     auto mp3 = TagLib::MPEG::File(&roStream, TagLib::ID3v2::FrameFactory::instance());
+#endif
     if (!mp3.isValid()) {
         log_info("TagLibHandler {}: does not appear to be a valid mp3 file", item->getLocation().c_str());
         return;
@@ -438,13 +481,13 @@ void TagLibHandler::extractMP3(TagLib::IOStream& roStream, const std::shared_ptr
         return;
     }
 
-    auto sc = StringConverter::i2i(config);
+    auto sc = converterManager->i2i();
 
     auto&& frameListMap = mp3.ID3v2Tag()->frameListMap();
     // http://id3.org/id3v2.4.0-frames "4.2.6. User defined text information frame"
     bool hasTXXXFrames = frameListMap.contains("TXXX");
 
-    std::vector<std::string> auxTagsList = config->getArrayOption(CFG_IMPORT_LIBOPTS_ID3_AUXDATA_TAGS_LIST);
+    std::vector<std::string> auxTagsList = config->getArrayOption(ConfigVal::IMPORT_LIBOPTS_ID3_AUXDATA_TAGS_LIST);
     for (auto&& desiredFrame : auxTagsList) {
         if (desiredFrame.empty()) {
             continue;
@@ -563,14 +606,14 @@ void TagLibHandler::extractOgg(TagLib::IOStream& roStream, const std::shared_ptr
     if (!oggFile->tag())
         return;
 
-    auto sc = StringConverter::i2i(config);
+    auto sc = converterManager->i2i();
     auto propertyMap = oggFile->properties();
     populateAuxTags(item, propertyMap, sc);
 
     // Vorbis uses the FLAC binary picture structure...
     // https://wiki.xiph.org/VorbisComment#Cover_art
     // The unofficial COVERART field is not supported.
-    const TagLib::List<TagLib::FLAC::Picture*> picList = reinterpret_cast<TagLib::Ogg::XiphComment*>(oggFile->tag())->pictureList();
+    const auto picList = dynamic_cast<TagLib::Ogg::XiphComment*>(oggFile->tag())->pictureList();
     if (picList.isEmpty())
         return;
 
@@ -594,7 +637,7 @@ void TagLibHandler::extractASF(TagLib::IOStream& roStream, const std::shared_ptr
     }
     populateGenericTags(item, asf);
 
-    auto sc = StringConverter::i2i(config);
+    auto sc = converterManager->i2i();
     auto propertyMap = asf.properties();
     populateAuxTags(item, propertyMap, sc);
 
@@ -602,7 +645,7 @@ void TagLibHandler::extractASF(TagLib::IOStream& roStream, const std::shared_ptr
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 
     const TagLib::ASF::AttributeListMap& attrListMap = asf.tag()->attributeListMap();
@@ -625,7 +668,11 @@ void TagLibHandler::extractASF(TagLib::IOStream& roStream, const std::shared_ptr
 
 void TagLibHandler::extractFLAC(TagLib::IOStream& roStream, const std::shared_ptr<CdsItem>& item) const
 {
+#if TAGLIB_MAJOR_VERSION >= 2
+    auto flac = TagLib::FLAC::File(&roStream, true, TagLib::MPEG::Properties::Average, TagLib::ID3v2::FrameFactory::instance());
+#else
     auto flac = TagLib::FLAC::File(&roStream, TagLib::ID3v2::FrameFactory::instance());
+#endif
 
     if (!flac.isValid()) {
         log_info("TagLibHandler {}: does not appear to be a valid flac file", item->getLocation().c_str());
@@ -633,7 +680,7 @@ void TagLibHandler::extractFLAC(TagLib::IOStream& roStream, const std::shared_pt
     }
     populateGenericTags(item, flac);
 
-    auto sc = StringConverter::i2i(config);
+    auto sc = converterManager->i2i();
     auto propertyMap = flac.properties();
     populateAuxTags(item, propertyMap, sc);
 
@@ -641,7 +688,7 @@ void TagLibHandler::extractFLAC(TagLib::IOStream& roStream, const std::shared_pt
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 
     if (flac.pictureList().isEmpty()) {
@@ -668,7 +715,7 @@ void TagLibHandler::extractAPE(TagLib::IOStream& roStream, const std::shared_ptr
     }
     populateGenericTags(item, ape);
 
-    auto sc = StringConverter::i2i(config);
+    auto sc = converterManager->i2i();
     auto propertyMap = ape.properties();
     populateAuxTags(item, propertyMap, sc);
 
@@ -676,7 +723,7 @@ void TagLibHandler::extractAPE(TagLib::IOStream& roStream, const std::shared_ptr
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 }
 
@@ -690,7 +737,7 @@ void TagLibHandler::extractWavPack(TagLib::IOStream& roStream, const std::shared
     }
     populateGenericTags(item, wavpack);
 
-    auto sc = StringConverter::i2i(config);
+    auto sc = converterManager->i2i();
     auto propertyMap = wavpack.properties();
     populateAuxTags(item, propertyMap, sc);
 
@@ -698,13 +745,17 @@ void TagLibHandler::extractWavPack(TagLib::IOStream& roStream, const std::shared
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 }
 
 void TagLibHandler::extractMP4(TagLib::IOStream& roStream, const std::shared_ptr<CdsItem>& item) const
 {
+#if TAGLIB_MAJOR_VERSION >= 2
+    auto mp4 = TagLib::MP4::File(&roStream, true, TagLib::MP4::Properties::Average, TagLib::MP4::ItemFactory::instance());
+#else
     auto mp4 = TagLib::MP4::File(&roStream);
+#endif
 
     if (!mp4.isValid()) {
         log_info("TagLibHandler {}: does not appear to be a valid mp4 file", item->getLocation().c_str());
@@ -718,7 +769,7 @@ void TagLibHandler::extractMP4(TagLib::IOStream& roStream, const std::shared_ptr
         return;
     }
 
-    auto sc = StringConverter::i2i(config);
+    auto sc = converterManager->i2i();
     auto propertyMap = mp4.tag()->properties();
     populateAuxTags(item, propertyMap, sc);
 
@@ -726,7 +777,7 @@ void TagLibHandler::extractMP4(TagLib::IOStream& roStream, const std::shared_ptr
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 
     if (mp4.tag()->contains("covr")) {
@@ -759,7 +810,7 @@ void TagLibHandler::extractAiff(TagLib::IOStream& roStream, const std::shared_pt
     }
     populateGenericTags(item, aiff);
 
-    auto sc = StringConverter::i2i(config);
+    auto sc = converterManager->i2i();
     auto propertyMap = aiff.properties();
     populateAuxTags(item, propertyMap, sc);
 
@@ -767,7 +818,7 @@ void TagLibHandler::extractAiff(TagLib::IOStream& roStream, const std::shared_pt
     auto temp = audioProps->bitsPerSample();
     auto res = item->getResource(ContentHandler::DEFAULT);
     if (temp > 0) {
-        res->addAttribute(CdsResource::Attribute::BITS_PER_SAMPLE, fmt::to_string(temp));
+        res->addAttribute(ResourceAttribute::BITS_PER_SAMPLE, fmt::to_string(temp));
     }
 }
 
