@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -32,17 +34,31 @@
 #ifndef __SERVER_H__
 #define __SERVER_H__
 
-#include "action_request.h"
-#include "context.h"
-#include "request_handler.h"
-#include "subscription_request.h"
-#include "upnp_cds.h"
-#include "upnp_cm.h"
-#include "upnp_mrreg.h"
+#include <memory>
+#include <netinet/in.h>
+#include <string>
+#include <upnp.h>
+#include <vector>
 
-// forward declaration
+// forward declarations
+class ActionRequest;
+class ClientManager;
+class Config;
+class Content;
+class ConverterManager;
+class Context;
+class Database;
+class MetadataService;
+class Mime;
+class Quirks;
+class RequestHandler;
+class SubscriptionRequest;
 class Timer;
-class ContentManager;
+class UpnpXMLBuilder;
+class UpnpService;
+namespace Web {
+class SessionManager;
+}
 
 /// \brief Provides methods to initialize and shutdown
 /// and to retrieve various information about the server.
@@ -54,7 +70,7 @@ public:
     ///
     /// This function reads information from the config and initializes
     /// various variables (like server UDN and so forth).
-    void init();
+    void init(bool offln);
 
     /// \brief Cleanup routine to shutdown the server.
     ///
@@ -76,27 +92,35 @@ public:
     /// terminated. This is the case when upnp_clean() was called.
     bool getShutdownStatus() const;
 
-    void sendCDSSubscriptionUpdate(const std::string& updateString);
+    void sendSubscriptionUpdate(const std::string& updateString, const std::string& serviceId);
 
-    std::shared_ptr<ContentManager> getContent() const { return content; }
+    std::shared_ptr<Content> getContent() const { return content; }
+    std::vector<std::string> getCorsHosts() const { return corsHosts; }
 
 protected:
     std::shared_ptr<Config> config;
-    std::shared_ptr<ClientManager> clients;
+    std::shared_ptr<ClientManager> clientManager;
     std::shared_ptr<Mime> mime;
     std::shared_ptr<Database> database;
-    std::shared_ptr<Web::SessionManager> session_manager;
+    std::shared_ptr<Web::SessionManager> sessionManager;
+    std::shared_ptr<ConverterManager> converterManager;
     std::shared_ptr<Context> context;
 
     std::shared_ptr<Timer> timer;
-    std::shared_ptr<ContentManager> content;
+    std::shared_ptr<Content> content;
+    std::shared_ptr<MetadataService> metadataService;
+    std::shared_ptr<Server> self;
 
     std::string ip;
     in_port_t port {};
+    std::vector<std::string> validHosts {};
+    std::vector<std::string> corsHosts {};
 
     /// \brief This flag is set to true by the upnp_cleanup() function.
     bool server_shutdown_flag {};
 
+    bool offline {};
+
     /// \brief Handle for our upnp callbacks.
     UpnpDevice_Handle rootDeviceHandle {};
     UpnpDevice_Handle clientHandle {};
@@ -107,41 +131,14 @@ protected:
     /// persistent over reboots.
     std::string serverUDN;
 
-    /// \brief Name of the virtual web server directory.
-    ///
-    /// All requests going to /content/ will be handled by our web
-    /// callback functions.
-    /// \todo Is there any need that this is a variable? A constant
-    /// should be sufficient here.
-    std::string virtual_directory;
-
     /// \brief Time interval to send ssdp:alive advertisements.
     ///
     /// The value is read from the configuration.
     int aliveAdvertisementInterval {};
 
-    std::shared_ptr<UpnpXMLBuilder> xmlbuilder;
-
-    /// \brief ContentDirectoryService instance.
-    ///
-    /// The ContentDirectoryService class is instantiated in the
-    /// constructor. The class is responsible for processing
-    /// an ActionRequest or a SubscriptionRequest.
-    std::unique_ptr<ContentDirectoryService> cds;
-
-    /// \brief ConnectionManagerService instance.
-    ///
-    /// The ConnectionManagerService class is instantiated in the
-    /// constructor. The class is responsible for processing
-    /// an ActionRequest or a SubscriptionRequest.
-    std::unique_ptr<ConnectionManagerService> cmgr;
-
-    /// \brief MediaReceiverRegistrarService instance.
-    ///
-    /// This class is not fully functional, it always returns "true"
-    /// on IsAuthorized and IsValidated requests. It added to ensure
-    /// Xbox360 compatibility.
-    std::unique_ptr<MRRegistrarService> mrreg;
+    std::shared_ptr<UpnpXMLBuilder> upnpXmlBuilder;
+    std::shared_ptr<UpnpXMLBuilder> webXmlBuilder;
+    std::vector<std::unique_ptr<UpnpService>> serviceList;
 
     /// \brief Dispatched an ActionRequest between the services.
     /// \param request Incoming ActionRequest.
@@ -162,26 +159,27 @@ protected:
     void routeSubscriptionRequest(const SubscriptionRequest& request) const;
 
     /// \brief Registers callback functions for the internal web server.
-    /// \param filename Incoming filename.
+    /// \param filename Incoming filename/url.
+    /// \param quirks Client specific handling.
     ///
-    std::unique_ptr<RequestHandler> createRequestHandler(const char* filename) const;
+    std::unique_ptr<RequestHandler> createRequestHandler(const char* filename, const std::shared_ptr<Quirks>& quirks) const;
 
     /// \brief Registers callback functions for the internal web server.
     ///
     /// This function registers callbacks for the internal web server.
     /// The callback functions are:
-    /// \b web_get_info Query information on a file.
-    /// \b web_open Open a file.
-    /// \b web_read Sequentially read from a file.
-    /// \b web_write Sequentially write to a file (not supported).
-    /// \b web_seek Perform a seek on a file.
-    /// \b web_close Close file.
+    /// \arg \c GetInfoCallback Query information on a file.
+    /// \arg \c OpenCallback Open a file.
+    /// \arg \c ReadCallback Sequentially read from a file.
+    /// \arg \c WriteCallback Sequentially write to a file (not supported).
+    /// \arg \c SeekCallback Perform a seek on a file.
+    /// \arg \c CloseCallback Close file.
     ///
-    /// \return UPNP_E_SUCCESS Callbacks registered successfully, else error code.
+    /// \return \c UPNP_E_SUCCESS Callbacks registered successfully, else error code.
     static int registerVirtualDirCallbacks();
 
     /// \brief Dispatch incoming UPnP root device events.
-    /// \param eventtype Upnp_EventType, identifying what kind of event came in.
+    /// \param eventType identifying what kind of event came in.
     /// \param event Pointer to the event.
     ///
     /// This function is called when a UPnP root device event is received,
@@ -194,7 +192,7 @@ protected:
     int handleUpnpRootDeviceEvent(Upnp_EventType eventType, const void* event);
 
     /// \brief Dispatch incoming UPnP client events.
-    /// \param eventtype Upnp_EventType, identifying what kind of event came in.
+    /// \param eventType identifying what kind of event came in.
     /// \param event Pointer to the event.
     ///
     int handleUpnpClientEvent(Upnp_EventType eventType, const void* event);
@@ -204,7 +202,6 @@ protected:
     void writeBookmark(const std::string& addr);
     void emptyBookmark();
 
-    std::string getPresentationUrl() const;
     int startupInterface(const std::string& iface, in_port_t inPort);
 
     /// \brief Returns the content url of the server.
@@ -214,6 +211,17 @@ protected:
     /// that we actually get that port after startup. This function
     /// contains the port on which the server is actually running.
     std::string getVirtualUrl() const;
+    std::string getExternalUrl() const;
+
+    std::shared_ptr<Quirks> getQuirks(const UpnpFileInfo* info, bool isWeb) const;
+    /// \brief Upnp callbacks
+    static int HostValidateCallback(const char* host, void* cookie);
+    static int GetInfoCallback(const char* filename, UpnpFileInfo* info, const void* cookie, const void** requestCookie);
+    static UpnpWebFileHandle OpenCallback(const char* filename, enum UpnpOpenFileMode mode, const void* cookie, const void* requestCookie);
+    static int ReadCallback(UpnpWebFileHandle f, char* buf, std::size_t length, const void* cookie, const void* requestCookie);
+    static int WriteCallback(UpnpWebFileHandle f, char* buf, std::size_t length, const void* cookie, const void* requestCookie);
+    static int SeekCallback(UpnpWebFileHandle f, off_t offset, int whence, const void* cookie, const void* requestCookie);
+    static int CloseCallback(UpnpWebFileHandle f, const void* cookie, const void* requestCookie);
 };
 
 #endif // __SERVER_H__
