@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -28,17 +30,16 @@
 */
 
 /// \file transcode_ext_handler.cc
+#define GRB_LOG_FAC GrbLogFacility::transcoding
 
 #include "transcode_ext_handler.h" // API
 
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <unistd.h>
-
-#include "cds_objects.h"
-#include "config/config_manager.h"
-#include "content/content_manager.h"
-#include "database/database.h"
+#include "cds/cds_objects.h"
+#include "config/config.h"
+#include "config/config_val.h"
+#include "config/result/transcoding.h"
+#include "content/content.h"
+#include "exceptions.h"
 #include "iohandler/buffered_io_handler.h"
 #include "iohandler/io_handler_chainer.h"
 #include "iohandler/process_io_handler.h"
@@ -50,6 +51,10 @@
 #include "iohandler/curl_io_handler.h"
 #endif
 
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 std::unique_ptr<IOHandler> TranscodeExternalHandler::serveContent(const std::shared_ptr<TranscodingProfile>& profile,
     const fs::path& location, const std::shared_ptr<CdsObject>& obj,
     const std::string& group, const std::string& range)
@@ -63,12 +68,12 @@ std::unique_ptr<IOHandler> TranscodeExternalHandler::serveContent(const std::sha
     if (obj->isItem()) {
         auto item = std::static_pointer_cast<CdsItem>(obj);
         auto mappings = config->getDictionaryOption(
-            CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
+            ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
 
         if (getValueOrDefault(mappings, mimeType) == CONTENT_TYPE_PCM) {
             auto res = obj->getResource(ContentHandler::DEFAULT);
-            std::string freq = res->getAttribute(CdsResource::Attribute::SAMPLEFREQUENCY));
-            std::string nrch = res->getAttribute(CdsResource::Attribute::NRAUDIOCHANNELS));
+            std::string freq = res->getAttribute(ResourceAttribute::SAMPLEFREQUENCY));
+            std::string nrch = res->getAttribute(ResourceAttribute::NRAUDIOCHANNELS));
 
             if (!freq.empty())
                 mimeType = mimeType + ";rate=" + freq;
@@ -106,13 +111,13 @@ std::unique_ptr<IOHandler> TranscodeExternalHandler::serveContent(const std::sha
 
     content->triggerPlayHook(group, obj);
 
-    auto processIoHandler = std::make_unique<ProcessIOHandler>(std::move(content), std::move(fifoName), std::move(mainProc), std::move(procList));
+    auto processIoHandler = std::make_unique<ProcessIOHandler>(content, std::move(fifoName), std::move(mainProc), std::move(procList));
     return std::make_unique<BufferedIOHandler>(config, std::move(processIoHandler), profile->getBufferSize(), profile->getBufferChunkSize(), profile->getBufferInitialFillSize());
 }
 
 fs::path TranscodeExternalHandler::makeFifo()
 {
-    fs::path tmpDir = config->getOption(CFG_SERVER_TMPDIR);
+    fs::path tmpDir = config->getOption(ConfigVal::SERVER_TMPDIR);
     auto fifoPath = tmpDir / fmt::format("grb-tr-{}", generateRandomId());
 
     log_debug("Creating FIFO: {}", fifoPath.string());
@@ -157,9 +162,9 @@ fs::path TranscodeExternalHandler::openCurlFifo(const fs::path& location, std::v
     auto ret = makeFifo();
 
     try {
-        auto cIoh = std::make_unique<CurlIOHandler>(config, url, nullptr,
-            config->getIntOption(CFG_EXTERNAL_TRANSCODING_CURL_BUFFER_SIZE),
-            config->getIntOption(CFG_EXTERNAL_TRANSCODING_CURL_FILL_SIZE));
+        auto cIoh = std::make_unique<CurlIOHandler>(config, url,
+            config->getIntOption(ConfigVal::EXTERNAL_TRANSCODING_CURL_BUFFER_SIZE),
+            config->getIntOption(ConfigVal::EXTERNAL_TRANSCODING_CURL_FILL_SIZE));
         auto pIoh = std::make_unique<ProcessIOHandler>(content, ret, nullptr);
         auto ch = std::make_unique<IOHandlerChainer>(std::move(cIoh), std::move(pIoh), 16384);
         procList.push_back(std::make_unique<ProcListItem>(std::move(ch)));
