@@ -3,7 +3,7 @@ Gerbera - https://gerbera.io/
 
     grb_fs.cc - this file is part of Gerbera.
 
-    Copyright (C) 2022 Gerbera Contributors
+    Copyright (C) 2022-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -19,9 +19,14 @@ Gerbera - https://gerbera.io/
 */
 
 /// \file grb_fs.cc
-
+#define GRB_LOG_FAC GrbLogFacility::content
 #include "grb_fs.h" // API
 
+#include "exceptions.h"
+#include "util/logger.h"
+#include "util/tools.h"
+
+#include <algorithm>
 #include <fcntl.h>
 #include <fstream>
 #include <sstream>
@@ -32,7 +37,25 @@ Gerbera - https://gerbera.io/
 #define _DEFAULT_SOURCE
 #endif
 
-#include "util/tools.h"
+std::string rtrimPath(std::string& s, unsigned char sep)
+{
+    if (!s.empty()) {
+        s.erase(std::find_if(s.rbegin(), s.rend() - 1, [&](unsigned char ch) {
+            return ch != sep;
+        }).base(),
+            s.end());
+    }
+    return s;
+}
+
+bool isSubDir(const fs::path& path, const fs::path& parent)
+{
+    auto pathStr = fmt::format("{}/", path.string());
+    auto chkStr = fmt::format("{}/", parent.string());
+    rtrimPath(pathStr);
+    rtrimPath(chkStr);
+    return startswith(pathStr, chkStr);
+}
 
 bool isRegularFile(const fs::path& path, std::error_code& ec) noexcept
 {
@@ -271,13 +294,7 @@ bool isTheora(const fs::path& oggFilename)
 
 fs::path getLastPath(const fs::path& path)
 {
-    auto it = std::prev(path.end()); // filename
-    if (it != path.end())
-        it = std::prev(it); // last path
-    if (it != path.end())
-        return *it;
-
-    return {};
+    return path.parent_path().filename();
 }
 
 #ifndef HAVE_FFMPEG
