@@ -4,7 +4,7 @@
 
     mime.cc - this file is part of Gerbera.
 
-    Copyright (C) 2021-2022 Gerbera Contributors
+    Copyright (C) 2021-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -22,17 +22,20 @@
 */
 
 /// \file mime.cc
-
+#define GRB_LOG_FAC GrbLogFacility::content
 #include "mime.h" // API
 
-#include "config/config_manager.h"
+#include "config/config.h"
+#include "config/config_val.h"
+#include "exceptions.h"
+#include "util/logger.h"
 #include "util/tools.h"
 
 Mime::Mime(const std::shared_ptr<Config>& config)
-    : extension_map_case_sensitive(config->getBoolOption(CFG_IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_CASE_SENSITIVE))
-    , ignore_unknown_extensions(config->getBoolOption(CFG_IMPORT_MAPPINGS_IGNORE_UNKNOWN_EXTENSIONS))
-    , extension_mimetype_map(config->getDictionaryOption(CFG_IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_LIST))
-    , ignoredExtensions(config->getArrayOption(CFG_IMPORT_MAPPINGS_IGNORED_EXTENSIONS))
+    : extension_map_case_sensitive(config->getBoolOption(ConfigVal::IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_CASE_SENSITIVE))
+    , ignore_unknown_extensions(config->getBoolOption(ConfigVal::IMPORT_MAPPINGS_IGNORE_UNKNOWN_EXTENSIONS))
+    , extension_mimetype_map(config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_EXTENSION_TO_MIMETYPE_LIST))
+    , ignoredExtensions(config->getArrayOption(ConfigVal::IMPORT_MAPPINGS_IGNORED_EXTENSIONS))
 {
     if (ignore_unknown_extensions && (extension_mimetype_map.empty())) {
         log_warning("Ignore unknown extensions set, but no mappings specified");
@@ -42,13 +45,14 @@ Mime::Mime(const std::shared_ptr<Config>& config)
 
 #ifdef HAVE_MAGIC
     // init filemagic
-    int magicFlags = config->getBoolOption(CFG_IMPORT_FOLLOW_SYMLINKS) ? MAGIC_MIME_TYPE | MAGIC_SYMLINK : MAGIC_MIME_TYPE;
+    int magicFlags = config->getBoolOption(ConfigVal::IMPORT_FOLLOW_SYMLINKS) ? MAGIC_MIME_TYPE | MAGIC_SYMLINK : MAGIC_MIME_TYPE;
     magicCookie = magic_open(magicFlags);
     if (!magicCookie) {
         throw_std_runtime_error("magic_open failed");
     }
 
-    std::string optMagicFile = config->getOption(CFG_IMPORT_MAGIC_FILE);
+    std::string optMagicFile = config->getOption(ConfigVal::IMPORT_MAGIC_FILE);
+    log_debug("magic '{}'", optMagicFile);
     const char* magicFile = !optMagicFile.empty() ? optMagicFile.c_str() : nullptr;
     if (magic_load(magicCookie, magicFile) == -1) {
         auto errMsg = magic_error(magicCookie);
@@ -72,6 +76,7 @@ Mime::~Mime()
 #ifdef HAVE_MAGIC
 std::string Mime::fileToMimeType(const fs::path& path, const std::string& defval)
 {
+    const std::lock_guard<std::mutex> lock(mime_mutex);
     const char* mimeType = magic_file(magicCookie, path.c_str());
     if (!mimeType || mimeType[0] == '\0') {
         return defval;
@@ -86,17 +91,19 @@ std::string Mime::bufferToMimeType(const void* buffer, std::size_t length)
 }
 #endif
 
-std::string Mime::getMimeType(const fs::path& path, const std::string& defval)
+std::pair<bool, std::string> Mime::getMimeType(const fs::path& path, const std::string& defval)
 {
-    std::string extension = path.extension();
-    if (!extension.empty())
+    auto extension = path.extension().string();
+    if (!extension.empty() && extension.at(0) == '.')
         extension.erase(0, 1); // remove leading .
 
-    if (!extension_map_case_sensitive)
+    if (!extension_map_case_sensitive) {
         extension = toLower(extension);
+    }
 
     if (std::find(ignoredExtensions.begin(), ignoredExtensions.end(), extension) != ignoredExtensions.end()) {
-        return {};
+        log_debug("Ignoring file {} because of extension", path.string());
+        return { true, "" };
     }
     std::string mimeType = getValueOrDefault(extension_mimetype_map, extension, "");
     if (mimeType.empty() && !ignore_unknown_extensions) {
@@ -107,5 +114,5 @@ std::string Mime::getMimeType(const fs::path& path, const std::string& defval)
         mimeType = defval.empty() ? extension : defval;
 #endif
     }
-    return mimeType;
+    return { ignore_unknown_extensions, mimeType };
 }
