@@ -4,7 +4,7 @@
 
     mime.h - this file is part of Gerbera.
 
-    Copyright (C) 2021-2022 Gerbera Contributors
+    Copyright (C) 2021-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -27,6 +27,7 @@
 #define __MIME_H__
 
 #include <map>
+#include <mutex>
 
 #include "util/grb_fs.h"
 
@@ -51,9 +52,10 @@ public:
 
     /// \brief Extracts mimetype from a buffer using filemagic
     std::string bufferToMimeType(const void* buffer, std::size_t length);
+    std::mutex mime_mutex;
 #endif // HAVE_MAGIC
 
-    std::string getMimeType(const fs::path& path, const std::string& defval = "");
+    std::pair<bool, std::string> getMimeType(const fs::path& path, const std::string& defval = "");
 
 private:
     bool extension_map_case_sensitive;
