@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -28,6 +30,7 @@
 */
 
 /// \file process_executor.cc
+#define GRB_LOG_FAC GrbLogFacility::proc
 
 #include "process_executor.h" // API
 
@@ -41,6 +44,8 @@
 #include "exceptions.h"
 #include "logger.h"
 
+#include "fmt/core.h"
+
 ProcessExecutor::ProcessExecutor(const std::string& command, const std::vector<std::string>& arglist, const std::map<std::string, std::string>& env, std::vector<fs::path> tempPaths)
     : tempPaths(std::move(tempPaths))
 {
@@ -70,7 +75,8 @@ ProcessExecutor::ProcessExecutor(const std::string& command, const std::vector<s
             log_debug("setenv: {}='{}'", eName, eValue);
         }
         log_debug("Launching process: {} {}", command, fmt::join(arglist, " "));
-        execvp(command.c_str(), const_cast<char**>(argv.data()));
+        if (execvp(command.c_str(), const_cast<char**>(argv.data())))
+            log_error("Failed to execvp {} {}", command, fmt::join(arglist, " "));
         break;
     default:
         break;
