@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -32,38 +34,37 @@
 #ifndef __TIMER_H__
 #define __TIMER_H__
 
+#include "grb_time.h"
+#include "thread_runner.h"
+
 #include <algorithm>
 #include <atomic>
 #include <memory>
 
-#include "common.h"
-#include "thread_runner.h"
-#include "tools.h"
-
 class Timer {
 public:
+    enum class TimerParamType {
+        IDAutoscan,
+#ifdef ONLINE_SERVICES
+        IDOnlineContent,
+#endif
+    };
+
     /// \brief This is the parameter class for timerNotify
     class Parameter {
     public:
-        enum timer_param_t {
-            IDAutoscan,
-#ifdef ONLINE_SERVICES
-            IDOnlineContent,
-#endif
-        };
-
-        Parameter(timer_param_t param, int id)
+        Parameter(TimerParamType param, int id)
             : param(param)
             , id(id)
         {
         }
 
-        timer_param_t whoami() const { return param; }
+        TimerParamType whoami() const { return param; }
         void setID(int id) { this->id = id; }
         int getID() const { return id; }
 
     protected:
-        timer_param_t param;
+        TimerParamType param;
         int id;
     };
 
@@ -81,11 +82,14 @@ public:
 
     /// \brief Add a subscriber
     ///
-    /// @param timerSubscriber Caller must ensure that before this pointer is
+    /// \param timerSubscriber Caller must ensure that before this pointer is
     /// freed the subscriber is removed by calling removeTimerSubscriber() with
     /// the same parameter argument, unless the subscription is for a one-shot
     /// timer and the subscriber has already been notified (and removed from the
     /// subscribers list).
+    /// \param notifyInterval timespan between two notifications of the subscriber
+    /// \param parameter additional parameter to provide when raising
+    /// \param once only call once
     void addTimerSubscriber(Subscriber* timerSubscriber, std::chrono::seconds notifyInterval, std::shared_ptr<Parameter> parameter, bool once = false);
     void removeTimerSubscriber(Subscriber* timerSubscriber, std::shared_ptr<Parameter> parameter, bool dontFail = false);
     void triggerWait();
@@ -104,9 +108,12 @@ protected:
         void notify()
         {
             try {
-                subscriber->timerNotify(parameter);
+                if (subscriber)
+                    subscriber->timerNotify(parameter);
+                else
+                    log_error("subscriber null");
             } catch (const std::runtime_error& e) {
-                log_error("timer caught exception!\n");
+                log_error("timer caught exception!");
             }
         }
         void updateNextNotify()
