@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -31,20 +33,23 @@
 #ifndef __TOOLS_H__
 #define __TOOLS_H__
 
-#include <chrono>
+#include <algorithm>
+#include <cinttypes>
 #include <map>
 #include <optional>
+#include <string>
 #include <vector>
 
-#include "common.h"
-
 // forward declaration
 class IOHandler;
 
+// default protocol
+constexpr auto PROTOCOL = std::string_view("http-get");
+
 /// \brief splits the given string into array of strings using a separator character.
 /// \param str String to split
 /// \param sep separator character
-/// \param treat subsequent separators as empty array elements
+/// \param empty treat subsequent separators as empty array elements
 /// \return array of strings
 std::vector<std::string> splitString(std::string_view str, char sep, bool empty = false);
 
@@ -55,12 +60,22 @@ void trimStringInPlace(std::string& str);
 std::string trimString(std::string str);
 
 /// \brief returns true if str starts with check
-constexpr bool startswith(std::string_view str, std::string_view check)
+constexpr bool startswith(std::string_view str, std::string_view prefix)
 {
 #if __cpp_lib_starts_ends_with
-    return str.starts_with(check);
+    return str.starts_with(prefix);
 #else
-    return str.rfind(check, 0) == 0;
+    return str.rfind(prefix, 0) == 0;
+#endif
+}
+
+/// \brief returns true if str starts with check
+constexpr bool endswith(std::string_view str, std::string_view suffix)
+{
+#if __cpp_lib_starts_ends_with
+    return str.ends_with(check);
+#else
+    return str.size() >= suffix.size() && str.compare(str.size() - suffix.size(), suffix.size(), suffix) == 0;
 #endif
 }
 
@@ -74,9 +89,10 @@ std::string toLower(std::string str);
 std::string& toLowerInPlace(std::string& str);
 
 /// \brief convert string to integer
-int stoiString(const std::string& str, int def = 0, int base = 10);
+std::int32_t stoiString(const std::string& str, std::int32_t def = 0, std::int32_t base = 10);
+std::int64_t stolString(const std::string& str, std::int64_t def = 0, std::int64_t base = 10);
 /// \brief convert string to unsigned long
-unsigned long stoulString(const std::string& str, int def = 0, int base = 10);
+unsigned long stoulString(const std::string& str, unsigned long def = 0, unsigned long base = 10);
 
 /// \brief  Used to replace potential multiple following //../ with single /
 void reduceString(std::string& str, char ch);
@@ -85,12 +101,8 @@ void reduceString(std::string& str, char ch);
 void replaceString(std::string& str, std::string_view from, std::string_view to);
 void replaceAllString(std::string& str, std::string_view from, std::string_view to);
 
-/// \brief Render browser friendly uri (handle IPv6)
-std::string renderWebUri(std::string_view ip, int port);
-
 /// \brief Render HTML that is doing a redirect to the given ip, port and html page.
-/// \param ip IP address as string.
-/// \param port Port as string.
+/// \param addr IP address as string.
 /// \param page HTML document to redirect to.
 /// \return string representing the desired HTML document.
 std::string httpRedirectTo(std::string_view addr, std::string_view page = "");
@@ -116,21 +128,8 @@ std::string hexMd5(const void* data, std::size_t length);
 /// \brief Generates hex md5 sum of the given string.
 std::string hexStringMd5(std::string_view str);
 
-/// \brief Converts a string to a URL (meaning: %20 instead of space and so on)
-/// \param str String to be converted.
-/// \return string that contains the url-escaped representation of the original string.
-std::string urlEscape(std::string_view str);
-
-/// \brief Opposite of urlEscape :)
-std::string urlUnescape(std::string_view str);
-
-std::string dictEncode(const std::map<std::string, std::string>& dict);
-std::string dictEncodeSimple(const std::map<std::string, std::string>& dict);
-std::map<std::string, std::string> dictDecode(std::string_view url, bool unEscape = true);
-std::map<std::string, std::string> pathToMap(std::string_view url);
-
 /// \brief Convert an array of strings to a CSV list, with additional protocol information
-/// \param array that needs to be converted
+/// \param mimeTypes array that needs to be converted
 /// \return string containing the CSV list
 std::string mimeTypesToCsv(const std::vector<std::string>& mimeTypes);
 
@@ -139,33 +138,21 @@ std::string mimeTypesToCsv(const std::vector<std::string>& mimeTypes);
 ///
 /// \param mimetype the mimetype that should be inserted
 /// \param protocol the protocol which should be inserted (default: "http-get")
+/// \param extend extend the protocolInfo
 /// \return The rendered protocolInfo String
 std::string renderProtocolInfo(std::string_view mimetype, std::string_view protocol = PROTOCOL, std::string_view extend = "");
 
 /// \brief Extracts mimetype from the protocol info string.
 /// \param protocol info string as used in the protocolInfo attribute
+/// \return mime-type in protocol
 std::string getMTFromProtocolInfo(std::string_view protocol);
 
 /// \brief Parses a protocolInfo string (see renderProtocolInfo).
 ///
-/// \param protocolInfoStr the String from renderProtocolInfo.
+/// \param protocolInfo the String from renderProtocolInfo.
 /// \return Protocol (i.e. http-get).
 std::string_view getProtocol(std::string_view protocolInfo);
 
-template <typename TP>
-std::chrono::seconds toSeconds(TP tp)
-{
-    auto asSystemTime = std::chrono::time_point_cast<std::chrono::system_clock::duration>(tp - TP::clock::now()
-        + std::chrono::system_clock::now());
-    return std::chrono::duration_cast<std::chrono::seconds>(asSystemTime.time_since_epoch());
-}
-
-/// \brief Converts a number of milliseconds to "H*:MM:SS.F*" representation as required by the UPnP duration spec
-std::string millisecondsToHMSF(int milliseconds);
-
-/// \brief converts a "H*:MM:SS.F*" representation to milliseconds
-int HMSFToMilliseconds(std::string_view time);
-
 std::string escape(std::string_view string, char escapeChar, char toEscape);
 
 /// \brief Returns the first string if it isn't "nullptr", otherwise the fallback string.
@@ -179,43 +166,6 @@ std::string fallbackString(const std::string& first, const std::string& fallback
 /// \return return the (unsigned int) hash value
 unsigned int stringHash(std::string_view str);
 
-/// \brief Iterator over values of a sequential enum between begin and end
-template <typename C, C beginVal, C endVal>
-class EnumIterator {
-private:
-    using val_t = std::underlying_type_t<C>;
-    val_t val;
-
-public:
-    explicit EnumIterator(const C& f)
-        : val(static_cast<val_t>(f))
-    {
-    }
-    EnumIterator()
-        : val(static_cast<val_t>(beginVal))
-    {
-    }
-
-    EnumIterator operator++()
-    {
-        ++val;
-        return *this;
-    }
-
-    C operator*() { return static_cast<C>(val); }
-
-    EnumIterator begin() const { return *this; }
-    EnumIterator end() const
-    {
-        static const EnumIterator endIter = EnumIterator(endVal);
-        return endIter;
-    }
-    bool operator!=(const EnumIterator& i) const
-    {
-        return val != i.val;
-    }
-};
-
 /// \brief Get value of map, iff not key is not in map return defval
 template <typename K, typename V>
 V getValueOrDefault(const std::vector<std::pair<K, V>>& m, const K& key, const V& defval)
@@ -236,12 +186,6 @@ V getValueOrDefault(const std::map<K, V>& m, const K& key, const V& defval)
 std::string getValueOrDefault(const std::vector<std::pair<std::string, std::string>>& m, const std::string& key, const std::string& defval = "");
 std::string getValueOrDefault(const std::map<std::string, std::string>& m, const std::string& key, const std::string& defval = "");
 
-std::chrono::seconds currentTime();
-std::chrono::milliseconds currentTimeMS();
-
-std::chrono::milliseconds getDeltaMillis(std::chrono::milliseconds ms);
-std::chrono::milliseconds getDeltaMillis(std::chrono::milliseconds first, std::chrono::milliseconds second);
-
 /// \brief Parses a command line, splitting the arguments into an array and
 /// substitutes %in and %out tokens with given strings.
 ///
