@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -30,25 +32,25 @@
 /// \file url.h
 /// \brief Definition of the URLL class.
 
-#ifdef HAVE_CURL
-
 #ifndef __URL_H__
 #define __URL_H__
 
+#ifdef HAVE_CURL
+
 #include <curl/curl.h>
 #include <memory>
-
-#include "common.h"
+#include <string>
 
 class URL {
 public:
-    /// \brief This is a simplified version of the File_Info class as used
+    /// \brief This is a simplified version of the UpnpFileInfo class as used
     /// in libupnp.
     class Stat {
     public:
         /// \brief Initializes the class with given values, the values
         /// can not be changed afterwards.
         ///
+        /// \param url address of the web site
         /// \param size size of the media in bytes
         /// \param mimetype mime type of the media
         Stat(std::string url, curl_off_t size, std::string mimetype)
@@ -68,32 +70,41 @@ public:
         std::string mimetype;
     };
 
+    /// \brief create url wrapper object
+    /// \param url address to open
+    /// \param curlHandle an initialized and ready to use curl handle
+    URL(const std::string& url, CURL* curlHandle = nullptr);
+    ~URL();
+
     /// \brief downloads either the content or the headers to the buffer.
     ///
     /// This function uses an already initialized curl_handle, the reason
     /// is, that curl might keep the connection open if we do subsequent
     /// requests to the same server.
     ///
-    /// \param curl_handle an initialized and ready to use curl handle
-    /// \param URL
-    /// \param only_header set true if you only want the header and not the
-    /// body
-    /// \param vebose enable curl verbose option
-    static std::string download(const std::string& url,
-        long* httpRetcode,
-        CURL* curlHandle = nullptr,
+    /// \param httpRetcode return code of the http call
+    /// \param onlyHeader set true if you only want the header and not the body
+    /// \param verbose enable curl verbose option
+    /// \param redirect allow redirection
+    /// \return downloaded header and content
+    std::pair<std::string, std::string> download(long* httpRetcode,
         bool onlyHeader = false,
         bool verbose = false,
         bool redirect = false);
 
-    static Stat getInfo(const std::string& url, CURL* curlHandle = nullptr);
+    Stat getInfo();
 
 protected:
     /// \brief This function is installed as a callback for libcurl, when
     /// we download data from a remote site.
+    /// \return number of read bytes
     static std::size_t dl(char* buf, std::size_t size, std::size_t nmemb, std::ostringstream* data);
+
+    std::string url;
+    CURL* curlHandle;
+    bool cleanup { false };
 };
 
-#endif //__URL_H__
-
 #endif // HAVE_CURL
+
+#endif //__URL_H__
