@@ -13,6 +13,8 @@
 
     Copyright (C) 2020 Patrick Ammann <pammann@gmx.net>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -35,32 +37,31 @@
 #define __UTIL_XML_TO_JSON_H__
 
 #include <map>
-#include <memory>
-#include <sstream>
-#include <stack>
 
 #include <pugixml.hpp>
 
+enum class FieldType {
+    STRING,
+    ENCSTRING,
+    NUMBER,
+    BOOL,
+};
+
 class Xml2Json {
 public:
-    class Hints {
-    public:
-        void setArrayName(const pugi::xml_node& node, std::string_view name) { asArray.insert_or_assign(node, name); }
-        void setFieldType(const std::string& node, std::string_view type) { asType.insert_or_assign(node, type); }
+    std::string getJson(const pugi::xml_node& node);
 
-    private:
-        std::map<pugi::xml_node, std::string> asArray;
-        std::map<std::string, std::string> asType;
-        friend class Xml2Json;
-    };
-
-    static std::string getJson(const pugi::xml_node& root, const Hints& hints);
+    void setArrayName(const pugi::xml_node& node, std::string_view name) { asArray.insert_or_assign(node, name); }
+    void setFieldType(const std::string& node, FieldType type) { asType.insert_or_assign(node, type); }
 
 private:
-    static void handleElement(std::ostringstream& buf, const pugi::xml_node& node, const Hints& hints);
     static std::string getAsString(std::string_view str);
-    static std::string getValue(const std::string& name, const char* text, const Hints& hints);
-    static std::pair<bool, std::string_view> isArray(const pugi::xml_node& node, const Hints& hints);
+    static std::string getEncString(std::string_view str);
+    std::string getValue(const std::string& name, const char* text);
+    std::pair<bool, std::string_view> isArray(const pugi::xml_node& node);
+
+    std::map<pugi::xml_node, std::string> asArray;
+    std::map<std::string, FieldType> asType;
 };
 
 #endif // __UTIL_XML_TO_JSON_H__
