@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    containers.cc - this file is part of MediaTomb.
+    web/containers.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,25 +29,22 @@
     $Id$
 */
 
-/// \file containers.cc
+/// \file web/containers.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include "cds_objects.h"
-#include "content/autoscan.h"
+#include "cds/cds_container.h"
+#include "config/config_val.h"
+#include "config/result/autoscan.h"
 #include "database/database.h"
-#include "server.h"
-#include "upnp_xml.h"
-
-Web::Containers::Containers(const std::shared_ptr<ContentManager>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder)
-    : WebRequestHandler(content)
-    , xmlBuilder(std::move(xmlBuilder))
-{
-}
+#include "exceptions.h"
+#include "upnp/xml_builder.h"
+#include "util/xml_to_json.h"
 
 void Web::Containers::process()
 {
-    log_debug(("containers.cc: containers::process()"));
+    log_debug("start process()");
     checkRequest();
 
     int parentID = intParam("parent_id", INVALID_OBJECT_ID);
@@ -55,8 +54,8 @@ void Web::Containers::process()
     auto root = xmlDoc->document_element();
 
     auto containers = root.append_child("containers");
-    xml2JsonHints->setArrayName(containers, "container");
-    xml2JsonHints->setFieldType("title", "string");
+    xml2Json->setArrayName(containers, "container");
+    xml2Json->setFieldType("title", FieldType::STRING);
     containers.append_attribute("parent_id") = parentID;
     containers.append_attribute("type") = "database";
     if (!param("select_it").empty())
@@ -65,8 +64,6 @@ void Web::Containers::process()
     auto browseParam = BrowseParam(database->loadObject(DEFAULT_CLIENT_GROUP, parentID), BROWSE_DIRECT_CHILDREN | BROWSE_CONTAINERS);
     auto arr = database->browse(browseParam);
     for (auto&& obj : arr) {
-        // if (obj->isContainer())
-        //{
         auto cont = std::static_pointer_cast<CdsContainer>(obj);
         auto ce = containers.append_child("container");
         ce.append_attribute("id") = cont->getID();
@@ -82,17 +79,19 @@ void Web::Containers::process()
 
         std::string autoscanMode = "none";
         if (autoscanType > 0) {
-            autoscanMode = "timed";
+            autoscanMode = AUTOSCAN_TIMED;
 #ifdef HAVE_INOTIFY
-            if (config->getBoolOption(CFG_IMPORT_AUTOSCAN_USE_INOTIFY)) {
+            if (config->getBoolOption(ConfigVal::IMPORT_AUTOSCAN_USE_INOTIFY)) {
                 auto adir = database->getAutoscanDirectory(cont->getID());
-                if (adir && (adir->getScanMode() == AutoscanDirectory::ScanMode::INotify))
-                    autoscanMode = "inotify";
+                if (adir && (adir->getScanMode() == AutoscanScanMode::INotify))
+                    autoscanMode = AUTOSCAN_INOTIFY;
             }
 #endif
         }
         ce.append_attribute("autoscan_mode") = autoscanMode.c_str();
         ce.append_attribute("title") = cont->getTitle().c_str();
-        //}
+        ce.append_attribute("location") = cont->getLocation().c_str();
+        ce.append_attribute("upnp_class") = cont->getClass().c_str();
     }
+    log_debug("end process()");
 }
