@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    directories.cc - this file is part of MediaTomb.
+    web/directories.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,41 +29,57 @@
     $Id$
 */
 
-/// \file directories.cc
+/// \file web/directories.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include <array>
-
-#include "content/content_manager.h"
-#include "database/database.h"
+#include "config/config_val.h"
+#include "config/result/autoscan.h"
+#include "content/content.h"
 #include "util/string_converter.h"
 #include "util/tools.h"
+#include "util/xml_to_json.h"
+
+#include <algorithm>
+#include <array>
 
 using dirInfo = std::pair<fs::path, bool>;
 
+Web::Directories::Directories(const std::shared_ptr<Content>& content,
+    std::shared_ptr<ConverterManager> converterManager,
+    const std::shared_ptr<Server>& server,
+    const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
+    const std::shared_ptr<Quirks>& quirks)
+    : WebRequestHandler(content, server, xmlBuilder, quirks)
+    , converterManager(std::move(converterManager))
+{
+}
+
 void Web::Directories::process()
 {
+    static auto RootId = fmt::format("{}", CDS_ID_ROOT);
+
     checkRequest();
 
     std::string parentID = param("parent_id");
-    auto path = fs::path(parentID.empty() || parentID == "0" ? FS_ROOT_DIRECTORY : hexDecodeString(parentID));
+    auto path = fs::path(parentID.empty() || parentID == RootId ? FS_ROOT_DIRECTORY : hexDecodeString(parentID));
 
     auto root = xmlDoc->document_element();
 
     auto containers = root.append_child("containers");
-    xml2JsonHints->setArrayName(containers, "container");
-    xml2JsonHints->setFieldType("title", "string");
+    xml2Json->setArrayName(containers, "container");
+    xml2Json->setFieldType("title", FieldType::STRING);
     containers.append_attribute("parent_id") = parentID.c_str();
     if (!param("select_it").empty())
         containers.append_attribute("select_it") = param("select_it").c_str();
     containers.append_attribute("type") = "filesystem";
 
     // don't bother users with system directorties
-    auto&& excludesFullpath = config->getArrayOption(CFG_IMPORT_SYSTEM_DIRECTORIES);
-    auto&& includesFullpath = config->getArrayOption(CFG_IMPORT_VISIBLE_DIRECTORIES);
+    auto&& excludesFullpath = config->getArrayOption(ConfigVal::IMPORT_SYSTEM_DIRECTORIES);
+    auto&& includesFullpath = config->getArrayOption(ConfigVal::IMPORT_VISIBLE_DIRECTORIES);
     // don't bother users with special or config directorties
-    constexpr auto excludesDirname = std::array {
+    constexpr std::array excludesDirname {
         "lost+found",
     };
     bool excludeConfigDirs = true;
@@ -94,7 +112,7 @@ void Web::Directories::process()
         filesMap.emplace(id, std::pair(filepath, hasContent));
     }
 
-    auto f2i = StringConverter::f2i(config);
+    auto f2i = converterManager->f2i();
     for (auto&& [key, val] : filesMap) {
         auto file = val.first;
         auto&& has = val.second;
@@ -113,5 +131,7 @@ void Web::Directories::process()
             }
         }
         ce.append_attribute("title") = f2i->convert(file.filename()).c_str();
+        ce.append_attribute("location") = f2i->convert(file).c_str();
+        ce.append_attribute("upnp_class") = "folder";
     }
 }
