@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    edit_load.cc - this file is part of MediaTomb.
+    web/edit_load.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,58 +29,21 @@
     $Id$
 */
 
-/// \file edit_load.cc
+/// \file web/edit_load.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include <array>
-#include <fmt/chrono.h>
-
-#include "cds_objects.h"
+#include "cds/cds_container.h"
+#include "cds/cds_item.h"
 #include "database/database.h"
-#include "metadata/metadata_handler.h"
-#include "server.h"
-#include "upnp_xml.h"
+#include "exceptions.h"
+#include "upnp/clients.h"
+#include "upnp/xml_builder.h"
 #include "util/tools.h"
-#include "util/upnp_clients.h"
+#include "util/xml_to_json.h"
 
-Web::EditLoad::EditLoad(const std::shared_ptr<ContentManager>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder)
-    : WebRequestHandler(content)
-    , xmlBuilder(std::move(xmlBuilder))
-{
-}
-
-static constexpr auto upnpFlags = std::array {
-    std::pair("Restricted", OBJECT_FLAG_RESTRICTED),
-    std::pair("Searchable", OBJECT_FLAG_SEARCHABLE),
-    std::pair("UseResourceRef", OBJECT_FLAG_USE_RESOURCE_REF),
-    std::pair("PersistentContainer", OBJECT_FLAG_PERSISTENT_CONTAINER),
-    std::pair("PlaylistRef", OBJECT_FLAG_PLAYLIST_REF),
-    std::pair("ProxyUrl", OBJECT_FLAG_PROXY_URL),
-    std::pair("OnlineService", OBJECT_FLAG_ONLINE_SERVICE),
-    std::pair("OggTheora", OBJECT_FLAG_OGG_THEORA),
-};
-
-static std::string mapFlags(int flags)
-{
-    if (!flags)
-        return "None";
-
-    std::vector<std::string> myFlags;
-
-    for (auto [uLabel, uFlag] : upnpFlags) {
-        if (flags & uFlag) {
-            myFlags.emplace_back(uLabel);
-            flags &= ~uFlag;
-        }
-    }
-
-    if (flags) {
-        myFlags.push_back(fmt::format("{:#04x}", flags));
-    }
-
-    return fmt::format("{}", fmt::join(myFlags, " | "));
-}
+#include <fmt/chrono.h>
 
 /// \brief: process request 'edit_load' to list contents of a folder
 void Web::EditLoad::process()
@@ -94,8 +59,8 @@ void Web::EditLoad::process()
 
     // set handling of json properties
     auto root = xmlDoc->document_element();
-    xml2JsonHints->setFieldType("value", "string");
-    xml2JsonHints->setFieldType("title", "string");
+    xml2Json->setFieldType("value", FieldType::STRING);
+    xml2Json->setFieldType("title", FieldType::STRING);
 
     // write object core info
     auto item = root.append_child("item");
@@ -110,7 +75,7 @@ void Web::EditLoad::process()
     classEl.append_attribute("editable") = true;
 
     auto flagsEl = item.append_child("flags");
-    flagsEl.append_attribute("value") = mapFlags(obj->getFlags()).c_str();
+    flagsEl.append_attribute("value") = CdsObject::mapFlags(obj->getFlags()).c_str();
     flagsEl.append_attribute("editable") = false;
 
     if (obj->getMTime() > std::chrono::seconds::zero()) {
@@ -137,8 +102,8 @@ void Web::EditLoad::process()
 
     // write metadata
     auto metaData = item.append_child("metadata");
-    xml2JsonHints->setArrayName(metaData, "metadata");
-    xml2JsonHints->setFieldType("metavalue", "string");
+    xml2Json->setArrayName(metaData, "metadata");
+    xml2Json->setFieldType("metavalue", FieldType::STRING);
 
     for (auto&& [key, val] : obj->getMetaData()) {
         auto metaEntry = metaData.append_child("metadata");
@@ -149,8 +114,8 @@ void Web::EditLoad::process()
 
     // write auxdata
     auto auxData = item.append_child("auxdata");
-    xml2JsonHints->setArrayName(auxData, "auxdata");
-    xml2JsonHints->setFieldType("auxvalue", "string");
+    xml2Json->setArrayName(auxData, "auxdata");
+    xml2Json->setFieldType("auxvalue", FieldType::STRING);
 
     for (auto&& [key, val] : obj->getAuxData()) {
         auto auxEntry = auxData.append_child("auxdata");
@@ -160,9 +125,9 @@ void Web::EditLoad::process()
     }
 
     auto resources = item.append_child("resources");
-    xml2JsonHints->setArrayName(resources, "resources");
-    xml2JsonHints->setFieldType("resvalue", "string");
-    xml2JsonHints->setFieldType("rawvalue", "string");
+    xml2Json->setArrayName(resources, "resources");
+    xml2Json->setFieldType("resvalue", FieldType::ENCSTRING);
+    xml2Json->setFieldType("rawvalue", FieldType::STRING);
 
     auto objItem = std::dynamic_pointer_cast<CdsItem>(obj);
 
@@ -175,18 +140,18 @@ void Web::EditLoad::process()
 
         resEntry = resources.append_child("resources");
         resEntry.append_attribute("resname") = "handlerType";
-        resEntry.append_attribute("resvalue") = MetadataHandler::mapContentHandler2String(resItem->getHandlerType()).c_str();
+        resEntry.append_attribute("resvalue") = EnumMapper::mapContentHandler2String(resItem->getHandlerType()).c_str();
         resEntry.append_attribute("editable") = false;
 
         resEntry = resources.append_child("resources");
         resEntry.append_attribute("resname") = "purpose";
-        resEntry.append_attribute("resvalue") = CdsResource::getPurposeDisplay(resItem->getPurpose()).c_str();
+        resEntry.append_attribute("resvalue") = EnumMapper::getPurposeDisplay(resItem->getPurpose()).c_str();
         resEntry.append_attribute("editable") = false;
 
         // write resource content
         if (objItem) {
-            std::string url = xmlBuilder->renderResourceURL(*objItem, *resItem);
-            if (resItem->getPurpose() == CdsResource::Purpose::Thumbnail) {
+            std::string url = xmlBuilder->renderResourceURL(*objItem, *resItem, {});
+            if (resItem->getPurpose() == ResourcePurpose::Thumbnail) {
                 auto image = resources.append_child("resources");
                 image.append_attribute("resname") = "image";
                 image.append_attribute("resvalue") = url.c_str();
@@ -211,7 +176,7 @@ void Web::EditLoad::process()
             auto val = resItem->getAttribute(attr);
             if (!val.empty()) {
                 auto resEntry = resources.append_child("resources");
-                resEntry.append_attribute("resname") = CdsResource::getAttributeDisplay(attr).c_str();
+                resEntry.append_attribute("resname") = EnumMapper::getAttributeDisplay(attr).c_str();
                 auto aVal = resItem->getAttributeValue(attr);
                 if (aVal != val)
                     resEntry.append_attribute("rawvalue") = val.c_str();
@@ -231,7 +196,7 @@ void Web::EditLoad::process()
     // write item meta info
     if (obj->isItem()) {
         auto description = item.append_child("description");
-        description.append_attribute("value") = objItem->getMetaData(M_DESCRIPTION).c_str();
+        description.append_attribute("value") = objItem->getMetaData(MetadataFields::M_DESCRIPTION).c_str();
         description.append_attribute("editable") = true;
 
         auto location = item.append_child("location");
@@ -277,7 +242,7 @@ void Web::EditLoad::process()
 
         if (obj->isExternalItem()) {
             auto protocol = item.append_child("protocol");
-            protocol.append_attribute("value") = getProtocol(objItem->getResource(ContentHandler::DEFAULT)->getAttribute(CdsResource::Attribute::PROTOCOLINFO)).data();
+            protocol.append_attribute("value") = getProtocol(objItem->getResource(ContentHandler::DEFAULT)->getAttribute(ResourceAttribute::PROTOCOLINFO)).data();
             protocol.append_attribute("editable") = true;
         }
     }
