@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    edit_save.cc - this file is part of MediaTomb.
+    web/edit_save.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,13 +29,15 @@
     $Id$
 */
 
-/// \file edit_save.cc
+/// \file web/edit_save.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include "content/content_manager.h"
-#include "database/database.h"
+#include "content/content.h"
+#include "exceptions.h"
 #include "util/tools.h"
+#include "util/xml_to_json.h"
 
 void Web::EditSave::process()
 {
