@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    pages.h - this file is part of MediaTomb.
+    web/pages.h - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,21 +29,28 @@
     $Id$
 */
 
-/// \file pages.h
-/// \brief Defines the various WebRequestHandler subclasses which process requests coming to the ui.
+/// \file web/pages.h
+/// \brief Defines the various WebRequestHandler sub classes which process requests coming from WebUi.
 #ifndef __WEB_PAGES_H__
 #define __WEB_PAGES_H__
 
-#include "cds_objects.h"
-#include "common.h"
-#include "config/config_setup.h"
-#include "request_handler.h"
 #include "web_request_handler.h"
 
-// forward declaration
+#include "util/grb_fs.h"
+
+#include <chrono>
+
+// forward declarations
+class AutoscanDirectory;
+class CdsItemExternalURL;
+class CdsItem;
 class Config;
+class ConfigSetup;
+class ConfigValue;
+class ConverterManager;
 class Database;
 class UpnpXMLBuilder;
+enum class ConfigVal;
 
 namespace Web {
 
@@ -51,50 +60,62 @@ class SessionManager;
 /// \brief Authentication handler (used over AJAX)
 class Auth : public WebRequestHandler {
 protected:
-    std::chrono::seconds timeout {};
+    std::chrono::seconds timeout = std::chrono::seconds::zero();
+    bool accountsEnabled { false };
 
 public:
-    explicit Auth(const std::shared_ptr<ContentManager>& content);
+    explicit Auth(const std::shared_ptr<Content>& content,
+        const std::shared_ptr<Server>& server,
+        const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
+        const std::shared_ptr<Quirks>& quirks);
     void process() override;
 };
 
-/// \brief Browser container tree
+/// \brief Call from WebUi to create container tree in database view
 class Containers : public WebRequestHandler {
-protected:
-    std::shared_ptr<UpnpXMLBuilder> xmlBuilder;
+    using WebRequestHandler::WebRequestHandler;
 
 public:
-    explicit Containers(const std::shared_ptr<ContentManager>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder);
     void process() override;
 };
 
-/// \brief Browser directory tree
+/// \brief Call from WebUi to create directory tree in filesystem view
 class Directories : public WebRequestHandler {
-    using WebRequestHandler::WebRequestHandler;
-
 public:
+    explicit Directories(const std::shared_ptr<Content>& content,
+        std::shared_ptr<ConverterManager> converterManager,
+        const std::shared_ptr<Server>& server,
+        const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
+        const std::shared_ptr<Quirks>& quirks);
     void process() override;
-};
 
-/// \brief Browser file list
-class Files : public WebRequestHandler {
-    using WebRequestHandler::WebRequestHandler;
-
-public:
-    void process() override;
-};
-
-/// \brief Browser item list
-class Items : public WebRequestHandler {
 protected:
-    std::shared_ptr<UpnpXMLBuilder> xmlBuilder;
+    std::shared_ptr<ConverterManager> converterManager;
+};
+
+/// \brief Call from WebUi to list files in filesystem view
+class Files : public WebRequestHandler {
+public:
+    explicit Files(const std::shared_ptr<Content>& content,
+        std::shared_ptr<ConverterManager> converterManager,
+        const std::shared_ptr<Server>& server,
+        const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
+        const std::shared_ptr<Quirks>& quirks);
+    void process() override;
+
+protected:
+    std::shared_ptr<ConverterManager> converterManager;
+};
+
+/// \brief Call from WebUi to list items in database view
+class Items : public WebRequestHandler {
+    using WebRequestHandler::WebRequestHandler;
 
 public:
-    explicit Items(const std::shared_ptr<ContentManager>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder);
     void process() override;
 };
 
-/// \brief Browser add item
+/// \brief Call from WebUi to add item from filesystem view
 class Add : public WebRequestHandler {
     using WebRequestHandler::WebRequestHandler;
 
@@ -102,7 +123,7 @@ public:
     void process() override;
 };
 
-/// \brief Browser remove item
+/// \brief Call from WebUi to Remove item in database view
 class Remove : public WebRequestHandler {
     using WebRequestHandler::WebRequestHandler;
 
@@ -110,17 +131,15 @@ public:
     void process() override;
 };
 
-/// \brief Browser remove item
+/// \brief Call from WebUi to Edit Item in database view
 class EditLoad : public WebRequestHandler {
-protected:
-    std::shared_ptr<UpnpXMLBuilder> xmlBuilder;
+    using WebRequestHandler::WebRequestHandler;
 
 public:
-    explicit EditLoad(const std::shared_ptr<ContentManager>& content, std::shared_ptr<UpnpXMLBuilder> xmlBuilder);
     void process() override;
 };
 
-/// \brief Browser remove item
+/// \brief Call from WebUi to Save Item properties in database view
 class EditSave : public WebRequestHandler {
     using WebRequestHandler::WebRequestHandler;
 
@@ -128,7 +147,7 @@ public:
     void process() override;
 };
 
-/// \brief Browser add object.
+/// \brief Call from WebUi to Add Object in database view
 class AddObject : public WebRequestHandler {
     using WebRequestHandler::WebRequestHandler;
 
@@ -136,12 +155,13 @@ public:
     void process() override;
 
 protected:
-    void addContainer(int parentID);
-    std::shared_ptr<CdsItem> addItem(int parentID);
-    std::shared_ptr<CdsItemExternalURL> addUrl(int parentID, bool addProtocol);
+    void addContainer(int parentID, const std::string& title, const std::string& upnp_class);
+    std::shared_ptr<CdsItem> addItem(int parentID, const std::string& title, const std::string& upnp_class, const fs::path& location);
+    std::shared_ptr<CdsItemExternalURL> addUrl(int parentID, const std::string& title, const std::string& upnp_class, bool addProtocol, const fs::path& location);
+    bool isHiddenFile(const std::shared_ptr<CdsObject>& cdsObj);
 };
 
-/// \brief autoscan add and remove
+/// \brief Call from WebUi to add or remove autoscan
 class Autoscan : public WebRequestHandler {
     using WebRequestHandler::WebRequestHandler;
 
@@ -152,7 +172,7 @@ protected:
     static void autoscan2XML(const std::shared_ptr<AutoscanDirectory>& adir, pugi::xml_node& element);
 };
 
-/// \brief nothing :)
+/// \brief Call from WebUi to do nothing :)
 class VoidType : public WebRequestHandler {
     using WebRequestHandler::WebRequestHandler;
 
@@ -177,12 +197,19 @@ public:
 };
 
 /// \brief Chooses and creates the appropriate handler for processing the request.
+/// \param context runtime context
+/// \param content content handler
+/// \param server server instance
+/// \param xmlBuilder builder for xml string
+/// \param quirks hook to client specific behaviour
 /// \param page identifies what type of the request we are dealing with.
 /// \return the appropriate request handler.
 std::unique_ptr<WebRequestHandler> createWebRequestHandler(
     const std::shared_ptr<Context>& context,
-    const std::shared_ptr<ContentManager>& content,
+    const std::shared_ptr<Content>& content,
+    const std::shared_ptr<Server>& server,
     const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
+    const std::shared_ptr<Quirks>& quirks,
     const std::string& page);
 
 /// \brief Browse clients list
@@ -193,29 +220,50 @@ public:
     void process() override;
 };
 
-/// \brief load configuration
+/// \brief Call from WebUi to load configuration
 class ConfigLoad : public WebRequestHandler {
 protected:
     std::vector<ConfigValue> dbEntries;
     std::map<std::string, std::string> allItems;
-    void createItem(pugi::xml_node& item, const std::string& name, config_option_t id, config_option_t aid, const std::shared_ptr<ConfigSetup>& cs = nullptr);
+    void createItem(pugi::xml_node& item, const std::string& name, ConfigVal id, ConfigVal aid, const std::shared_ptr<ConfigSetup>& cs = nullptr);
+
+    void writeDatabaseStatus(pugi::xml_node& values);
+    void writeSimpleProperties(pugi::xml_node& values);
+    void writeClientConfig(pugi::xml_node& values);
+    void writeImportTweaks(pugi::xml_node& values);
+    void writeDynamicContent(pugi::xml_node& values);
+    void writeBoxLayout(pugi::xml_node& values);
+    void writeTranscoding(pugi::xml_node& values);
+    void writeAutoscan(pugi::xml_node& values);
+    void writeDictionaries(pugi::xml_node& values);
+    void writeVectors(pugi::xml_node& values);
+    void writeArrays(pugi::xml_node& values);
+    void updateEntriesFromDatabase(pugi::xml_node& root, pugi::xml_node& values);
+
     template <typename T>
     static void setValue(pugi::xml_node& item, const T& value);
 
     static void addTypeMeta(pugi::xml_node& meta, const std::shared_ptr<ConfigSetup>& cs);
 
 public:
-    explicit ConfigLoad(const std::shared_ptr<ContentManager>& content);
+    explicit ConfigLoad(const std::shared_ptr<Content>& content,
+        const std::shared_ptr<Server>& server,
+        const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
+        const std::shared_ptr<Quirks>& quirks);
     void process() override;
 };
 
-/// \brief save configuration
+/// \brief Call from WebUi to save configuration
 class ConfigSave : public WebRequestHandler {
 protected:
     std::shared_ptr<Context> context;
 
 public:
-    explicit ConfigSave(std::shared_ptr<Context> context, const std::shared_ptr<ContentManager>& content);
+    explicit ConfigSave(std::shared_ptr<Context> context,
+        const std::shared_ptr<Content>& content,
+        const std::shared_ptr<Server>& server,
+        const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
+        const std::shared_ptr<Quirks>& quirks);
     void process() override;
 };
 
