@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    remove.cc - this file is part of MediaTomb.
+    web/remove.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,13 +29,15 @@
     $Id$
 */
 
-/// \file remove.cc
+/// \file web/remove.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include "content/content_manager.h"
+#include "content/content.h"
 #include "database/database.h"
 #include "util/tools.h"
+#include "util/xml_to_json.h"
 
 void Web::Remove::process()
 {
@@ -44,7 +48,14 @@ void Web::Remove::process()
     int objectID = intParam("object_id");
     bool all = intParam("all");
 
-    content->removeObject(nullptr, objectID, true, all);
+    std::shared_ptr<CdsObject> obj;
+    try {
+        obj = database->loadObject(objectID);
+    } catch (const std::runtime_error&) {
+        log_error("Trying to remove an object ID which is no longer in the database! {}", objectID);
+        return;
+    }
+    content->removeObject(nullptr, obj, "", true, all);
 
     log_debug("remove: returning");
 }
