@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -38,6 +40,8 @@
 
 #include "util/timer.h"
 
+static constexpr auto SESSION_TIMEOUT_CHECK_INTERVAL = std::chrono::minutes(5);
+
 // forward declaration
 class Config;
 
@@ -112,7 +116,7 @@ protected:
     std::chrono::seconds timeout;
 
     /// \brief time of last access to the session, returned by getLastAccessTime()
-    std::chrono::seconds last_access {};
+    std::chrono::seconds last_access = std::chrono::seconds::zero();
 
     /// \brief arbitrary but unique string representing the ID of the session (returned by getID())
     std::string sessionID;
@@ -147,7 +151,8 @@ public:
     std::shared_ptr<Session> createSession(std::chrono::seconds timeout);
 
     /// \brief Returns the instance to a Session with a given sessionID
-    /// \param ID of the Session.
+    /// \param sessionID id of the Session.
+    /// \param doLock lock the thread
     /// \return instance of the Session with a given ID or nullptr if no session with that ID was found.
     std::shared_ptr<Session> getSession(const std::string& sessionID, bool doLock = true) const;
 
