@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    tasks.cc - this file is part of MediaTomb.
+    web/tasks.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,11 +29,15 @@
     $Id$
 */
 
-/// \file tasks.cc
+/// \file web/tasks.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include "content/content_manager.h"
+#include "content/content.h"
+#include "exceptions.h"
+#include "util/generic_task.h"
+#include "util/xml_to_json.h"
 
 void Web::Tasks::process()
 {
@@ -44,14 +50,14 @@ void Web::Tasks::process()
 
     if (action == "list") {
         auto tasksEl = root.append_child("tasks");
-        xml2JsonHints->setArrayName(tasksEl, "tasks");
+        xml2Json->setArrayName(tasksEl, "tasks");
         auto taskList = content->getTasklist();
         for (auto&& task : taskList) {
             appendTask(task, tasksEl);
         }
     } else if (action == "cancel") {
         int taskID = intParam("task_id");
-        content->invalidateTask(taskID);
+        content->invalidateTask(taskID, TaskOwner::ContentManagerTask);
     } else
         throw_std_runtime_error("called with illegal action");
 }
