@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    web_autoscan.cc - this file is part of MediaTomb.
+    web/web_autoscan.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,13 +29,16 @@
     $Id$
 */
 
-/// \file web_autoscan.cc
+/// \file web/web_autoscan.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include "content/autoscan.h"
-#include "content/content_manager.h"
+#include "config/result/autoscan.h"
+#include "content/content.h"
 #include "database/database.h"
+#include "exceptions.h"
+#include "util/xml_to_json.h"
 
 void Web::Autoscan::process()
 {
@@ -82,6 +87,7 @@ void Web::Autoscan::process()
             // add or update
             bool recursive = boolParam("recursive");
             bool hidden = boolParam("hidden");
+            bool followSymlinks = boolParam("followSymlinks");
 
             std::vector<std::string> mediaType;
             if (boolParam("audio"))
@@ -105,19 +111,26 @@ void Web::Autoscan::process()
             if (boolParam("videoMusicVideo"))
                 mediaType.emplace_back("MusicVideo");
             int mt = AutoscanDirectory::makeMediaType(fmt::format("{}", fmt::join(mediaType, "|")));
-            // bool persistent = boolParam("persistent");
+#if 0
+            bool persistent = boolParam("persistent");
+#endif
 
-            AutoscanDirectory::ScanMode scanMode = AutoscanDirectory::remapScanmode(scanModeStr);
-            int interval = intParam("interval", 0);
-            if (scanMode == AutoscanDirectory::ScanMode::Timed && interval <= 0)
+            AutoscanScanMode scanMode = AutoscanDirectory::remapScanmode(scanModeStr);
+            int interval;
+            auto intervalParam = param("interval");
+            parseTime(interval, intervalParam);
+            if (scanMode == AutoscanScanMode::Timed && interval <= 0)
                 throw_std_runtime_error("illegal interval given");
 
             int objectID = fromFs ? content->ensurePathExistence(path) : intParam("object_id");
 
-            // log_info("adding autoscan {}: location={}, scan_mode={}, recursive={}, audio={}, image={}, video={}, interval={}, hidden={}",
-            //     objectID, "", AutoscanDirectory::mapScanmode(scanMode),
-            //     recursive, audio, image, video, interval, hidden);
+            log_debug("adding autoscan {}: location={}, recursive={}, mediaType={}, interval={}, hidden={}",
+                objectID, "", recursive, fmt::join(mediaType, "|"), interval, hidden);
 
+            auto containerMap = AutoscanDirectory::ContainerTypesDefaults;
+            containerMap[AutoscanMediaMode::Audio] = param("ctAudio");
+            containerMap[AutoscanMediaMode::Image] = param("ctImage");
+            containerMap[AutoscanMediaMode::Video] = param("ctVideo");
             auto autoscan = std::make_shared<AutoscanDirectory>(
                 "", // location
                 scanMode,
@@ -125,6 +138,7 @@ void Web::Autoscan::process()
                 false, // persistent
                 interval,
                 hidden,
+                followSymlinks,
                 mt);
             autoscan->setObjectID(objectID);
             content->setAutoscanDirectory(autoscan);
@@ -139,7 +153,7 @@ void Web::Autoscan::process()
         // ---
 
         auto autoscansEl = root.append_child("autoscans");
-        xml2JsonHints->setArrayName(autoscansEl, "autoscan");
+        xml2Json->setArrayName(autoscansEl, "autoscan");
         for (auto&& autoscanDir : autoscanList) {
             auto autoscanEl = autoscansEl.append_child("autoscan");
             autoscanEl.append_attribute("objectID") = autoscanDir->getObjectID();
@@ -147,8 +161,6 @@ void Web::Autoscan::process()
             autoscanEl.append_child("location").append_child(pugi::node_pcdata).set_value(autoscanDir->getLocation().c_str());
             autoscanEl.append_child("scan_mode").append_child(pugi::node_pcdata).set_value(AutoscanDirectory::mapScanmode(autoscanDir->getScanMode()));
             autoscanEl.append_child("from_config").append_child(pugi::node_pcdata).set_value(autoscanDir->persistent() ? "1" : "0");
-            // autoscanEl.append_child("scan_level").append_child(pugi::node_pcdata)
-            //     .set_value(AutoscanDirectory::mapScanlevel(autoscanDir->getScanLevel()).c_str());
         }
     } else
         throw_std_runtime_error("called with illegal action");
@@ -160,6 +172,7 @@ void Web::Autoscan::autoscan2XML(const std::shared_ptr<AutoscanDirectory>& adir,
         element.append_child("scan_mode").append_child(pugi::node_pcdata).set_value("none");
         element.append_child("recursive").append_child(pugi::node_pcdata).set_value("0");
         element.append_child("hidden").append_child(pugi::node_pcdata).set_value("0");
+        element.append_child("followSymlinks").append_child(pugi::node_pcdata).set_value("0");
         element.append_child("interval").append_child(pugi::node_pcdata).set_value("1800");
         element.append_child("persistent").append_child(pugi::node_pcdata).set_value("0");
         element.append_child("mediaType").append_child(pugi::node_pcdata).set_value("-1");
@@ -173,10 +186,15 @@ void Web::Autoscan::autoscan2XML(const std::shared_ptr<AutoscanDirectory>& adir,
         element.append_child("videoMovie").append_child(pugi::node_pcdata).set_value("0");
         element.append_child("videoTV").append_child(pugi::node_pcdata).set_value("0");
         element.append_child("videoMusicVideo").append_child(pugi::node_pcdata).set_value("0");
+
+        element.append_child("ctAudio").append_child(pugi::node_pcdata).set_value(AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Audio).c_str());
+        element.append_child("ctImage").append_child(pugi::node_pcdata).set_value(AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Image).c_str());
+        element.append_child("ctVideo").append_child(pugi::node_pcdata).set_value(AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Video).c_str());
     } else {
         element.append_child("scan_mode").append_child(pugi::node_pcdata).set_value(AutoscanDirectory::mapScanmode(adir->getScanMode()));
         element.append_child("recursive").append_child(pugi::node_pcdata).set_value(adir->getRecursive() ? "1" : "0");
         element.append_child("hidden").append_child(pugi::node_pcdata).set_value(adir->getHidden() ? "1" : "0");
+        element.append_child("followSymlinks").append_child(pugi::node_pcdata).set_value(adir->getFollowSymlinks() ? "1" : "0");
         element.append_child("interval").append_child(pugi::node_pcdata).set_value(fmt::to_string(adir->getInterval().count()).c_str());
         element.append_child("persistent").append_child(pugi::node_pcdata).set_value(adir->persistent() ? "1" : "0");
         element.append_child("mediaType").append_child(pugi::node_pcdata).set_value(fmt::to_string(adir->getMediaType()).c_str());
@@ -190,5 +208,9 @@ void Web::Autoscan::autoscan2XML(const std::shared_ptr<AutoscanDirectory>& adir,
         element.append_child("videoMovie").append_child(pugi::node_pcdata).set_value(adir->hasContent(UPNP_CLASS_VIDEO_MOVIE) ? "1" : "0");
         element.append_child("videoTV").append_child(pugi::node_pcdata).set_value(adir->hasContent(UPNP_CLASS_VIDEO_BROADCAST) ? "1" : "0");
         element.append_child("videoMusicVideo").append_child(pugi::node_pcdata).set_value(adir->hasContent(UPNP_CLASS_VIDEO_MUSICVIDEOCLIP) ? "1" : "0");
+
+        element.append_child("ctAudio").append_child(pugi::node_pcdata).set_value(adir->getContainerTypes().at(AutoscanMediaMode::Audio).c_str());
+        element.append_child("ctImage").append_child(pugi::node_pcdata).set_value(adir->getContainerTypes().at(AutoscanMediaMode::Image).c_str());
+        element.append_child("ctVideo").append_child(pugi::node_pcdata).set_value(adir->getContainerTypes().at(AutoscanMediaMode::Video).c_str());
     }
 }
