@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    web_request_handler.cc - this file is part of MediaTomb.
+    web/web_request_handler.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2024 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,24 +29,37 @@
     $Id$
 */
 
-/// \file web_request_handler.cc
+/// \file web/web_request_handler.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "web_request_handler.h" // API
 
-#include "config/config_manager.h"
-#include "content/content_manager.h"
+#include "common.h"
+#include "config/config.h"
+#include "config/config_val.h"
+#include "content/content.h"
+#include "context.h"
+#include "exceptions.h"
 #include "iohandler/mem_io_handler.h"
-#include "util/tools.h"
-#include "util/upnp_headers.h"
+#include "server.h"
+#include "session_manager.h"
+#include "upnp/compat.h"
+#include "upnp/headers.h"
+#include "upnp/quirks.h"
+#include "util/generic_task.h"
 #include "util/url_utils.h"
 #include "util/xml_to_json.h"
 #include "web/pages.h"
 
 namespace Web {
 
-WebRequestHandler::WebRequestHandler(const std::shared_ptr<ContentManager>& content)
-    : RequestHandler(content)
+WebRequestHandler::WebRequestHandler(const std::shared_ptr<Content>& content,
+    std::shared_ptr<Server> server,
+    const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
+    const std::shared_ptr<Quirks>& quirks)
+    : RequestHandler(content, xmlBuilder, quirks)
     , sessionManager(this->content->getContext()->getSessionManager())
+    , server(std::move(server))
 {
 }
 
@@ -81,13 +96,11 @@ void WebRequestHandler::checkRequest(bool checkLogin)
     session->access();
 }
 
-void WebRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
+bool WebRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
 {
     this->filename = filename;
-
-    auto&& [path, parameters] = URLUtils::splitUrl(filename, URL_UI_PARAM_SEPARATOR);
-
-    auto decodedParams = dictDecode(parameters);
+    auto&& parameters = URLUtils::getQuery(this->filename);
+    auto decodedParams = URLUtils::dictDecode(parameters);
     if (params.empty()) {
         params = std::move(decodedParams);
     } else {
@@ -96,28 +109,31 @@ void WebRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
 
     UpnpFileInfo_set_FileLength(info, -1); // length is unknown
 
-    UpnpFileInfo_set_LastModified(info, 0);
+    UpnpFileInfo_set_LastModified(info, currentTime().count()); // always use current time
     UpnpFileInfo_set_IsDirectory(info, 0);
     UpnpFileInfo_set_IsReadable(info, 1);
 
     std::string contentType = "application/json; charset=UTF-8";
 
-#ifdef USING_NPUPNP
-    info->content_type = std::move(contentType);
-#else
-    UpnpFileInfo_set_ContentType(info, contentType.c_str());
-#endif
     Headers headers;
     headers.addHeader("Cache-Control", "no-cache, must-revalidate");
     headers.addHeader("SameSite", "Lax");
+    GrbUpnpFileInfoSetContentType(info, contentType);
+#ifdef UPNP_NEEDS_CORS
+    headers.addHeader("Access-Control-Allow-Origin", fmt::format("{}", fmt::join(server->getCorsHosts(), " ")));
+#endif
+
+    if (quirks)
+        quirks->updateHeaders(headers);
     headers.writeHeaders(info);
+    return quirks && quirks->getClient();
 }
 
-std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, enum UpnpOpenFileMode mode)
+std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, const std::shared_ptr<Quirks>& quirks, enum UpnpOpenFileMode mode)
 {
     this->filename = filename;
-    auto&& [path, parameters] = URLUtils::splitUrl(filename, URL_UI_PARAM_SEPARATOR);
-    auto decodedParams = dictDecode(parameters);
+    auto&& parameters = URLUtils::getQuery(this->filename);
+    auto decodedParams = URLUtils::dictDecode(parameters);
     if (params.empty()) {
         params = std::move(decodedParams);
     } else {
@@ -130,7 +146,7 @@ std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, enum Up
     decl.append_attribute("encoding") = "UTF-8";
     auto root = xmlDoc->append_child("root");
 
-    xml2JsonHints = std::make_unique<Xml2Json::Hints>();
+    xml2Json = std::make_unique<Xml2Json>();
 
     std::string error;
     int errorCode = 0;
@@ -138,7 +154,7 @@ std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, enum Up
     std::string output;
     // processing page, creating output
     try {
-        if (!config->getBoolOption(CFG_SERVER_UI_ENABLED)) {
+        if (!config->getBoolOption(ConfigVal::SERVER_UI_ENABLED)) {
             log_warning("The UI is disabled in the configuration file. See README.");
             error = "The UI is disabled in the configuration file. See README.";
             errorCode = 900;
@@ -185,7 +201,7 @@ std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, enum Up
     }
 
     try {
-        output = Xml2Json::getJson(root, *xml2JsonHints);
+        output = xml2Json->getJson(root);
     } catch (const std::runtime_error& e) {
         log_error("Web marshalling error: {}", e.what());
     }
