@@ -1,12 +1,35 @@
+/*GRB*
+
+    Gerbera - https://gerbera.io/
+
+    test_resolution.cc - this file is part of Gerbera.
+
+    Copyright (C) 2016-2024 Gerbera Contributors
+
+    Gerbera is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License version 2
+    as published by the Free Software Foundation.
+
+    Gerbera is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with Gerbera.  If not, see <http://www.gnu.org/licenses/>.
+
+    $Id$
+*/
+
+#include "metadata/resolution.h"
+
 #include <gtest/gtest.h>
 
-#include "metadata//resolution.h"
-
 TEST(ResolutionTest, parse) {
-    auto res = Resolution("122586668x54589448448485");
+    auto res = Resolution("122586668x448448485");
 
     EXPECT_EQ(res.x(), 122586668);
-    EXPECT_EQ(res.y(), 54589448448485);
+    EXPECT_EQ(res.y(), 448448485);
 }
 
 TEST(ResolutionTest, parseWithSpace) {
@@ -38,4 +61,4 @@ TEST(ResolutionTest, throwOnBad) {
     EXPECT_THROW(Resolution("0x"), std::runtime_error);
     EXPECT_THROW(Resolution("0x1"), std::runtime_error);
     EXPECT_THROW(Resolution("1x0"), std::runtime_error);
-}
\ No newline at end of file
+}
