@@ -1,10 +1,10 @@
 /*GRB*
 
-Gerbera - https://gerbera.io/
+    Gerbera - https://gerbera.io/
 
     test_database.cc - this file is part of Gerbera.
 
-    Copyright (C) 2020-2022 Gerbera Contributors
+    Copyright (C) 2020-2024 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -17,22 +17,27 @@ Gerbera - https://gerbera.io/
 
     You should have received a copy of the GNU General Public License
     along with Gerbera.  If not, see <http://www.gnu.org/licenses/>.
+
+    $Id$
 */
 
 /// \file test_database.cc
-#include <gtest/gtest.h>
-#include <pugixml.hpp>
-
-#include "cds_objects.h"
+#include "cds/cds_objects.h"
+#include "config/result/autoscan.h"
 #include "database/sqlite3/sqlite_database.h"
+#include "exceptions.h"
 #include "sqlite_config_fake.h"
-#include "upnp_xml.h"
+#include "upnp/xml_builder.h"
+#include "util/tools.h"
 
 #if HAVE_MYSQL
 #include "database/mysql/mysql_database.h"
 #include "mysql_config_fake.h"
 #endif
 
+#include <gtest/gtest.h>
+#include <pugixml.hpp>
+
 class TestSqliteDatabase : public Sqlite3Database {
     friend class Sqlite3DatabaseTest;
 };
@@ -40,12 +45,12 @@ class TestSqliteDatabase : public Sqlite3Database {
 class DatabaseTestBase : public ::testing::Test {
 
 protected:
-    void testUpgrade(config_option_t option);
+    void testUpgrade(ConfigVal option);
     std::shared_ptr<Database> subject;
     std::shared_ptr<Config> config;
 };
 
-void DatabaseTestBase::testUpgrade(config_option_t option)
+void DatabaseTestBase::testUpgrade(ConfigVal option)
 {
     const fs::path& upgradeFile = config->getOption(option);
     pugi::xml_document xmlDoc;
@@ -74,7 +79,7 @@ public:
     void SetUp() override
     {
         config = std::make_shared<SqliteConfigFake>();
-        subject = std::make_shared<Sqlite3Database>(config, nullptr, nullptr);
+        subject = std::make_shared<Sqlite3Database>(config, nullptr, nullptr, nullptr);
     }
 
     void TearDown() override
@@ -85,7 +90,7 @@ public:
 
 TEST_F(Sqlite3DatabaseTest, CheckInitScript)
 {
-    auto sqlFilePath = config->getOption(CFG_SERVER_STORAGE_SQLITE_INIT_SQL_FILE);
+    auto sqlFilePath = config->getOption(ConfigVal::SERVER_STORAGE_SQLITE_INIT_SQL_FILE);
     auto sql = GrbFile(sqlFilePath).readTextFile();
     auto&& myHash = stringHash(sql);
 
@@ -94,7 +99,7 @@ TEST_F(Sqlite3DatabaseTest, CheckInitScript)
 
 TEST_F(Sqlite3DatabaseTest, CheckUpgradeCommands)
 {
-    testUpgrade(CFG_SERVER_STORAGE_SQLITE_UPGRADE_FILE);
+    testUpgrade(ConfigVal::SERVER_STORAGE_SQLITE_UPGRADE_FILE);
 }
 
 // test is blocking on CONAN
@@ -142,7 +147,7 @@ public:
     void SetUp() override
     {
         config = std::make_shared<MySQLConfigFake>();
-        subject = std::make_shared<MySQLDatabase>(config, nullptr);
+        subject = std::make_shared<MySQLDatabase>(config, nullptr, nullptr);
     }
 
     void TearDown() override
@@ -152,12 +157,12 @@ public:
 
 TEST_F(MysqlDatabaseTest, CheckUpgradeCommands)
 {
-    testUpgrade(CFG_SERVER_STORAGE_MYSQL_UPGRADE_FILE);
+    testUpgrade(ConfigVal::SERVER_STORAGE_MYSQL_UPGRADE_FILE);
 }
 
 TEST_F(MysqlDatabaseTest, CheckInitScript)
 {
-    auto sqlFilePath = config->getOption(CFG_SERVER_STORAGE_MYSQL_INIT_SQL_FILE);
+    auto sqlFilePath = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_INIT_SQL_FILE);
     auto sql = GrbFile(sqlFilePath).readTextFile();
     auto&& myHash = stringHash(sql);
 
