@@ -1,12 +1,30 @@
+/*GRB*
+    Gerbera - https://gerbera.io/
+
+    database_mock.h - this file is part of Gerbera.
+
+    Copyright (C) 2016-2024 Gerbera Contributors
+
+    Gerbera is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License version 2
+    as published by the Free Software Foundation.
+
+    Gerbera is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with Gerbera.  If not, see <http://www.gnu.org/licenses/>.
+
+    $Id$
+*/
 #ifndef __DATABASE_MOCK_H__
 #define __DATABASE_MOCK_H__
 
-#include <gtest/gtest.h>
-
-#include "config/config.h"
-#include "config/config_setup.h"
 #include "database/database.h"
-#include "util/upnp_clients.h"
+
+#include <gtest/gtest.h>
 
 class DatabaseMock : public Database {
 public:
@@ -29,8 +47,8 @@ public:
 
     std::vector<std::string> getMimeTypes() override { return {}; }
     std::vector<std::shared_ptr<CdsObject>> findObjectByContentClass(const std::string& contentClass) override { return {}; }
-    std::shared_ptr<CdsObject> findObjectByPath(const fs::path& path, bool wasRegularFile = false) override { return {}; }
-    int findObjectIDByPath(const fs::path& fullpath, bool wasRegularFile = false) override { return INVALID_OBJECT_ID; }
+    std::shared_ptr<CdsObject> findObjectByPath(const fs::path& path, DbFileType fileType = DbFileType::Auto) override { return {}; }
+    int findObjectIDByPath(const fs::path& fullpath, DbFileType fileType = DbFileType::Auto) override { return INVALID_OBJECT_ID; }
     std::string incrementUpdateIDs(const std::unordered_set<int>& ids) override { return {}; }
 
     std::shared_ptr<CdsObject> loadObject(int objectID) override { return nullptr; }
@@ -38,14 +56,18 @@ public:
     int getChildCount(int contId, bool containers = true, bool items = true, bool hideFsRoot = false) override { return 0; }
     std::map<int, int> getChildCounts(const std::vector<int>& contId, bool containers, bool items, bool hideFsRoot) override { return {}; }
 
-    std::unique_ptr<ChangedContainers> removeObject(int objectID, bool all) override { return {}; }
-    std::unordered_set<int> getObjects(int parentID, bool withoutContainer) override { return {}; }
+    std::unique_ptr<ChangedContainers> removeObject(int objectID, const fs::path& path, bool all) override { return {}; }
+    std::size_t getObjects(int parentID, bool withoutContainer, std::unordered_set<int>& ret, bool full) override { return 0; }
+    std::vector<int> getRefObjects(int objectId) override { return {}; }
+    std::unordered_set<int> getUnreferencedObjects() override { return {}; }
+
     std::unique_ptr<ChangedContainers> removeObjects(const std::unordered_set<int>& list, bool all = false) override { return {}; }
 
     std::shared_ptr<CdsObject> loadObjectByServiceID(const std::string& serviceID) override { return {}; }
     std::vector<int> getServiceObjectIDs(char servicePrefix) override { return {}; }
 
-    int getTotalFiles(bool isVirtual = false, const std::string& mimeType = "", const std::string& upnpClass = "") override { return 0; }
+    long long getFileStats(const StatsParam& stats) override { return 0; }
+    std::map<std::string, long long> getGroupStats(const StatsParam& stats) override { return {}; };
 
     std::string getInternalSetting(const std::string& key) override { return {}; }
     void storeInternalSetting(const std::string& key, const std::string& value) override { }
@@ -55,15 +77,15 @@ public:
     void updateConfigValue(const std::string& key, const std::string& item, const std::string& value, const std::string& status = "unchanged") override { }
 
     /* clients methods */
-    std::vector<ClientCacheEntry> getClients() override { return {}; }
-    void saveClients(const std::vector<ClientCacheEntry>& cache) override { }
+    std::vector<ClientObservation> getClients() override { return {}; }
+    void saveClients(const std::vector<ClientObservation>& cache) override { }
     std::shared_ptr<ClientStatusDetail> getPlayStatus(const std::string& group, int objectId) override { return {}; };
     void savePlayStatus(const std::shared_ptr<ClientStatusDetail>& detail) override { };
     std::vector<std::shared_ptr<ClientStatusDetail>> getPlayStatusList(int objectId) override { return {}; }
     std::vector<std::map<std::string, std::string>> getClientGroupStats() override { return {}; }
 
-    std::shared_ptr<AutoscanList> getAutoscanList(AutoscanDirectory::ScanMode scanode) override { return {}; }
-    void updateAutoscanList(AutoscanDirectory::ScanMode scanmode, const std::shared_ptr<AutoscanList>& list) override { }
+    std::shared_ptr<AutoscanList> getAutoscanList(AutoscanScanMode scanode) override { return {}; }
+    void updateAutoscanList(AutoscanScanMode scanmode, const std::shared_ptr<AutoscanList>& list) override { }
 
     std::shared_ptr<AutoscanDirectory> getAutoscanDirectory(int objectID) override { return {}; }
     void addAutoscanDirectory(const std::shared_ptr<AutoscanDirectory>& adir) override { }
