@@ -1,3 +1,25 @@
+/*GRB*
+
+    Gerbera - https://gerbera.io/
+
+    duk_helper.h - this file is part of Gerbera.
+
+    Copyright (C) 2016-2024 Gerbera Contributors
+
+    Gerbera is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License version 2
+    as published by the Free Software Foundation.
+
+    Gerbera is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with Gerbera.  If not, see <http://www.gnu.org/licenses/>.
+
+    $Id$
+*/
 #ifdef HAVE_JS
 
 #ifndef GERBERA_DUK_HELPER_H
@@ -33,6 +55,9 @@ public:
     // Adds a new object to the duk_context with meta data
     static void createObject(duk_context* ctx, const std::map<std::string, std::string>& objectProps, const std::map<std::string, std::string>& metaProps);
 
+    // Print error message based on duktape
+    static void printError(duk_context* ctx, const std::string& message, const std::string& item);
+
 private:
     // Extracts the property value given a complex associative array key
     // Example: key = `meta['dc:title']`
