@@ -1,9 +1,26 @@
-#ifdef HAVE_JS
+/*GRB*
 
-#include <duktape.h>
-#include <gmock/gmock.h>
-#include <gtest/gtest.h>
-#include <memory>
+    Gerbera - https://gerbera.io/
+
+    test_import_script.cc - this file is part of Gerbera.
+
+    Copyright (C) 2020-2024 Gerbera Contributors
+
+    Gerbera is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License version 2
+    as published by the Free Software Foundation.
+
+    Gerbera is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with Gerbera.  If not, see <http://www.gnu.org/licenses/>.
+
+    $Id$
+*/
+#ifdef HAVE_JS
 
 #include "content/onlineservice/online_service.h"
 #include "metadata/metadata_handler.h"
@@ -13,37 +30,18 @@
 #include "mock/duk_helper.h"
 #include "mock/script_test_fixture.h"
 
-class ImportScriptTest : public ScriptTestFixture {
+#include <duktape.h>
+#include <fmt/format.h>
+#include <memory>
+
+class ImportScriptTest : public CommonScriptTestFixture {
 public:
     ImportScriptTest()
     {
-        commonScriptMock = std::make_unique<::testing::NiceMock<CommonScriptMock>>();
         scriptName = "import.js";
     }
-
-    ~ImportScriptTest() override
-    {
-        commonScriptMock.reset();
-    }
-
-    // As Duktape requires static methods, so must the mock expectations be
-    static std::unique_ptr<CommonScriptMock> commonScriptMock;
 };
 
-std::unique_ptr<CommonScriptMock> ImportScriptTest::commonScriptMock;
-
-static duk_ret_t print(duk_context* ctx)
-{
-    std::string msg = ScriptTestFixture::print(ctx);
-    return ImportScriptTest::commonScriptMock->print(msg);
-}
-
-static duk_ret_t getPlaylistType(duk_context* ctx)
-{
-    std::string playlistMimeType = ScriptTestFixture::getPlaylistType(ctx);
-    return ImportScriptTest::commonScriptMock->getPlaylistType(playlistMimeType);
-}
-
 static duk_ret_t addContainerTree(duk_context* ctx)
 {
     std::map<std::string, std::string> map {
@@ -53,6 +51,7 @@ static duk_ret_t addContainerTree(duk_context* ctx)
         { "/Audio/All - full name", "44" },
         { "/Audio/Artists/Artist/All - full name", "45" },
         { "/Audio/Artists/Artist/Album", "46" },
+        { "/Audio/Artists/Artist/Album Chronology/2018 - Album", "462" },
         { "/Audio/Albums/Album", "47" },
         { "/Audio/Genres/Genre", "481" },
         { "/Audio/Genres/Genre2", "482" },
@@ -73,18 +72,6 @@ static duk_ret_t addContainerTree(duk_context* ctx)
     return ImportScriptTest::commonScriptMock->addContainerTree(tree);
 }
 
-static duk_ret_t createContainerChain(duk_context* ctx)
-{
-    std::vector<std::string> array = ScriptTestFixture::createContainerChain(ctx);
-    return ImportScriptTest::commonScriptMock->createContainerChain(array);
-}
-
-static duk_ret_t getLastPath(duk_context* ctx)
-{
-    std::string inputPath = ScriptTestFixture::getLastPath(ctx);
-    return ImportScriptTest::commonScriptMock->getLastPath(inputPath);
-}
-
 static duk_ret_t addCdsObject(duk_context* ctx)
 {
     std::vector<std::string> keys {
@@ -120,10 +107,11 @@ static duk_ret_t getRootPath(duk_context* ctx)
 // that are called from the import.js script
 // * These are static methods, which makes mocking difficult.
 static duk_function_list_entry js_global_functions[] = {
-    { "print", print, DUK_VARARGS },
-    { "getPlaylistType", getPlaylistType, 1 },
-    { "createContainerChain", createContainerChain, 1 },
-    { "getLastPath", getLastPath, 1 },
+    { "print", CommonScriptTestFixture::js_print, DUK_VARARGS },
+    { "print2", CommonScriptTestFixture::js_print2, DUK_VARARGS },
+    { "getPlaylistType", CommonScriptTestFixture::js_getPlaylistType, 1 },
+    { "createContainerChain", CommonScriptTestFixture::js_createContainerChain, 1 },
+    { "getLastPath", CommonScriptTestFixture::js_getLastPath, 1 },
     { "addCdsObject", addCdsObject, 3 },
     { "getYear", getYear, 1 },
     { "getRootPath", getRootPath, 2 },
@@ -131,6 +119,46 @@ static duk_function_list_entry js_global_functions[] = {
     { nullptr, nullptr, 0 },
 };
 
+static const std::vector<boxConfig> audioBox {
+    { "Audio/allAlbums", "Albums", "object.container" },
+    { "Audio/allArtists", "Artists", "object.container" },
+    { "Audio/allAudio", "All Audio", "object.container" },
+    { "Audio/allComposers", "Composers", "object.container" },
+    { "Audio/allDirectories", "Directories", "object.container" },
+    { "Audio/allGenres", "Genres", "object.container" },
+    { "Audio/allSongs", "All Songs", "object.container" },
+    { "Audio/allTracks", "All - full name", "object.container" },
+    { "Audio/allYears", "Year", "object.container" },
+    { "Audio/audioRoot", "Audio", "object.container" },
+    { "Audio/artistChronology", "Album Chronology", "object.container" },
+};
+static const std::vector<boxConfig> videoBox {
+    { "Video/allDates", "Date", "object.container" },
+    { "Video/allDirectories", "Directories", "object.container" },
+    { "Video/allVideo", "All Video", "object.container" },
+    { "Video/allYears", "Year", "object.container" },
+    { "Video/unknown", "Unknown", "object.container" },
+    { "Video/videoRoot", "Video", "object.container" },
+    { "Video/allDates", "Date", "object.container" },
+};
+static const std::vector<boxConfig> imageBox {
+    { "Image/allDates", "Date", "object.container" },
+    { "Image/allDirectories", "Directories", "object.container" },
+    { "Image/allImages", "All Photos", "object.container" },
+    { "Image/allYears", "Year", "object.container" },
+    { "Image/imageRoot", "Photos", "object.container" },
+    { "Image/unknown", "Unknown", "object.container" },
+};
+static const std::vector<boxConfig> trailerBox {
+    { "Trailer/trailerRoot", "Online Services", "object.container" },
+    { "Trailer/appleTrailers", "Apple Trailers", "object.container" },
+    { "Trailer/allTrailers", "All Trailers", "object.container" },
+    { "Trailer/allGenres", "Genres", "object.container" },
+    { "Trailer/relDate", "Release Date", "object.container" },
+    { "Trailer/postDate", "Post Date", "object.container" },
+    { "Trailer/unknown", "Unknown", "object.container" },
+};
+
 template <typename Map>
 bool mapCompare(Map const& lhs, Map const& rhs)
 {
@@ -211,6 +239,7 @@ TEST_F(ImportScriptTest, AddsAudioItemToVariousCdsContainerChains)
     // for verification.
     EXPECT_CALL(*commonScriptMock, getPlaylistType(Eq("audio/mpeg"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, getYear(Eq("2018-01-01"))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, getRootPath("object/script/path", location)).WillOnce(Return(1));
 
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Audio", "All Audio"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asAudioAllAudio), "42", UNDEFINED)).WillOnce(Return(0));
@@ -241,7 +270,10 @@ TEST_F(ImportScriptTest, AddsAudioItemToVariousCdsContainerChains)
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Audio", "Year", "2018"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asAudioAllAudio), "50", UNDEFINED)).WillOnce(Return(0));
 
-    addGlobalFunctions(ctx, js_global_functions);
+    EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Audio", "Artists", artist, "Album Chronology", "2018 - Album"))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asAudioAllAudio), "462", UNDEFINED)).WillOnce(Return(0));
+
+    addGlobalFunctions(ctx, js_global_functions, {}, audioBox);
     dukMockItem(ctx, mimetype, id, theora, title, meta, aux, res, location, onlineService);
     executeScript(ctx);
 }
@@ -252,7 +284,11 @@ TEST_F(ImportScriptTest, AddsVideoItemToCdsContainerChainWithDirs)
     std::string mimetype = "video/mpeg";
     std::string id = "2";
     std::string location = "/home/gerbera/video.mp4";
-    auto onlineService = int(OS_None);
+#ifdef ONLINE_SERVICES
+    auto onlineService = to_underlying(OnlineServiceType::None);
+#else
+    auto onlineService = 0;
+#endif
     int theora = 0;
     std::map<std::string, std::string> aux;
     std::vector<std::pair<std::string, std::string>> meta;
@@ -276,57 +312,7 @@ TEST_F(ImportScriptTest, AddsVideoItemToCdsContainerChainWithDirs)
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Video", "Directories", "home", "gerbera"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asVideoAllVideo), "61", UNDEFINED)).WillOnce(Return(0));
 
-    addGlobalFunctions(ctx, js_global_functions);
-    dukMockItem(ctx, mimetype, id, theora, title, meta, aux, res, location, onlineService);
-    executeScript(ctx);
-}
-
-TEST_F(ImportScriptTest, AddsAppleTrailerVideoItemToCdsContainerChains)
-{
-    std::string title = "Video Title";
-    std::string mimetype = "video/mpeg";
-    std::string date = "2018-01-01";
-    std::string genre = "Genre";
-    std::string id = "2";
-    std::string location = "/home/gerbera/video.mp4";
-    auto onlineService = int(OS_ATrailers);
-    int theora = 0;
-    std::map<std::string, std::string> res;
-
-    std::vector<std::pair<std::string, std::string>> meta {
-        { "dc:date", date },
-        { "upnp:genre", genre },
-    };
-
-    std::map<std::string, std::string> aux {
-        { "T0", date },
-    };
-
-    // Represents the values passed to `addCdsObject`, extracted from keys defined there.
-    std::map<std::string, std::string> asVideoAllVideo {
-        { "title", title },
-        { "metaData['dc:date']", date },
-        { "metaData['upnp:genre']", genre },
-    };
-
-    // Expecting the common script calls
-    // and will proxy through the mock objects
-    // for verification.
-    EXPECT_CALL(*commonScriptMock, getPlaylistType(Eq("video/mpeg"))).WillOnce(Return(1));
-
-    EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Online Services", "Apple Trailers", "All Trailers"))).WillOnce(Return(1));
-    EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asVideoAllVideo), "80", UNDEFINED)).WillOnce(Return(0));
-
-    EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Online Services", "Apple Trailers", "Genres", "Genre"))).WillOnce(Return(1));
-    EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asVideoAllVideo), "81", UNDEFINED)).WillOnce(Return(0));
-
-    EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Online Services", "Apple Trailers", "Release Date", "2018-01"))).WillOnce(Return(1));
-    EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asVideoAllVideo), "82", UNDEFINED)).WillOnce(Return(0));
-
-    EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Online Services", "Apple Trailers", "Post Date", "2018-01"))).WillOnce(Return(1));
-    EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asVideoAllVideo), "83", UNDEFINED)).WillOnce(Return(0));
-
-    addGlobalFunctions(ctx, js_global_functions);
+    addGlobalFunctions(ctx, js_global_functions, {}, videoBox);
     dukMockItem(ctx, mimetype, id, theora, title, meta, aux, res, location, onlineService);
     executeScript(ctx);
 }
@@ -338,7 +324,11 @@ TEST_F(ImportScriptTest, AddsImageItemToCdsContainerChains)
     std::string date = "2018-01-01";
     std::string id = "2";
     std::string location = "/home/gerbera/image.jpg";
-    auto onlineService = int(OS_ATrailers);
+#ifdef ONLINE_SERVICES
+    auto onlineService = to_underlying(OnlineServiceType::None);
+#else
+    auto onlineService = 0;
+#endif
     int theora = 0;
 
     std::vector<std::pair<std::string, std::string>> meta {
@@ -373,7 +363,7 @@ TEST_F(ImportScriptTest, AddsImageItemToCdsContainerChains)
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Photos", "Directories", "home", "gerbera"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asImagePhotos), "73", UNDEFINED)).WillOnce(Return(0));
 
-    addGlobalFunctions(ctx, js_global_functions);
+    addGlobalFunctions(ctx, js_global_functions, {}, imageBox);
     dukMockItem(ctx, mimetype, id, theora, title, meta, aux, res, location, onlineService);
     executeScript(ctx);
 }
@@ -384,7 +374,11 @@ TEST_F(ImportScriptTest, AddsOggTheoraVideoItemToCdsContainerChainWithDirs)
     std::string mimetype = "application/ogg";
     std::string id = "2";
     std::string location = "/home/gerbera/video.ogg";
-    auto onlineService = int(OS_None);
+#ifdef ONLINE_SERVICES
+    auto onlineService = to_underlying(OnlineServiceType::None);
+#else
+    auto onlineService = 0;
+#endif
     int theora = 1;
     std::map<std::string, std::string> aux;
     std::vector<std::pair<std::string, std::string>> meta;
@@ -408,7 +402,7 @@ TEST_F(ImportScriptTest, AddsOggTheoraVideoItemToCdsContainerChainWithDirs)
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Video", "Directories", "home", "gerbera"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asVideoAllVideo), "61", UNDEFINED)).WillOnce(Return(0));
 
-    addGlobalFunctions(ctx, js_global_functions);
+    addGlobalFunctions(ctx, js_global_functions, {}, videoBox);
     dukMockItem(ctx, mimetype, id, theora, title, meta, aux, res, location, onlineService);
     executeScript(ctx);
 }
@@ -474,6 +468,7 @@ TEST_F(ImportScriptTest, AddsOggTheoraAudioItemToVariousCdsContainerChains)
     // for verification.
     EXPECT_CALL(*commonScriptMock, getPlaylistType(Eq("application/ogg"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, getYear(Eq("2018-01-01"))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, getRootPath("object/script/path", location)).WillOnce(Return(1));
 
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Audio", "All Audio"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asAudioAllAudio), "42", UNDEFINED)).WillOnce(Return(0));
@@ -502,7 +497,10 @@ TEST_F(ImportScriptTest, AddsOggTheoraAudioItemToVariousCdsContainerChains)
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Audio", "Year", "2018"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asAudioAllAudio), "50", UNDEFINED)).WillOnce(Return(0));
 
-    addGlobalFunctions(ctx, js_global_functions);
+    EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Audio", "Artists", artist, "Album Chronology", "2018 - Album"))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, addCdsObject(IsIdenticalMap(asAudioAllAudio), "462", UNDEFINED)).WillOnce(Return(0));
+
+    addGlobalFunctions(ctx, js_global_functions, {}, audioBox);
     dukMockItem(ctx, mimetype, id, theora, title, meta, aux, res, location, onlineService);
     executeScript(ctx);
 }
