@@ -1,51 +1,50 @@
+/*GRB*
+
+    Gerbera - https://gerbera.io/
+
+    test_internal_m3u8_playlist.cc - this file is part of Gerbera.
+
+    Copyright (C) 2016-2024 Gerbera Contributors
+
+    Gerbera is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License version 2
+    as published by the Free Software Foundation.
+
+    Gerbera is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with Gerbera.  If not, see <http://www.gnu.org/licenses/>.
+
+    $Id$
+*/
 #ifdef HAVE_JS
 
-#include <duktape.h>
-#include <gmock/gmock.h>
-#include <gtest/gtest.h>
-#include <memory>
-
-#include "cds_objects.h"
+#include "cds/cds_objects.h"
 #include "util/string_converter.h"
 
 #include "mock/common_script_mock.h"
 #include "mock/duk_helper.h"
 #include "mock/script_test_fixture.h"
 
+#include <duktape.h>
+#include <memory>
+
 // Extends ScriptTestFixture to allow
 // for unique testing of the Internal URL M3U8 Playlist
 // processing
-class InternalUrlM3U8PlaylistTest : public ScriptTestFixture {
+class InternalUrlM3U8PlaylistTest : public CommonScriptTestFixture {
 public:
-    // As Duktape requires static methods, so must the mock expectations be
-    static std::unique_ptr<CommonScriptMock> commonScriptMock;
-
     InternalUrlM3U8PlaylistTest()
     {
-        commonScriptMock = std::make_unique<::testing::NiceMock<CommonScriptMock>>();
         scriptName = "playlists.js";
-    }
-
-    ~InternalUrlM3U8PlaylistTest() override
-    {
-        commonScriptMock.reset();
+        functionName = "importPlaylist";
+        objectName = "playlist";
     }
 };
 
-std::unique_ptr<CommonScriptMock> InternalUrlM3U8PlaylistTest::commonScriptMock;
-
-static duk_ret_t getPlaylistType(duk_context* ctx)
-{
-    std::string playlistMimeType = ScriptTestFixture::getPlaylistType(ctx);
-    return InternalUrlM3U8PlaylistTest::commonScriptMock->getPlaylistType(playlistMimeType);
-}
-
-static duk_ret_t print(duk_context* ctx)
-{
-    std::string msg = ScriptTestFixture::print(ctx);
-    return InternalUrlM3U8PlaylistTest::commonScriptMock->print(msg);
-}
-
 static duk_ret_t addContainerTree(duk_context* ctx)
 {
     std::map<std::string, std::string> map {
@@ -57,18 +56,6 @@ static duk_ret_t addContainerTree(duk_context* ctx)
     return InternalUrlM3U8PlaylistTest::commonScriptMock->addContainerTree(tree);
 }
 
-static duk_ret_t createContainerChain(duk_context* ctx)
-{
-    std::vector<std::string> array = ScriptTestFixture::createContainerChain(ctx);
-    return InternalUrlM3U8PlaylistTest::commonScriptMock->createContainerChain(array);
-}
-
-static duk_ret_t getLastPath(duk_context* ctx)
-{
-    std::string inputPath = ScriptTestFixture::getLastPath(ctx);
-    return InternalUrlM3U8PlaylistTest::commonScriptMock->getLastPath(inputPath);
-}
-
 // Proxy the Duktape script with `readln`
 // Mimics reading the playlist file line by line
 // Uses the `CommonScriptMock` to track expectations
@@ -130,10 +117,12 @@ static duk_ret_t getCdsObject(duk_context* ctx)
 // that are called from the playlists.js script
 // * These are static methods, which makes mocking difficult.
 static duk_function_list_entry js_global_functions[] = {
-    { "print", print, DUK_VARARGS },
-    { "getPlaylistType", getPlaylistType, 1 },
-    { "createContainerChain", createContainerChain, 1 },
-    { "getLastPath", getLastPath, 1 },
+    { "print", CommonScriptTestFixture::js_print, DUK_VARARGS },
+    { "print2", CommonScriptTestFixture::js_print2, DUK_VARARGS },
+    { "getPlaylistType", CommonScriptTestFixture::js_getPlaylistType, 1 },
+    { "createContainerChain", CommonScriptTestFixture::js_createContainerChain, 1 },
+    { "getLastPath", CommonScriptTestFixture::js_getLastPath, 1 },
+    { "getLastPath2", CommonScriptTestFixture::js_getLastPath2, 2 },
     { "readln", readln, 1 },
     { "addCdsObject", addCdsObject, 3 },
     { "copyObject", copyObject, 1 },
@@ -142,6 +131,12 @@ static duk_function_list_entry js_global_functions[] = {
     { nullptr, nullptr, 0 },
 };
 
+static const std::vector<boxConfig> playlistBox {
+    { "Playlist/playlistRoot", "Playlists", "object.container" },
+    { "Playlist/allPlaylists", "All Playlists", "object.container" },
+    { "Playlist/allDirectories", "Directories", "object.container" },
+};
+
 template <typename Map>
 bool mapCompare(Map const& lhs, Map const& rhs)
 {
@@ -180,9 +175,10 @@ TEST_F(InternalUrlM3U8PlaylistTest, AddsCdsObjectFromPlaylistWithInternalUrlPlay
     // and will proxy through the mock objects
     // for verification.
     EXPECT_CALL(*commonScriptMock, getPlaylistType(Eq("audio/x-mpegurl"))).WillOnce(Return(1));
-    EXPECT_CALL(*commonScriptMock, print(Eq("Processing playlist: /location/of/playlist.m3u8"))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, print2(Eq("Info"), Eq("Processing playlist: /location/of/playlist.m3u8"))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, print2(Eq("Debug"), Eq("Playlist 'example\xE2\x9C\x85.mp3' Adding entry: 1 /home/gerbera/example\xE2\x9C\x85.mp3"))).WillRepeatedly(Return(1));
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Playlists", "All Playlists", "Playlist Title"))).WillOnce(Return(1));
-    EXPECT_CALL(*commonScriptMock, getLastPath(Eq("/location/of/playlist.m3u8"))).WillOnce(Return(1));
+    EXPECT_CALL(*commonScriptMock, getLastPath2(Eq("/location/of/playlist.m3u8"),Eq(1))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, addContainerTree(ElementsAre("Playlists", "Directories", "of", "Playlist Title"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, readln(Eq("#EXTM3U"))).WillOnce(Return(1));
     EXPECT_CALL(*commonScriptMock, readln(Eq("#EXTINF:123, Example Artist, Thumbs Up Inc. ðŸ‘"))).WillOnce(Return(1));
@@ -193,9 +189,7 @@ TEST_F(InternalUrlM3U8PlaylistTest, AddsCdsObjectFromPlaylistWithInternalUrlPlay
     EXPECT_CALL(*commonScriptMock, getCdsObject(Eq("/home/gerbera/exampleâœ….mp3"))).WillRepeatedly(Return(1));
     EXPECT_CALL(*commonScriptMock, copyObject(Eq(true))).WillRepeatedly(Return(1));
 
-    addGlobalFunctions(ctx, js_global_functions);
-    dukMockPlaylist(ctx, "Playlist Title", "/location/of/playlist.m3u8", "audio/x-mpegurl");
-
-    executeScript(ctx);
+    addGlobalFunctions(ctx, js_global_functions, {}, playlistBox);
+    callFunction(ctx, dukMockPlaylist, {{"title", "Playlist Title"}, {"location", "/location/of/playlist.m3u8"}, {"mimetype", "audio/x-mpegurl"}});
 }
 #endif
