@@ -1,8 +1,51 @@
+/*GRB*
+    Gerbera - https://gerbera.io/
+
+    test_tools.cc - this file is part of Gerbera.
+
+    Copyright (C) 2016-2024 Gerbera Contributors
+
+    Gerbera is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License version 2
+    as published by the Free Software Foundation.
+
+    Gerbera is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with Gerbera.  If not, see <http://www.gnu.org/licenses/>.
+
+    $Id$
+*/
+
 #include "util/grb_fs.h"
+#include "util/grb_net.h"
+#include "util/grb_time.h"
 #include "util/tools.h"
+#include "util/url_utils.h"
 
 #include <gtest/gtest.h>
 
+TEST(ToolsTest, simpleDate)
+{
+    std::string dt = "2009-08-22T18:51:40+0200";
+    EXPECT_EQ(makeSimpleDate(dt), "2009-08-22T16:51:40Z");
+    dt = "2009-08-22T18:51:40-0100";
+    EXPECT_EQ(makeSimpleDate(dt), "2009-08-22T19:51:40Z");
+    dt = "2009-08-22T18:51:40-0500";
+    EXPECT_EQ(makeSimpleDate(dt), "2009-08-22T23:51:40Z");
+    dt = "2009-08-22T18:51:40-0900";
+    EXPECT_EQ(makeSimpleDate(dt), "2009-08-23T03:51:40Z");
+    dt = "2009-08-22T18:51:40+0100";
+    EXPECT_EQ(makeSimpleDate(dt), "2009-08-22T17:51:40Z");
+    dt = "2009-08-22T00:51:40+0930";
+    EXPECT_EQ(makeSimpleDate(dt), "2009-08-21T15:21:40Z");
+    dt = "2009-08-22T00:01:41+0930";
+    EXPECT_EQ(makeSimpleDate(dt), "2009-08-21T14:31:41Z");
+}
+
 TEST(ToolsTest, millisecondsToHMSF)
 {
     EXPECT_EQ(millisecondsToHMSF(0), "0:00:00.000");
@@ -22,7 +65,7 @@ TEST(ToolsTest, readWriteBinaryFile)
 
     std::vector<std::byte> expected(source.size());
     std::transform(source.begin(), source.end(), expected.begin(),
-        [](char c) { return std::byte(c); });
+        [](char c) { return static_cast<std::byte>(c); });
 
     EXPECT_EQ(*result, expected);
     fs::remove(testFile);
@@ -46,14 +89,36 @@ TEST(ToolsTest, writeFileThrowsIfCantOpenFile)
     EXPECT_THROW(GrbFile(testFile).writeBinaryFile(data.data(), data.size()), std::runtime_error);
 }
 
+TEST(ToolsTest, parseIpV6)
+{
+    auto addr = std::make_shared<GrbNet>("2001:0db8:85a3:0000:0000:8a2e:0370:7334");
+    EXPECT_TRUE(GrbNet("2001:0db8:85a3:0000:0000:8a2e:0370:7334%eth0").equals(addr));
+}
+
 TEST(ToolsTest, renderWebUriV4)
 {
-    EXPECT_EQ(renderWebUri("192.168.5.5", 7777), "192.168.5.5:7777");
+    EXPECT_EQ(GrbNet::renderWebUri("192.168.5.5", 7777), "192.168.5.5:7777");
 }
 
 TEST(ToolsTest, renderWebUriV6)
 {
-    EXPECT_EQ(renderWebUri("2001:0db8:85a3:0000:0000:8a2e:0370:7334", 7777), "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]:7777");
+    EXPECT_EQ(GrbNet::renderWebUri("2001:0db8:85a3:0000:0000:8a2e:0370:7334", 7777), "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]:7777");
+}
+
+TEST(ToolsTest, pathTrimTest)
+{
+    std::string test = "/regular/path";
+    EXPECT_EQ(rtrimPath(test), test);
+    std::string test2 = "/regular/path//";
+    EXPECT_EQ(rtrimPath(test2), test);
+}
+
+TEST(ToolsTest, subdirTest)
+{
+    std::string test = "/regular/path";
+    std::string test2 = "/regular/path/sub/";
+    EXPECT_EQ(isSubDir(test, test2), false);
+    EXPECT_EQ(isSubDir(test2, test), true);
 }
 
 TEST(ToolsTest, splitStringTest)
@@ -152,47 +217,110 @@ TEST(ToolsTest, dictEncodeTest)
     std::map<std::string, std::string> values;
     values.emplace("Key 1", "Value 1");
     values.emplace("Key2", "Value 2");
-    EXPECT_EQ(dictEncode(values), "Key%201=Value%201&Key2=Value%202");
+    EXPECT_EQ(URLUtils::dictEncode(values), "Key%201=Value%201&Key2=Value%202");
 }
 
 TEST(ToolsTest, dictDecodeTest)
 {
-    std::map<std::string, std::string> values = dictDecode("Key%201=Value%201&Key2=Value%202");
+    std::map<std::string, std::string> values = URLUtils::dictDecode("Key%201=Value%201&Key2=Value%202");
     ASSERT_EQ(values.size(), 2);
     EXPECT_EQ(values["Key 1"], "Value 1");
     EXPECT_EQ(values["Key2"], "Value 2");
 
-    values = dictDecode("");
+    values = URLUtils::dictDecode("");
     EXPECT_EQ(values.size(), 0);
 }
 
 TEST(ToolsTest, pathToMapTest)
 {
-    std::map<std::string, std::string> values = pathToMap("Key 1/Value 1/Key 2/Value 2/Key3//");
+    std::map<std::string, std::string> values = URLUtils::pathToMap("Key 1/Value 1/Key 2/Value 2/Key3//");
     ASSERT_EQ(values.size(), 3);
     EXPECT_EQ(values["Key 1"], "Value 1");
     EXPECT_EQ(values["Key 2"], "Value 2");
     EXPECT_EQ(values["Key3"], "");
 
-    values = pathToMap("Key 1/Value 1/Key 2/");
+    values = URLUtils::pathToMap("Key 1/Value 1/Key 2/");
     ASSERT_EQ(values.size(), 2);
     EXPECT_EQ(values["Key 1"], "Value 1");
     EXPECT_EQ(values["Key 2"], "");
 
-    values = pathToMap("Key 1/Value 1/Key 2");
+    values = URLUtils::pathToMap("Key 1/Value 1/Key 2");
     ASSERT_EQ(values.size(), 2);
     EXPECT_EQ(values["Key 1"], "Value 1");
     EXPECT_EQ(values["Key 2"], "");
 
-    values = pathToMap("Key 1/");
+    values = URLUtils::pathToMap("Key 1/");
     ASSERT_EQ(values.size(), 1);
     EXPECT_EQ(values["Key 1"], "");
 
-    values = pathToMap("/");
+    values = URLUtils::pathToMap("/");
     ASSERT_EQ(values.size(), 1);
     EXPECT_EQ(values[""], "");
 
-    values = pathToMap("/meh");
+    values = URLUtils::pathToMap("/meh");
     ASSERT_EQ(values.size(), 1);
     EXPECT_EQ(values[""], "meh");
 }
+
+TEST(ToolsTest, parseTimeTest)
+{
+    std::string value = "01:01:01:01";
+    int result = -1;
+    ASSERT_EQ(parseTime(result, value), true);
+    EXPECT_EQ(result, 90061);
+    EXPECT_EQ(value, "90061");
+
+    value = "01:01:01";
+    result = -1;
+    ASSERT_EQ(parseTime(result, value), true);
+    EXPECT_EQ(result, 3661);
+    EXPECT_EQ(value, "3661");
+
+    value = "51:01";
+    result = -1;
+    ASSERT_EQ(parseTime(result, value), true);
+    EXPECT_EQ(result, 3061);
+    EXPECT_EQ(value, "3061");
+
+    value = "61:01";
+    result = -1;
+    ASSERT_EQ(parseTime(result, value), true);
+    EXPECT_EQ(result, 3661);
+    EXPECT_EQ(value, "3661");
+
+    value = "10";
+    result = -1;
+    ASSERT_EQ(parseTime(result, value), true);
+    EXPECT_EQ(result, 10);
+    EXPECT_EQ(value, "10");
+
+    value = "-10";
+    result = -1;
+    ASSERT_EQ(parseTime(result, value), true);
+    EXPECT_EQ(result, -10);
+    EXPECT_EQ(value, "-10");
+
+    value = "-51:01";
+    result = -1;
+    ASSERT_EQ(parseTime(result, value), false);
+    EXPECT_EQ(result, 0);
+    EXPECT_EQ(value, "-51:01");
+
+    value = "51:x";
+    result = -1;
+    ASSERT_EQ(parseTime(result, value), false);
+    EXPECT_EQ(result, 0);
+    EXPECT_EQ(value, "51:x");
+
+    value = "51:x1";
+    result = -1;
+    ASSERT_EQ(parseTime(result, value), false);
+    EXPECT_EQ(result, 0);
+    EXPECT_EQ(value, "51:x1");
+
+    value = "1x:1x";
+    result = -1;
+    ASSERT_EQ(parseTime(result, value), false);
+    EXPECT_EQ(result, 0);
+    EXPECT_EQ(value, "1x:1x");
+}
