@@ -1,6 +1,28 @@
-#include <gtest/gtest.h>
+/*GRB*
+    Gerbera - https://gerbera.io/
 
-#include "util/upnp_headers.h"
+    test_upnp_headers.cc - this file is part of Gerbera.
+
+    Copyright (C) 2016-2024 Gerbera Contributors
+
+    Gerbera is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License version 2
+    as published by the Free Software Foundation.
+
+    Gerbera is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with Gerbera.  If not, see <http://www.gnu.org/licenses/>.
+
+    $Id$
+*/
+
+#include "upnp/headers.h"
+
+#include <gtest/gtest.h>
 
 class HeadersHelperTest : public ::testing::Test {
 
@@ -265,4 +287,42 @@ TEST_F(HeadersHelperTest, HandlesExtraContentTwo)
     auto actual = Headers::readHeaders(info);
     EXPECT_EQ(actual, expected);
 }
+
+TEST_F(HeadersHelperTest, DoesNotOverwriteWithAdd)
+{
+    // arrange
+    std::string key = "foo";
+    std::string value = "wombat";
+    std::string secondValue = "echidna";
+    std::map<std::string, std::string> expected;
+    expected[key] = value;
+
+    // act
+    subject.addHeader(key, value);
+    subject.addHeader(key, secondValue);
+    subject.writeHeaders(info);
+
+    // assert
+    auto actual = Headers::readHeaders(info);
+    EXPECT_EQ(actual, expected);
+}
+
+TEST_F(HeadersHelperTest, DoesOverwriteWithUpdate)
+{
+    // arrange
+    std::string key = "foo";
+    std::string value = "wombat";
+    std::string secondValue = "echidna";
+    std::map<std::string, std::string> expected;
+    expected[key] = secondValue;
+
+    // act
+    subject.addHeader(key, value);
+    subject.updateHeader(key, secondValue);
+    subject.writeHeaders(info);
+
+    // assert
+    auto actual = Headers::readHeaders(info);
+    EXPECT_EQ(actual, expected);
+}
 #endif
